@@ -32,25 +32,50 @@ argc = len(sys.argv)
 # Supported commands
 valid_commands = [
     'install-apk',
+    'multiwin-replay',
     'replay'
 ]
 
 # Arguments
 # gfxrecon install-apk <file>
 # gfxrecon replay [-p | --push-file <file-on-desktop>] <file-on-device>
+# gfxrecon multiwin-replay [-p | --push-file <file-on-desktop>] <file-on-device>
 
 # Application info
 app_name = 'com.lunarg.gfxreconstruct.replay'
 app_activity = '"com.lunarg.gfxreconstruct.replay/android.app.NativeActivity"'
+multiwin_app_activity = '"com.lunarg.gfxreconstruct.replay/.ReplayActivity"'
 app_action = 'android.intent.action.MAIN'
 app_category = 'android.intent.category.LAUNCHER'
 
 # ADB commands
 adb_install = 'adb install -g -t -r'
 adb_sdk_version = 'adb shell getprop ro.build.version.sdk'
-adb_start = 'adb shell am start -n {} -a {} -c {}'.format(app_activity, app_action, app_category)
 adb_stop = 'adb shell am force-stop {}'.format(app_name)
 adb_push = 'adb push'
+adb_devices = 'adb devices'
+
+# Environment variable for android serial number
+android_serial = 'ANDROID_SERIAL'
+
+class DeviceSelectionException(Exception):
+    pass
+
+def QueryAvailableDevices():
+    result = subprocess.run(shlex.split(adb_devices, posix='win' not in sys.platform), capture_output=True, check=True)
+    devices = result.stdout.decode().strip().splitlines()[1:]
+    return [device.split('\t')[0] for device in devices]
+
+def CheckDeviceSelection():
+    devices = QueryAvailableDevices()
+    if len(devices) <= 1:
+        return
+
+    selection = os.getenv(android_serial)
+    if selection is None or selection == '':
+        raise DeviceSelectionException('Multiple devices detected - you must specify which one to use by setting ANDROID_SERIAL environment variable.')
+    if selection not in devices:
+        raise DeviceSelectionException(f'Selected ({selection}) device not present. Available devices: {devices}')
 
 def CreateCommandParser():
     parser = argparse.ArgumentParser(description='GFXReconstruct utility launcher for Android.')
@@ -61,6 +86,7 @@ def CreateCommandParser():
 def CreateInstallApkParser():
     parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]) + ' install-apk', description='Install the replay tool.')
     parser.add_argument('file', help='APK file to install')
+    parser.add_argument('-s', '--select', metavar='DEVICE_ID', help='Specify the destination device id. Needed if multiple devices are attached.')
     return parser
 
 def CreateReplayParser():
@@ -71,6 +97,7 @@ def CreateReplayParser():
     parser.add_argument('--log-file', metavar='DEVICE_FILE', help='Write log messages to a file at the specified path instead of logcat (forwarded to replay tool)')
     parser.add_argument('--pause-frame', metavar='N', help='Pause after replaying frame number N (forwarded to replay tool)')
     parser.add_argument('--paused', action='store_true', default=False, help='Pause after replaying the first frame (same as "--pause-frame 1"; forwarded to replay tool)')
+    parser.add_argument('--cpu-mask', metavar='binary_mask', help='Set of CPU cores used by the replayer. `binary-mask` is a succession of "0" and "1" that specifies used/unused cores read from left to right. For example "10010" activates the first and fourth cores and deactivate all other cores. If the option is not set, all cores can be used. If the option is set only for some cores, the other cores are not used. (forwarded to replay tool)')
     parser.add_argument('--screenshot-all', action='store_true', default=False, help='Generate screenshots for all frames.  When this option is specified, --screenshots is ignored (forwarded to replay tool)')
     parser.add_argument('--screenshots', metavar='RANGES', help='Generate screenshots for the specified frames.  Target frames are specified as a comma separated list of frame ranges.  A frame range can be specified as a single value, to specify a single frame, or as two hyphenated values, to specify the first and last frames to process.  Frame ranges should be specified in ascending order and cannot overlap.  Note that frame numbering is 1-based (i.e. the first frame is frame 1).  Example: 200,301-305 will generate six screenshots (forwarded to replay tool)')
     parser.add_argument('--screenshot-format', metavar='FORMAT', choices=['bmp', 'png'], help='Image file format to use for screenshot generation.  Available formats are: bmp, png (forwarded to replay tool)')
@@ -100,7 +127,7 @@ def CreateReplayParser():
     parser.add_argument('--vssb', '--virtual-swapchain-skip-blit', action='store_true', default=False, help='Skip blit to real swapchain to gain performance during replay.')
     parser.add_argument('--use-captured-swapchain-indices', action='store_true', default=False, help='Same as "--swapchain captured". Ignored if the "--swapchain" option is used.')
     parser.add_argument('file', nargs='?', help='File on device to play (forwarded to replay tool)')
-    parser.add_argument('--dump-resources', metavar='DUMP_RESOURCES', help='--dump-resources <filename> Extract --dump-resources args from the specified file.')
+    parser.add_argument('--dump-resources', metavar='DUMP_RESOURCES', help='The capture file will be examined, and <submit-index,command-index,draw-call-index> will be converted to <arg> as used in --dump-resources <arg>.  The converted args will be used used as the args for dump resources.')
     parser.add_argument('--dump-resources-before-draw', action='store_true', default=False, help= 'In addition to dumping gpu resources after the Vulkan draw calls specified by the --dump-resources argument, also dump resources before the draw calls.')
     parser.add_argument('--dump-resources-image-format', metavar='FORMAT', choices=['bmp', 'png'], help='Image file format to use when dumping image resources. Available formats are: bmp, png')
     parser.add_argument('--dump-resources-scale', metavar='DR_SCALE', help='tScale images generated by dump resources by the given scale factor. The scale factor must be a floating point number greater than 0. Values greater than 10 are capped at 10. Default value is 1.0.')
@@ -111,8 +138,17 @@ def CreateReplayParser():
     parser.add_argument('--dump-resources-json-output-per-command', action='store_true', default=False, help= 'Enables storing a json output file for each dumped command. Default is disabled.')
     parser.add_argument('--dump-resources-dump-immutable-resources', action='store_true', default=False, help= 'Dump immutable immutable shader resources.')
     parser.add_argument('--dump-resources-dump-all-image-subresources', action='store_true', default=False, help= 'Dump all available mip levels and layers when dumping images.')
+    parser.add_argument('--dump-resources-dump-raw-images', action='store_true', default=False, help= 'Dump images verbatim as raw binary files.')
+    parser.add_argument('--dump-resources-dump-separate-alpha', action='store_true', default=False, help= 'Dump image alpha in a separate image file.')
     parser.add_argument('--pbi-all', action='store_true', default=False, help='Print all block information.')
     parser.add_argument('--pbis', metavar='RANGES', default=False, help='Print block information between block index1 and block index2')
+    parser.add_argument('--pcj', '--pipeline-creation-jobs', action='store_true', default=False, help='Specify the number of pipeline-creation-jobs or background-threads.')
+    parser.add_argument('--save-pipeline-cache', metavar='DEVICE_FILE', help='If set, produces pipeline caches at replay time instead of using the one saved at capture time and save those caches in DEVICE_FILE. (forwarded to replay tool)')
+    parser.add_argument('--load-pipeline-cache', metavar='DEVICE_FILE', help='If set, loads data created by the `--save-pipeline-cache` option in DEVICE_FILE and uses it to create the pipelines instead of the pipeline caches saved at capture time. (forwarded to replay tool)')
+    parser.add_argument('--add-new-pipeline-caches', action='store_true', default=False, help='If set, allows gfxreconstruct to create new vkPipelineCache objects when it encounters a pipeline created without cache. This option can be used in coordination with `--save-pipeline-cache` and `--load-pipeline-cache`. (forwarded to replay tool)')
+    parser.add_argument('--quit-after-frame', metavar='FRAME', help='Specify a frame after which replay will terminate.')
+    parser.add_argument('--screenshot-ignore-FrameBoundaryANDROID', action='store_true', default=False, help='If set, frames switced with vkFrameBoundANDROID will be ignored from the screenshot handler.')
+
     return parser
 
 def MakeExtrasString(args):
@@ -136,6 +172,10 @@ def MakeExtrasString(args):
     if args.paused:
         arg_list.append('--paused')
 
+    if args.cpu_mask:
+        arg_list.append('--cpu-mask')
+        arg_list.append('{}'.format(args.cpu_mask))
+
     if args.screenshot_all:
         arg_list.append('--screenshot-all')
     elif args.screenshots:
@@ -270,6 +310,12 @@ def MakeExtrasString(args):
     if args.dump_resources_dump_all_image_subresources:
         arg_list.append('--dump-resources-dump-all-image-subresources')
 
+    if args.dump_resources_dump_raw_images:
+        arg_list.append('--dump-resources-dump-raw-images')
+
+    if args.dump_resources_dump_separate_alpha:
+        arg_list.append('--dump-resources-dump-separate-alpha')
+
     if args.pbi_all:
         arg_list.append('--pbi-all')
 
@@ -277,6 +323,28 @@ def MakeExtrasString(args):
         arg_list.append('--pbis')
         arg_list.append('{}'.format(args.pbis))
 
+    if args.pcj:
+        arg_list.append('--pcj')
+        arg_list.append('{}'.format(args.pcj))
+
+    if args.save_pipeline_cache:
+        arg_list.append('--save-pipeline-cache')
+        arg_list.append('{}'.format(args.save_pipeline_cache))
+
+    if args.load_pipeline_cache:
+        arg_list.append('--load-pipeline-cache')
+        arg_list.append('{}'.format(args.load_pipeline_cache))
+
+    if args.add_new_pipeline_caches:
+        arg_list.append('--add-new-pipeline-caches')
+
+    if args.quit_after_frame:
+        arg_list.append('--quit-after-frame')
+        arg_list.append('{}'.format(args.quit_after_frame))
+
+    if args.screenshot_ignore_FrameBoundaryANDROID:
+        arg_list.append('--screenshot-ignore-FrameBoundaryANDROID')
+
     if args.file:
         arg_list.append(args.file)
     elif not args.version:
@@ -291,15 +359,20 @@ def InstallApk(install_args):
     sdk = int(subprocess.check_output(shlex.split(adb_sdk_version)).decode())
     force_queryable = ' --force-queryable' if sdk >= 30 else ''
     cmd = adb_install + force_queryable + ' ' + args.file
+
+    CheckDeviceSelection()
+
     print('Executing:', cmd)
     subprocess.check_call(shlex.split(cmd, posix='win' not in sys.platform))
 
-def Replay(replay_args):
+def ReplayCommon(replay_args, activity):
     replay_parser = CreateReplayParser()
     args = replay_parser.parse_args(replay_args)
 
     extras = MakeExtrasString(args)
 
+    CheckDeviceSelection()
+
     if extras:
         if args.push_file:
             cmd = ' '.join([adb_push, args.push_file, args.file])
@@ -309,13 +382,23 @@ def Replay(replay_args):
         print('Executing:', adb_stop)
         subprocess.check_call(shlex.split(adb_stop, posix='win' not in sys.platform))
 
+        adb_start = 'adb shell am start -n {} -a {} -c {}'.format(activity, app_action, app_category)
+
         cmd = ' '.join([adb_start, '--es', '"args"', '"{}"'.format(extras)])
         print('Executing:', cmd)
 
         # Specify posix=False to prevent removal of quotes from adb extras.
         subprocess.check_call(shlex.split(cmd, posix=False))
 
+def Replay(replay_args):
+    ReplayCommon(replay_args, app_activity)
+
+def MultiWinReplay(replay_args):
+    ReplayCommon(replay_args, multiwin_app_activity)
+
 if __name__ == '__main__':
+    devices = QueryAvailableDevices()
+
     command_parser = CreateCommandParser()
     command = command_parser.parse_args()
 
@@ -323,3 +406,5 @@ if __name__ == '__main__':
         InstallApk(command.args)
     elif command.command == 'replay':
         Replay(command.args)
+    elif command.command == 'multiwin-replay':
+        MultiWinReplay(command.args)
