@@ -252,6 +252,7 @@ void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkWriteDescriptorS
                 break;
             case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV:
             case VK_DESCRIPTOR_TYPE_MUTABLE_EXT:
+            case VK_DESCRIPTOR_TYPE_PARTITIONED_ACCELERATION_STRUCTURE_NV:
                 GFXRECON_LOG_WARNING("Descriptor type not supported at " __FILE__ ", line: %d.", __LINE__);
                 break;
             case VK_DESCRIPTOR_TYPE_MAX_ENUM:
@@ -408,9 +409,9 @@ void FieldToJson(nlohmann::ordered_json&                      jdata,
     }
 }
 
-void FieldToJson(nlohmann::ordered_json&                                     jdata,
-                 const Decoded_VkPushDescriptorSetWithTemplateInfoKHR* const pData,
-                 const util::JsonOptions&                                    options)
+void FieldToJson(nlohmann::ordered_json&                                  jdata,
+                 const Decoded_VkPushDescriptorSetWithTemplateInfo* const pData,
+                 const util::JsonOptions&                                 options)
 {
     HandleToJson(jdata["descriptorUpdateTemplate"], pData->descriptorUpdateTemplate, options);
     HandleToJson(jdata["layout"], pData->layout, options);
@@ -418,5 +419,149 @@ void FieldToJson(nlohmann::ordered_json&                                     jda
     FieldToJson(jdata["pData"], &pData->pData, options);
 }
 
+void FieldToJson(nlohmann::ordered_json&                                  jdata,
+                 const Decoded_VkIndirectExecutionSetCreateInfoEXT* const pData,
+                 const util::JsonOptions&                                 options)
+{
+    FieldToJson(jdata["type"], pData->decoded_type, options);
+    switch (pData->decoded_type)
+    {
+        case VK_INDIRECT_EXECUTION_SET_INFO_TYPE_PIPELINES_EXT:
+            FieldToJson(jdata["info"], pData->info->pPipelineInfo, options);
+            break;
+        case VK_INDIRECT_EXECUTION_SET_INFO_TYPE_SHADER_OBJECTS_EXT:
+            FieldToJson(jdata["info"], pData->info->pShaderInfo, options);
+            break;
+        default:
+            break;
+    }
+}
+
+void FieldToJson(nlohmann::ordered_json&                               jdata,
+                 const Decoded_VkIndirectCommandsLayoutTokenEXT* const pData,
+                 const util::JsonOptions&                              options)
+{
+    FieldToJson(jdata["type"], pData->decoded_type, options);
+    switch (pData->decoded_type)
+    {
+        case VK_INDIRECT_COMMANDS_TOKEN_TYPE_PUSH_CONSTANT_EXT:
+        case VK_INDIRECT_COMMANDS_TOKEN_TYPE_SEQUENCE_INDEX_EXT:
+            FieldToJson(jdata["data"], pData->data->pPushConstant, options);
+            break;
+        case VK_INDIRECT_COMMANDS_TOKEN_TYPE_VERTEX_BUFFER_EXT:
+            FieldToJson(jdata["data"], pData->data->pVertexBuffer, options);
+            break;
+        case VK_INDIRECT_COMMANDS_TOKEN_TYPE_INDEX_BUFFER_EXT:
+            FieldToJson(jdata["data"], pData->data->pIndexBuffer, options);
+            break;
+        case VK_INDIRECT_COMMANDS_TOKEN_TYPE_EXECUTION_SET_EXT:
+            FieldToJson(jdata["data"], pData->data->pExecutionSet, options);
+            break;
+        default:
+            break;
+    }
+    FieldToJson(jdata["offset"], pData->offset, options);
+}
+
+void FieldToJson(nlohmann::ordered_json& jdata, const format::DeviceMemoryType& data, const util::JsonOptions& options)
+{
+    FieldToJson(decode::VkMemoryPropertyFlags_t(), jdata["property_flags"], data.property_flags, options);
+    FieldToJson(jdata["heap_index"], data.heap_index, options);
+}
+
+void FieldToJson(nlohmann::ordered_json& jdata, const format::DeviceMemoryHeap& data, const util::JsonOptions& options)
+{
+    FieldToJson(jdata["size"], data.size, options);
+    FieldToJson(decode::VkMemoryHeapFlags_t(), jdata["flags"], data.flags, options);
+}
+
+void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkCopyMemoryToImageInfo* data, const JsonOptions& options)
+{
+    if (data && data->decoded_value)
+    {
+        const VkCopyMemoryToImageInfo&         decoded_value = *data->decoded_value;
+        const Decoded_VkCopyMemoryToImageInfo& meta_struct   = *data;
+
+        FieldToJson(jdata["sType"], decoded_value.sType, options);
+        FieldToJson(VkHostImageCopyFlags_t(), jdata["flags"], decoded_value.flags, options);
+        HandleToJson(jdata["dstImage"], meta_struct.dstImage, options);
+        FieldToJson(jdata["dstImageLayout"], decoded_value.dstImageLayout, options);
+        FieldToJson(jdata["regionCount"], decoded_value.regionCount, options);
+        FieldToJson(jdata["pRegions"], meta_struct.pRegions, options);
+        FieldToJson(jdata["pNext"], meta_struct.pNext, options);
+    }
+}
+
+void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkMemoryToImageCopy* data, const JsonOptions& options)
+{
+    if (data && data->decoded_value)
+    {
+        const VkMemoryToImageCopy&         decoded_value = *data->decoded_value;
+        const Decoded_VkMemoryToImageCopy& meta_struct   = *data;
+
+        FieldToJson(jdata["sType"], decoded_value.sType, options);
+
+        static thread_local uint64_t VkMemoryToImageCopy_pHostPointer_counter = 0;
+        const bool                   written                                  = RepresentBinaryFile(options,
+                                                 jdata["pHostPointer"],
+                                                 "memorytoimagecopy",
+                                                 VkMemoryToImageCopy_pHostPointer_counter,
+                                                 meta_struct.pHostPointer.GetLength(),
+                                                 meta_struct.pHostPointer.GetPointer());
+        VkMemoryToImageCopy_pHostPointer_counter += written;
+
+        FieldToJson(jdata["memoryRowLength"], decoded_value.memoryRowLength, options);
+        FieldToJson(jdata["memoryImageHeight"], decoded_value.memoryImageHeight, options);
+        FieldToJson(jdata["imageSubresource"], meta_struct.imageSubresource, options);
+        FieldToJson(jdata["imageOffset"], meta_struct.imageOffset, options);
+        FieldToJson(jdata["imageExtent"], meta_struct.imageExtent, options);
+        FieldToJson(jdata["pNext"], meta_struct.pNext, options);
+    }
+}
+
+void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkCopyImageToMemoryInfo* data, const JsonOptions& options)
+{
+    if (data && data->decoded_value)
+    {
+        const VkCopyImageToMemoryInfo&         decoded_value = *data->decoded_value;
+        const Decoded_VkCopyImageToMemoryInfo& meta_struct   = *data;
+
+        FieldToJson(jdata["sType"], decoded_value.sType, options);
+        FieldToJson(VkHostImageCopyFlags_t(), jdata["flags"], decoded_value.flags, options);
+        HandleToJson(jdata["srcImage"], meta_struct.srcImage, options);
+        FieldToJson(jdata["srcImageLayout"], decoded_value.srcImageLayout, options);
+        FieldToJson(jdata["regionCount"], decoded_value.regionCount, options);
+        FieldToJson(jdata["pRegions"], meta_struct.pRegions, options);
+        FieldToJson(jdata["pNext"], meta_struct.pNext, options);
+    }
+}
+
+void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkImageToMemoryCopy* data, const JsonOptions& options)
+{
+    if (data && data->decoded_value)
+    {
+        const VkImageToMemoryCopy&         decoded_value = *data->decoded_value;
+        const Decoded_VkImageToMemoryCopy& meta_struct   = *data;
+
+        FieldToJson(jdata["sType"], decoded_value.sType, options);
+
+        static thread_local uint64_t VkImageToMemoryCopy_pHostPointer_counter = 0;
+        const bool                   written                                  = RepresentBinaryFile(options,
+                                                 jdata["pHostPointer"],
+                                                 "imagetomemorycopy",
+                                                 VkImageToMemoryCopy_pHostPointer_counter,
+                                                 meta_struct.pHostPointer.GetLength(),
+                                                 meta_struct.pHostPointer.GetPointer());
+        VkImageToMemoryCopy_pHostPointer_counter += written;
+
+        FieldToJson(jdata["memoryRowLength"], decoded_value.memoryRowLength, options);
+        FieldToJson(jdata["memoryImageHeight"], decoded_value.memoryImageHeight, options);
+        FieldToJson(jdata["imageSubresource"], meta_struct.imageSubresource, options);
+        FieldToJson(jdata["imageOffset"], meta_struct.imageOffset, options);
+        FieldToJson(jdata["imageExtent"], meta_struct.imageExtent, options);
+        FieldToJson(jdata["pNext"], meta_struct.pNext, options);
+    }
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
