@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 2021-2022 LunarG, Inc.
-** Copyright (c) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+** Copyright (c) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -42,6 +42,10 @@
 #include "graphics/dx12_util.h"
 #include "application/application.h"
 
+#ifdef GFXRECON_AGS_SUPPORT
+#include "graphics/dx12_ags_marker_injector.h"
+#endif
+
 #include <functional>
 #include <unordered_map>
 #include <unordered_set>
@@ -64,6 +68,10 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
         gfxrecon::util::filepath::CheckReplayerName(info_record.AppName);
     }
 
+#ifdef GFXRECON_AGS_SUPPORT
+    void SetAgsMarkerInjector(AGSContext* ags_context = nullptr);
+#endif
+
     void SetFatalErrorHandler(std::function<void(const char*)> handler) { fatal_error_handler_ = handler; }
 
     void SetFpsInfo(graphics::FpsInfo* fps_info) { fps_info_ = fps_info; }
@@ -105,6 +113,9 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
         std::vector<format::InitDx12AccelerationStructureGeometryDesc>& geometry_descs,
         const uint8_t*                                                  build_inputs_data) override;
 
+    virtual void ProcessInitializeMetaCommand(const format::InitializeMetaCommand& command_header,
+                                              const uint8_t*                       parameters_data) override;
+
     virtual void Process_ID3D12Device_CheckFeatureSupport(format::HandleId object_id,
                                                           HRESULT          original_result,
                                                           D3D12_FEATURE    feature,
@@ -136,6 +147,46 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
         }
     }
 
+    // This method is used by custom commands for IDXGIAdapter::GetDesc, IDXGIAdapter1::GetDesc1,
+    // IDXGIAdapter2::GetDesc2, and IDXGIAdapter4::GetDesc3.
+    template <class T>
+    void PostCall_IDXGIAdapter_GetDesc(const ApiCallInfo&       call_info,
+                                       DxObjectInfo*            object_info,
+                                       HRESULT                  capture_return_value,
+                                       HRESULT                  replay_return_value,
+                                       StructPointerDecoder<T>* desc)
+    {
+        GFXRECON_UNREFERENCED_PARAMETER(call_info);
+        GFXRECON_UNREFERENCED_PARAMETER(object_info);
+
+        if (SUCCEEDED(capture_return_value) && SUCCEEDED(replay_return_value))
+        {
+            auto capture_desc = desc->GetPointer();
+            auto replay_desc  = desc->GetOutputPointer();
+
+            GFXRECON_ASSERT((capture_desc != nullptr) && (replay_desc != nullptr));
+
+            AddAdapterLuid(capture_desc->AdapterLuid, replay_desc->AdapterLuid);
+        }
+    }
+
+    void PostCall_IDXGIFactory2_GetSharedResourceAdapterLuid(const ApiCallInfo&                  call_info,
+                                                             DxObjectInfo*                       object_info,
+                                                             HRESULT                             capture_return_value,
+                                                             HRESULT                             replay_return_value,
+                                                             uint64_t                            hResource,
+                                                             StructPointerDecoder<Decoded_LUID>* pLuid);
+
+    void PostCall_ID3D12Device_GetAdapterLuid(const ApiCallInfo&  call_info,
+                                              DxObjectInfo*       object_info,
+                                              const Decoded_LUID& capture_return_value,
+                                              const LUID&         replay_return_value);
+
+    void PostCall_ApiCall_ID3D12SwapChainAssistant_GetLUID(const ApiCallInfo&  call_info,
+                                                           DxObjectInfo*       object_info,
+                                                           const Decoded_LUID& capture_return_value,
+                                                           const LUID&         replay_return_value);
+
     void
     PreCall_ID3D12GraphicsCommandList_ResourceBarrier(const ApiCallInfo&                                    call_info,
                                                       DxObjectInfo*                                         object_info,
@@ -154,6 +205,11 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
                                                    StructPointerDecoder<Decoded_D3D12_CONSTANT_BUFFER_VIEW_DESC>* pDesc,
                                                    Decoded_D3D12_CPU_DESCRIPTOR_HANDLE DestDescriptor);
 
+    void PostCall_ID3D12Device_CreateSampler(const ApiCallInfo&                                call_info,
+                                             DxObjectInfo*                                     object_info,
+                                             StructPointerDecoder<Decoded_D3D12_SAMPLER_DESC>* pDesc,
+                                             Decoded_D3D12_CPU_DESCRIPTOR_HANDLE               DestDescriptor);
+
     void
     PostCall_ID3D12Device_CreateShaderResourceView(const ApiCallInfo& call_info,
                                                    DxObjectInfo*      object_info,
@@ -233,10 +289,12 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
 
     void RemoveObject(DxObjectInfo* info);
 
-    void SetDumpTarget(TrackDumpDrawcall& track_dump_target);
+    void SetDumpTarget(TrackDumpDrawCall& track_dump_target);
 
     IDXGIAdapter* GetAdapter();
 
+    graphics::dx12::ActiveAdapterMap& GetAdaptersMap() { return adapters_; }
+
   protected:
     void MapGpuDescriptorHandle(D3D12_GPU_DESCRIPTOR_HANDLE& handle);
 
@@ -244,6 +302,10 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
 
     void MapGpuDescriptorHandles(D3D12_GPU_DESCRIPTOR_HANDLE* handles, size_t handles_len);
 
+    void MapCpuDescriptorHandle(D3D12_CPU_DESCRIPTOR_HANDLE& handle);
+
+    void MapCpuDescriptorHandle(uint8_t* dst_handle_ptr, const uint8_t* src_handle_ptr);
+
     void MapGpuVirtualAddress(D3D12_GPU_VIRTUAL_ADDRESS& address);
 
     void MapGpuVirtualAddress(uint8_t* dst_address_ptr, const uint8_t* src_address_ptr);
@@ -304,6 +366,75 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
         }
     }
 
+    template <typename T>
+    void AddObjects(
+        const format::HandleId* p_ids, size_t ids_len, T** pp_objects, size_t objects_len, format::ApiCallId call_id)
+    {
+        if ((p_ids != nullptr) && (pp_objects != nullptr))
+        {
+            size_t len = objects_len;
+            if (ids_len < objects_len)
+            {
+                len = ids_len;
+
+                // More objects were retrieved at replay than were retrieved at capture. The additional objects do not
+                // have IDs and cannot be added to the object table. Release the objects to avoid leaking.
+                for (auto i = ids_len; i < objects_len; ++i)
+                {
+                    reinterpret_cast<IUnknown*>(pp_objects[i])->Release();
+                }
+            }
+
+            for (size_t i = 0; i < len; ++i)
+            {
+                object_mapping::AddObject(&p_ids[i], &pp_objects[i], &object_info_table_);
+
+                if (options_.override_object_names)
+                {
+                    SetObjectName(&p_ids[i], &pp_objects[i], call_id);
+                }
+            }
+        }
+    }
+
+    template <typename T>
+    void AddObjects(const format::HandleId* p_ids,
+                    size_t                  ids_len,
+                    T**                     pp_objects,
+                    size_t                  objects_len,
+                    std::vector<T>&&        initial_infos,
+                    format::ApiCallId       call_id)
+    {
+        if ((p_ids != nullptr) && (pp_objects != nullptr))
+        {
+            size_t len = objects_len;
+            if (ids_len < objects_len)
+            {
+                len = ids_len;
+
+                // More objects were retrieved at replay than were retrieved at capture. The additional objects do not
+                // have IDs and cannot be added to the object table. Release the objects to avoid leaking.
+                for (auto i = ids_len; i < objects_len; ++i)
+                {
+                    reinterpret_cast<IUnknown*>(pp_objects[i])->Release();
+                }
+            }
+
+            assert(len <= initial_infos.size());
+
+            for (size_t i = 0; i < len; ++i)
+            {
+                auto info_iter = std::next(initial_infos.begin(), i);
+                object_mapping::AddObject(&p_ids[i], &pp_objects[i], std::move(*info_iter), &object_info_table_);
+
+                if (options_.override_object_names)
+                {
+                    SetObjectName(&p_ids[i], &pp_objects[i], call_id);
+                }
+            }
+        }
+    }
+
     void CheckReplayResult(const char* call_name, HRESULT capture_result, HRESULT replay_result);
 
     void* PreProcessExternalObject(uint64_t object_id, format::ApiCallId call_id, const char* call_name);
@@ -355,6 +486,12 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
                                                   DxObjectInfo*                           restrict_to_output_info,
                                                   HandlePointerDecoder<IDXGISwapChain1*>* swapchain);
 
+    HRESULT OverrideEnumAdapterByLuid(DxObjectInfo*                replay_object_info,
+                                      HRESULT                      original_result,
+                                      Decoded_LUID                 adapter_luid,
+                                      Decoded_GUID                 riid,
+                                      HandlePointerDecoder<void*>* adapter);
+
     HRESULT
     OverrideCreateDXGIFactory2(HRESULT                      original_result,
                                UINT                         flags,
@@ -367,6 +504,13 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
                                       Decoded_GUID                 riid,
                                       HandlePointerDecoder<void*>* device);
 
+    HRESULT OverrideD3D12DeviceFactoryCreateDevice(DxObjectInfo*                replay_object_info,
+                                                   HRESULT                      original_result,
+                                                   DxObjectInfo*                adapter_info,
+                                                   D3D_FEATURE_LEVEL            minimum_feature_level,
+                                                   Decoded_GUID                 riid,
+                                                   HandlePointerDecoder<void*>* device);
+
     void ProcessDxgiAdapterInfo(const format::DxgiAdapterInfoCommandHeader& adapter_info_header);
 
     void InitCommandQueueExtraInfo(ID3D12Device* device, HandlePointerDecoder<void*>* command_queue_decoder);
@@ -397,7 +541,7 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
                                  D3D12_RENDER_PASS_FLAGS                                             Flags);
 
     template <typename T>
-    void SetResourceSamplerFeedbackMipRegion(D3D12_RESOURCE_DESC1& desc_dest, T* desc_src){};
+    void SetResourceSamplerFeedbackMipRegion(D3D12_RESOURCE_DESC1& desc_dest, T* desc_src) {};
 
     template <>
     void SetResourceSamplerFeedbackMipRegion(D3D12_RESOURCE_DESC1& desc_dest, D3D12_RESOURCE_DESC1* desc_src)
@@ -734,6 +878,12 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
                                                Decoded_GUID                                                     riid,
                                                HandlePointerDecoder<void*>* pipelineState);
 
+    HRESULT OverrideCreatePipelineState(DxObjectInfo* device_object_info,
+                                        HRESULT       original_result,
+                                        StructPointerDecoder<Decoded_D3D12_PIPELINE_STATE_STREAM_DESC>* pDesc,
+                                        Decoded_GUID                                                    riid,
+                                        HandlePointerDecoder<void*>* ppPipelineState);
+
     HRESULT OverrideSetFullscreenState(DxObjectInfo* swapchain_info,
                                        HRESULT       original_result,
                                        BOOL          Fullscreen,
@@ -845,6 +995,39 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
 
     void OverrideExecuteBundle(DxObjectInfo* replay_object_info, DxObjectInfo* command_list_object_info);
 
+    HRESULT OverrideD3D12CreateVersionedRootSignatureDeserializerFromSubobjectInLibrary(
+        HRESULT                          return_value,
+        PointerDecoder<uint8_t>*         pSrcData,
+        SIZE_T                           SrcDataSizeInBytes,
+        WStringDecoder*                  RootSignatureSubobjectName,
+        Decoded_GUID                     pRootSignatureDeserializerInterface,
+        PointerDecoder<uint64_t, void*>* ppRootSignatureDeserializer);
+
+    void OverrideSetProgram(DxObjectInfo*                                         replay_object_info,
+                            StructPointerDecoder<Decoded_D3D12_SET_PROGRAM_DESC>* pDesc);
+
+    void OverrideDispatchGraph(DxObjectInfo*                                            replay_object_info,
+                               StructPointerDecoder<Decoded_D3D12_DISPATCH_GRAPH_DESC>* pDesc);
+
+    HRESULT OverrideCreateMetaCommand(DxObjectInfo*                device5_object_info,
+                                      HRESULT                      original_result,
+                                      Decoded_GUID                 command_Id,
+                                      UINT                         node_mask,
+                                      PointerDecoder<uint8_t>*     parameters_data,
+                                      SIZE_T                       parameters_data_sizeinbytes,
+                                      Decoded_GUID                 riid,
+                                      HandlePointerDecoder<void*>* meta_command);
+
+    void OverrideInitializeMetaCommand(DxObjectInfo*            command_list4_object_info,
+                                       DxObjectInfo*            meta_command,
+                                       PointerDecoder<uint8_t>* parameters_data,
+                                       SIZE_T                   parameters_data_sizeinbytes);
+
+    void OverrideExecuteMetaCommand(DxObjectInfo*            command_list4_object_info,
+                                    DxObjectInfo*            meta_command,
+                                    PointerDecoder<uint8_t>* parameters_data,
+                                    SIZE_T                   parameters_data_sizeinbytes);
+
     const Dx12ObjectInfoTable& GetObjectInfoTable() const { return object_info_table_; }
 
     Dx12ObjectInfoTable& GetObjectInfoTable() { return object_info_table_; }
@@ -888,6 +1071,59 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
 
     Dx12ResourceValueMapper* GetResourceValueMapper() { return resource_value_mapper_.get(); }
 
+    template <typename CountT>
+    void SetOutputArrayCount(format::HandleId object_id, VariableLengthArrayIndices index, CountT count)
+    {
+        auto info = GetObjectInfo(object_id);
+        if (info != nullptr)
+        {
+            info->array_counts[index] = static_cast<size_t>(count);
+        }
+    }
+
+    template <typename CountT, typename ArrayT>
+    CountT GetOutputArrayCount(const char*                   func_name,
+                               HRESULT                       original_result,
+                               format::HandleId              object_id,
+                               VariableLengthArrayIndices    index,
+                               const PointerDecoder<CountT>* original_count,
+                               const ArrayT*                 original_array)
+    {
+        assert((original_count != nullptr) && (original_array != nullptr));
+
+        CountT replay_count = 0;
+
+        if (!original_count->IsNull())
+        {
+            // Start with array count set equal to the capture count and then adjust if the replay count is different.
+            replay_count = (*original_count->GetPointer());
+
+            // When the array parameter is not null, adjust the count using the value stored by the previous call with a
+            // null array parameter. But only adjust the replay array count if the call succeeded on capture so that
+            // errors generated at capture continue to be generated at replay.
+            if (!original_array->IsNull() && (original_result == S_OK))
+            {
+                auto info = GetObjectInfo(object_id);
+                if (info != nullptr)
+                {
+                    auto entry = info->array_counts.find(index);
+                    if ((entry != info->array_counts.end()) && (entry->second != replay_count))
+                    {
+                        GFXRECON_LOG_INFO("Replay adjusted the %s array count: capture count = %" PRIuPTR
+                                          ", replay count = %" PRIuPTR,
+                                          func_name,
+                                          static_cast<size_t>(replay_count),
+                                          entry->second);
+                        replay_count = static_cast<CountT>(entry->second);
+                    }
+                }
+            }
+        }
+
+        return replay_count;
+    }
+
+  protected:
     DxReplayOptions                    options_;
     std::unique_ptr<Dx12DumpResources> dump_resources_{ nullptr };
 
@@ -925,8 +1161,16 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
         }
     };
 
+    IUnknown* GetCreateDeviceAdapter(DxObjectInfo* adapter_info);
+
+    void InitializeD3D12Device(HandlePointerDecoder<void*>* device);
+
     void DetectAdapters();
 
+    void AddAdapterLuid(const LUID& capture_luid, const LUID& replay_luid);
+
+    LUID GetAdapterLuid(const LUID& capture_luid);
+
     void RaiseFatalError(const char* message) const;
 
     HRESULT
@@ -1010,6 +1254,12 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
 
     std::wstring ConstructObjectName(format::HandleId capture_id, format::ApiCallId call_id);
 
+    void MapMetaCommandParameters(ID3D12Device5*                     device5,
+                                  const GUID&                        meta_command_guid,
+                                  D3D12_META_COMMAND_PARAMETER_STAGE stage,
+                                  uint8_t*                           parameters_data,
+                                  uint8_t                            parameters_data_sizeinbytes);
+
     std::unique_ptr<graphics::DX12ImageRenderer>          frame_buffer_renderer_;
     Dx12ObjectInfoTable                                   object_info_table_;
     std::shared_ptr<application::Application>             application_;
@@ -1018,6 +1268,7 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
     std::unordered_map<uint64_t, MappedMemoryEntry>       mapped_memory_;
     std::unordered_map<uint64_t, void*>                   heap_allocations_;
     std::unordered_map<uint64_t, HANDLE>                  event_objects_;
+    std::unordered_map<uint64_t, LUID>                    adapter_luid_map_;
     std::function<void(const char*)>                      fatal_error_handler_;
     Dx12DescriptorMap                                     descriptor_map_;
     graphics::Dx12GpuVaMap                                gpu_va_map_;
@@ -1042,6 +1293,11 @@ class Dx12ReplayConsumerBase : public Dx12Consumer
     std::unique_ptr<ScreenshotHandlerBase>                screenshot_handler_;
     std::unordered_map<ID3D12Resource*, ResourceInitInfo> resource_init_infos_;
     uint64_t                                              frame_end_marker_count_;
+    std::unordered_map<ID3D12MetaCommand*, GUID>          meta_command_guids_;
+
+#ifdef GFXRECON_AGS_SUPPORT
+    graphics::Dx12AgsMarkerInjector* ags_marker_injector_{ nullptr };
+#endif
 };
 
 GFXRECON_END_NAMESPACE(decode)
