@@ -1,7 +1,7 @@
 /*
 ** Copyright (c) 2018-2020,2022 Valve Corporation
 ** Copyright (c) 2018-2020,2022 LunarG, Inc.
-** Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+** Copyright (c) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -25,13 +25,18 @@
 #include "decode/file_processor.h"
 
 #include "decode/decode_allocator.h"
+#include "format/format.h"
 #include "format/format_util.h"
 #include "util/compressor.h"
+#include "util/file_path.h"
 #include "util/logging.h"
 #include "util/platform.h"
 
 #include <cassert>
+#include <cstdint>
+#include <memory>
 #include <numeric>
+#include <string>
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
@@ -40,9 +45,9 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 const uint32_t kFirstFrame = 0;
 
 FileProcessor::FileProcessor() :
-    file_header_{}, file_descriptor_(nullptr), current_frame_number_(kFirstFrame), bytes_read_(0),
-    error_state_(kErrorInvalidFileDescriptor), annotation_handler_(nullptr), compressor_(nullptr), block_index_(0),
-    api_call_index_(0), block_limit_(0), capture_uses_frame_markers_(false), first_frame_(kFirstFrame + 1)
+    current_frame_number_(kFirstFrame), error_state_(kErrorInvalidFileDescriptor), bytes_read_(0),
+    annotation_handler_(nullptr), compressor_(nullptr), block_index_(0), api_call_index_(0), block_limit_(0),
+    capture_uses_frame_markers_(false), first_frame_(kFirstFrame + 1), loading_trimmed_capture_state_(false)
 {}
 
 FileProcessor::FileProcessor(uint64_t block_limit) : FileProcessor()
@@ -58,9 +63,9 @@ FileProcessor::~FileProcessor()
         compressor_ = nullptr;
     }
 
-    if (file_descriptor_)
+    for (auto& file : active_files_)
     {
-        fclose(file_descriptor_);
+        util::platform::FileClose(file.second.fd);
     }
 
     DecodeAllocator::DestroyInstance();
@@ -76,24 +81,12 @@ void FileProcessor::WaitDecodersIdle()
 
 bool FileProcessor::Initialize(const std::string& filename)
 {
-    bool success = false;
+    bool success = OpenFile(filename);
 
-    int32_t result = util::platform::FileOpen(&file_descriptor_, filename.c_str(), "rb");
-
-    if ((result == 0) && (file_descriptor_ != nullptr))
+    if (success)
     {
-        success = ProcessFileHeader();
-
-        if (success)
-        {
-            filename_    = filename;
-            error_state_ = kErrorNone;
-        }
-        else
-        {
-            fclose(file_descriptor_);
-            file_descriptor_ = nullptr;
-        }
+        success = SetActiveFile(filename, true);
+        success = success && ProcessFileHeader();
     }
     else
     {
@@ -101,9 +94,47 @@ bool FileProcessor::Initialize(const std::string& filename)
         error_state_ = kErrorOpeningFile;
     }
 
+    // Find absolute path of capture file
+    if (success)
+    {
+        absolute_path_ = util::filepath::GetBasedir(filename);
+    }
+
     return success;
 }
 
+std::string FileProcessor::ApplyAbsolutePath(const std::string& file)
+{
+    if (absolute_path_.empty())
+    {
+        return file;
+    }
+
+    return absolute_path_ + file;
+}
+
+bool FileProcessor::OpenFile(const std::string& filename)
+{
+    if (active_files_.find(filename) == active_files_.end())
+    {
+        FILE* fd;
+        int   result = util::platform::FileOpen(&fd, filename.c_str(), "rb");
+        if (result || fd == nullptr)
+        {
+            GFXRECON_LOG_ERROR("Failed to open file %s", filename.c_str());
+            error_state_ = kErrorOpeningFile;
+            return false;
+        }
+        else
+        {
+            active_files_.emplace(std::piecewise_construct, std::forward_as_tuple(filename), std::forward_as_tuple(fd));
+            error_state_ = kErrorNone;
+        }
+    }
+
+    return true;
+}
+
 bool FileProcessor::ProcessNextFrame()
 {
     bool success = IsFileValid();
@@ -115,11 +146,11 @@ bool FileProcessor::ProcessNextFrame()
     else
     {
         // If not EOF, determine reason for invalid state.
-        if (file_descriptor_ == nullptr)
+        if (GetFileDescriptor() == nullptr)
         {
             error_state_ = kErrorInvalidFileDescriptor;
         }
-        else if (ferror(file_descriptor_))
+        else if (ferror(GetFileDescriptor()))
         {
             error_state_ = kErrorReadingFile;
         }
@@ -180,17 +211,20 @@ bool FileProcessor::ContinueDecoding()
 
 bool FileProcessor::ProcessFileHeader()
 {
-    bool success = false;
+    bool               success = false;
+    format::FileHeader file_header{};
 
-    if (ReadBytes(&file_header_, sizeof(file_header_)))
+    ActiveFiles& active_file = active_files_[file_stack_.front().filename];
+
+    if (ReadBytes(&file_header, sizeof(file_header)))
     {
-        success = format::ValidateFileHeader(file_header_);
+        success = format::ValidateFileHeader(file_header);
 
         if (success)
         {
-            file_options_.resize(file_header_.num_options);
+            file_options_.resize(file_header.num_options);
 
-            size_t option_data_size = file_header_.num_options * sizeof(format::FileOptionPair);
+            size_t option_data_size = file_header.num_options * sizeof(format::FileOptionPair);
 
             success = ReadBytes(file_options_.data(), option_data_size);
 
@@ -236,6 +270,25 @@ bool FileProcessor::ProcessFileHeader()
     return success;
 }
 
+void FileProcessor::DecrementRemainingCommands()
+{
+    if (file_stack_.empty())
+    {
+        return;
+    }
+
+    ActiveFileContext& current_file = GetCurrentFile();
+
+    if (!current_file.execute_till_eof)
+    {
+        --current_file.remaining_commands;
+        if (current_file.remaining_commands == 0)
+        {
+            file_stack_.pop_back();
+        }
+    }
+}
+
 bool FileProcessor::ProcessBlocks()
 {
     format::BlockHeader block_header;
@@ -243,12 +296,7 @@ bool FileProcessor::ProcessBlocks()
 
     while (success)
     {
-        if (enable_print_block_info_ && ((block_index_from_ < 0 || block_index_to_ < 0) ||
-                                         (block_index_from_ <= block_index_ && block_index_to_ >= block_index_)))
-        {
-            GFXRECON_LOG_INFO(
-                "block info: index: %" PRIu64 ", current frame: %" PRIu64 "", block_index_, current_frame_number_);
-        }
+        PrintBlockInfo();
         success = ContinueDecoding();
 
         if (success)
@@ -270,14 +318,11 @@ bool FileProcessor::ProcessBlocks()
 
                     if (success)
                     {
-                        success = ProcessFunctionCall(block_header, api_call_id);
+                        bool should_break = false;
+                        success           = ProcessFunctionCall(block_header, api_call_id, should_break);
 
-                        // Break from loop on frame delimiter.
-                        if (IsFrameDelimiter(api_call_id))
+                        if (should_break)
                         {
-                            // Make sure to increment the frame number on the way out.
-                            ++current_frame_number_;
-                            ++block_index_;
                             break;
                         }
                     }
@@ -294,14 +339,11 @@ bool FileProcessor::ProcessBlocks()
 
                     if (success)
                     {
-                        success = ProcessMethodCall(block_header, api_call_id);
+                        bool should_break = false;
+                        success           = ProcessMethodCall(block_header, api_call_id, should_break);
 
-                        // Break from loop on frame delimiter.
-                        if (IsFrameDelimiter(api_call_id))
+                        if (should_break)
                         {
-                            // Make sure to increment the frame number on the way out.
-                            ++current_frame_number_;
-                            ++block_index_;
                             break;
                         }
                     }
@@ -335,24 +377,11 @@ bool FileProcessor::ProcessBlocks()
 
                     if (success)
                     {
-                        success = ProcessFrameMarker(block_header, marker_type);
+                        bool should_break = false;
+                        success           = ProcessFrameMarker(block_header, marker_type, should_break);
 
-                        // Break from loop on frame delimiter.
-                        if (IsFrameDelimiter(block_header.type, marker_type))
+                        if (should_break)
                         {
-                            // If the capture file contains frame markers, it will have a frame marker for every
-                            // frame-ending API call such as vkQueuePresentKHR. If this is the first frame marker
-                            // encountered, reset the frame count and ignore frame-ending API calls in
-                            // IsFrameDelimiter(format::ApiCallId call_id).
-                            if (!capture_uses_frame_markers_)
-                            {
-                                capture_uses_frame_markers_ = true;
-                                current_frame_number_       = kFirstFrame;
-                            }
-
-                            // Make sure to increment the frame number on the way out.
-                            ++current_frame_number_;
-                            ++block_index_;
                             break;
                         }
                     }
@@ -415,7 +444,7 @@ bool FileProcessor::ProcessBlocks()
             }
             else
             {
-                if (!feof(file_descriptor_))
+                if (!feof(GetFileDescriptor()))
                 {
                     // No data has been read for the current block, so we don't use 'HandleBlockReadError' here, as it
                     // assumes that the block header has been successfully read and will print an incomplete block at
@@ -427,11 +456,24 @@ bool FileProcessor::ProcessBlocks()
                                        block_index_);
                     error_state_ = kErrorReadingBlockHeader;
                 }
+                else
+                {
+                    assert(!file_stack_.empty());
+
+                    ActiveFileContext& current_file = GetCurrentFile();
+                    if (current_file.execute_till_eof)
+                    {
+                        file_stack_.pop_back();
+                        success = !file_stack_.empty();
+                    }
+                }
             }
         }
         ++block_index_;
+        DecrementRemainingCommands();
     }
 
+    DecrementRemainingCommands();
     return success;
 }
 
@@ -491,14 +533,23 @@ bool FileProcessor::ReadCompressedParameterBuffer(size_t  compressed_buffer_size
 
 bool FileProcessor::ReadBytes(void* buffer, size_t buffer_size)
 {
-    size_t bytes_read = util::platform::FileRead(buffer, 1, buffer_size, file_descriptor_);
-    bytes_read_ += bytes_read;
-    return (bytes_read == buffer_size);
+    auto file_entry = active_files_.find(file_stack_.back().filename);
+    assert(file_entry != active_files_.end());
+
+    if (util::platform::FileRead(buffer, buffer_size, file_entry->second.fd))
+    {
+        bytes_read_ += buffer_size;
+        return true;
+    }
+    return false;
 }
 
 bool FileProcessor::SkipBytes(size_t skip_size)
 {
-    bool success = util::platform::FileSeek(file_descriptor_, skip_size, util::platform::FileSeekCurrent);
+    auto file_entry = active_files_.find(file_stack_.back().filename);
+    assert(file_entry != active_files_.end());
+
+    bool success = util::platform::FileSeek(file_entry->second.fd, skip_size, util::platform::FileSeekCurrent);
 
     if (success)
     {
@@ -509,10 +560,63 @@ bool FileProcessor::SkipBytes(size_t skip_size)
     return success;
 }
 
+bool FileProcessor::SeekActiveFile(const std::string& filename, int64_t offset, util::platform::FileSeekOrigin origin)
+{
+    auto file_entry = active_files_.find(file_stack_.back().filename);
+    assert(file_entry != active_files_.end());
+
+    bool success = util::platform::FileSeek(file_entry->second.fd, offset, origin);
+
+    if (success && origin == util::platform::FileSeekCurrent)
+    {
+        // These technically count as bytes read/processed.
+        bytes_read_ += offset;
+    }
+
+    return success;
+}
+
+bool FileProcessor::SeekActiveFile(int64_t offset, util::platform::FileSeekOrigin origin)
+{
+    return SeekActiveFile(file_stack_.back().filename, offset, origin);
+}
+
+bool FileProcessor::SetActiveFile(const std::string& filename, bool execute_till_eof)
+{
+    if (active_files_.find(filename) != active_files_.end())
+    {
+        file_stack_.emplace_back(filename, execute_till_eof);
+        return true;
+    }
+    else
+    {
+        return false;
+    }
+}
+
+bool FileProcessor::SetActiveFile(const std::string&             filename,
+                                  int64_t                        offset,
+                                  util::platform::FileSeekOrigin origin,
+                                  bool                           execute_till_eof)
+{
+    if (active_files_.find(filename) != active_files_.end())
+    {
+        file_stack_.emplace_back(filename, execute_till_eof);
+        return SeekActiveFile(filename, offset, origin);
+    }
+    else
+    {
+        return false;
+    }
+}
+
 void FileProcessor::HandleBlockReadError(Error error_code, const char* error_message)
 {
+    auto file_entry = active_files_.find(file_stack_.back().filename);
+    assert(file_entry != active_files_.end());
+
     // Report incomplete block at end of file as a warning, other I/O errors as an error.
-    if (feof(file_descriptor_) && !ferror(file_descriptor_))
+    if (feof(file_entry->second.fd) && !ferror(file_entry->second.fd))
     {
         GFXRECON_LOG_WARNING("Incomplete block at end of file");
     }
@@ -523,7 +627,9 @@ void FileProcessor::HandleBlockReadError(Error error_code, const char* error_mes
     }
 }
 
-bool FileProcessor::ProcessFunctionCall(const format::BlockHeader& block_header, format::ApiCallId call_id)
+bool FileProcessor::ProcessFunctionCall(const format::BlockHeader& block_header,
+                                        format::ApiCallId          call_id,
+                                        bool&                      should_break)
 {
     size_t      parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(call_id);
     uint64_t    uncompressed_size     = 0;
@@ -593,10 +699,20 @@ bool FileProcessor::ProcessFunctionCall(const format::BlockHeader& block_header,
         HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read function call block header");
     }
 
+    // Break from loop on frame delimiter.
+    if (IsFrameDelimiter(call_id))
+    {
+        // Make sure to increment the frame number on the way out.
+        ++current_frame_number_;
+        ++block_index_;
+        should_break = true;
+    }
     return success;
 }
 
-bool FileProcessor::ProcessMethodCall(const format::BlockHeader& block_header, format::ApiCallId call_id)
+bool FileProcessor::ProcessMethodCall(const format::BlockHeader& block_header,
+                                      format::ApiCallId          call_id,
+                                      bool&                      should_break)
 {
     size_t           parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(call_id);
     uint64_t         uncompressed_size     = 0;
@@ -672,6 +788,14 @@ bool FileProcessor::ProcessMethodCall(const format::BlockHeader& block_header, f
         HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read function call block header");
     }
 
+    // Break from loop on frame delimiter.
+    if (IsFrameDelimiter(call_id))
+    {
+        // Make sure to increment the frame number on the way out.
+        ++current_frame_number_;
+        ++block_index_;
+        should_break = true;
+    }
     return success;
 }
 
@@ -974,6 +1098,79 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
                     if (decoder->SupportsMetaDataId(meta_data_id))
                     {
                         decoder->DispatchCreateHardwareBufferCommand(header.thread_id,
+                                                                     0u,
+                                                                     header.memory_id,
+                                                                     header.buffer_id,
+                                                                     header.format,
+                                                                     header.width,
+                                                                     header.height,
+                                                                     header.stride,
+                                                                     header.usage,
+                                                                     header.layers,
+                                                                     entries);
+                    }
+                }
+            }
+            else
+            {
+                if (format::IsBlockCompressed(block_header.type))
+                {
+                    HandleBlockReadError(kErrorReadingCompressedBlockData,
+                                         "Failed to read create hardware buffer meta-data block");
+                }
+                else
+                {
+                    HandleBlockReadError(kErrorReadingBlockData,
+                                         "Failed to read create hardware buffer meta-data block");
+                }
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader,
+                                 "Failed to read create hardware buffer meta-data block header");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kCreateHardwareBufferCommand_deprecated2)
+    {
+        format::CreateHardwareBufferCommandHeader_deprecated2 header;
+
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success && ReadBytes(&header.memory_id, sizeof(header.memory_id));
+        success = success && ReadBytes(&header.buffer_id, sizeof(header.buffer_id));
+        success = success && ReadBytes(&header.format, sizeof(header.format));
+        success = success && ReadBytes(&header.width, sizeof(header.width));
+        success = success && ReadBytes(&header.height, sizeof(header.height));
+        success = success && ReadBytes(&header.stride, sizeof(header.stride));
+        success = success && ReadBytes(&header.usage, sizeof(header.usage));
+        success = success && ReadBytes(&header.layers, sizeof(header.layers));
+        success = success && ReadBytes(&header.planes, sizeof(header.planes));
+
+        if (success)
+        {
+            std::vector<format::HardwareBufferPlaneInfo> entries;
+
+            for (uint64_t i = 0; i < header.planes; ++i)
+            {
+                format::HardwareBufferPlaneInfo entry;
+
+                if (!ReadBytes(&entry, sizeof(entry)))
+                {
+                    success = false;
+                    break;
+                }
+
+                entries.emplace_back(std::move(entry));
+            }
+
+            if (success)
+            {
+                for (auto decoder : decoders_)
+                {
+                    if (decoder->SupportsMetaDataId(meta_data_id))
+                    {
+                        decoder->DispatchCreateHardwareBufferCommand(header.thread_id,
+                                                                     0u,
                                                                      header.memory_id,
                                                                      header.buffer_id,
                                                                      header.format,
@@ -1011,6 +1208,7 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
         format::CreateHardwareBufferCommandHeader header;
 
         success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success && ReadBytes(&header.device_id, sizeof(header.device_id));
         success = success && ReadBytes(&header.memory_id, sizeof(header.memory_id));
         success = success && ReadBytes(&header.buffer_id, sizeof(header.buffer_id));
         success = success && ReadBytes(&header.format, sizeof(header.format));
@@ -1045,6 +1243,7 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
                     if (decoder->SupportsMetaDataId(meta_data_id))
                     {
                         decoder->DispatchCreateHardwareBufferCommand(header.thread_id,
+                                                                     header.device_id,
                                                                      header.memory_id,
                                                                      header.buffer_id,
                                                                      header.format,
@@ -1819,6 +2018,239 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
                                  "Failed to read parent to child dependency meta-data block header");
         }
     }
+    else if (meta_data_type == format::MetaDataType::kSetEnvironmentVariablesCommand)
+    {
+        format::SetEnvironmentVariablesCommand header;
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success && ReadBytes(&header.string_length, sizeof(header.string_length));
+        if (!success)
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read environment variable block header");
+            return success;
+        }
+
+        success = ReadParameterBuffer(static_cast<size_t>(header.string_length));
+        if (!success)
+        {
+            HandleBlockReadError(kErrorReadingBlockData, "Failed to read environment variable block data");
+            return success;
+        }
+
+        const char* env_string = (const char*)parameter_buffer_.data();
+        for (auto decoder : decoders_)
+        {
+            decoder->DispatchSetEnvironmentVariablesCommand(header, env_string);
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kVulkanBuildAccelerationStructuresCommand)
+    {
+        format::VulkanMetaBuildAccelerationStructuresHeader header{};
+        size_t parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(meta_data_id);
+        success                      = ReadParameterBuffer(parameter_buffer_size);
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                if (decoder->SupportsMetaDataId(meta_data_id))
+                {
+                    DecodeAllocator::Begin();
+
+                    decoder->DispatchVulkanAccelerationStructuresBuildMetaCommand(parameter_buffer_.data(),
+                                                                                  parameter_buffer_size);
+
+                    DecodeAllocator::End();
+                }
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader,
+                                 "Failed to read acceleration structure init meta-data block header");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kVulkanCopyAccelerationStructuresCommand)
+    {
+        format::VulkanCopyAccelerationStructuresCommandHeader header{};
+        size_t parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(meta_data_id);
+        success                      = ReadParameterBuffer(parameter_buffer_size);
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                if (decoder->SupportsMetaDataId(meta_data_id))
+                {
+                    DecodeAllocator::Begin();
+
+                    decoder->DispatchVulkanAccelerationStructuresCopyMetaCommand(parameter_buffer_.data(),
+                                                                                 parameter_buffer_size);
+
+                    DecodeAllocator::End();
+                }
+            }
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kVulkanWriteAccelerationStructuresPropertiesCommand)
+    {
+        format::VulkanCopyAccelerationStructuresCommandHeader header{};
+        size_t parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(meta_data_id);
+        success                      = ReadParameterBuffer(parameter_buffer_size);
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                if (decoder->SupportsMetaDataId(meta_data_id))
+                {
+                    DecodeAllocator::Begin();
+
+                    decoder->DispatchVulkanAccelerationStructuresWritePropertiesMetaCommand(parameter_buffer_.data(),
+                                                                                            parameter_buffer_size);
+
+                    DecodeAllocator::End();
+                }
+            }
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kExecuteBlocksFromFile)
+    {
+        format::ExecuteBlocksFromFile exec_from_file;
+        success = ReadBytes(&exec_from_file.thread_id, sizeof(exec_from_file.thread_id));
+        success = success && ReadBytes(&exec_from_file.n_blocks, sizeof(exec_from_file.n_blocks));
+        success = success && ReadBytes(&exec_from_file.offset, sizeof(exec_from_file.offset));
+        success = success && ReadBytes(&exec_from_file.filename_length, sizeof(exec_from_file.filename_length));
+
+        if (success)
+        {
+            std::string filename_c_str(exec_from_file.filename_length, '\0');
+            success = success && ReadBytes(filename_c_str.data(), exec_from_file.filename_length);
+            if (success)
+            {
+                std::string filename = util::filepath::Join(absolute_path_, filename_c_str);
+
+                // Check for self references
+                if (!filename.compare(file_stack_.back().filename))
+                {
+                    GFXRECON_LOG_WARNING(
+                        "ExecuteBlocksFromFile is referencing itself. Probably this is not intentional.");
+                }
+
+                success = OpenFile(filename);
+                if (success)
+                {
+                    for (auto decoder : decoders_)
+                    {
+                        decoder->DispatchExecuteBlocksFromFile(
+                            exec_from_file.thread_id, exec_from_file.n_blocks, exec_from_file.offset, filename);
+                    }
+
+                    SetActiveFile(
+                        filename, exec_from_file.offset, util::platform::FileSeekSet, exec_from_file.n_blocks == 0);
+                    // We need to add 1 because it will be decremented right after this function returns
+                    file_stack_.back().remaining_commands = exec_from_file.n_blocks + 1;
+                }
+            }
+        }
+
+        if (!success)
+        {
+            HandleBlockReadError(kErrorReadingBlockData, "Failed to read runtime info meta-data block");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kViewRelativeLocation)
+    {
+        // This command does not support compression.
+        assert(block_header.type != format::BlockType::kCompressedMetaDataBlock);
+
+        format::ViewRelativeLocation Location;
+        format::ThreadId             thread_id;
+        success = ReadBytes(&thread_id, sizeof(thread_id));
+
+        format::ViewRelativeLocation location;
+        if (success)
+        {
+            success = ReadBytes(&location, sizeof(location));
+        }
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                if (decoder->SupportsMetaDataId(meta_data_id))
+                {
+                    decoder->DispatchViewRelativeLocation(thread_id, location);
+                }
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader, "Failed to ViewRelativeLocation meta-data block");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kInitializeMetaCommand)
+    {
+        format::InitializeMetaCommand header;
+
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success && ReadBytes(&header.capture_id, sizeof(header.capture_id));
+        success = success && ReadBytes(&header.block_index, sizeof(header.block_index));
+        success = success && ReadBytes(&header.total_number_of_initializemetacommand,
+                                       sizeof(header.total_number_of_initializemetacommand));
+        success = success && ReadBytes(&header.initialization_parameters_data_size,
+                                       sizeof(header.initialization_parameters_data_size));
+
+        if (success)
+        {
+            GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, header.initialization_parameters_data_size);
+            if (header.initialization_parameters_data_size > 0)
+            {
+                if (format::IsBlockCompressed(block_header.type))
+                {
+                    size_t uncompressed_size = 0;
+                    size_t compressed_size   = static_cast<size_t>(block_header.size) -
+                                             (sizeof(header) - sizeof(header.meta_header.block_header));
+
+                    success =
+                        ReadCompressedParameterBuffer(compressed_size,
+                                                      static_cast<size_t>(header.initialization_parameters_data_size),
+                                                      &uncompressed_size);
+                }
+                else
+                {
+                    success = ReadParameterBuffer(static_cast<size_t>(header.initialization_parameters_data_size));
+                }
+            }
+            if (success)
+            {
+                for (auto decoder : decoders_)
+                {
+                    if (decoder->SupportsMetaDataId(meta_data_id))
+                    {
+                        decoder->DispatchInitializeMetaCommand(header, parameter_buffer_.data());
+                    }
+                }
+            }
+            else
+            {
+                if (format::IsBlockCompressed(block_header.type))
+                {
+                    HandleBlockReadError(kErrorReadingCompressedBlockData,
+                                         "Failed to read init subresource data meta-data block");
+                }
+                else
+                {
+                    HandleBlockReadError(kErrorReadingBlockData,
+                                         "Failed to read init subresource data meta-data block");
+                }
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader,
+                                 "Failed to read init subresource data meta-data block header");
+        }
+    }
     else
     {
         if ((meta_data_type == format::MetaDataType::kReserved23) ||
@@ -1840,7 +2272,9 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
     return success;
 }
 
-bool FileProcessor::ProcessFrameMarker(const format::BlockHeader& block_header, format::MarkerType marker_type)
+bool FileProcessor::ProcessFrameMarker(const format::BlockHeader& block_header,
+                                       format::MarkerType         marker_type,
+                                       bool&                      should_break)
 {
     // Read the rest of the frame marker data. Currently frame markers are not dispatched to decoders.
     uint64_t frame_number = 0;
@@ -1870,6 +2304,24 @@ bool FileProcessor::ProcessFrameMarker(const format::BlockHeader& block_header,
         HandleBlockReadError(kErrorReadingBlockData, "Failed to read frame marker data");
     }
 
+    // Break from loop on frame delimiter.
+    if (IsFrameDelimiter(block_header.type, marker_type))
+    {
+        // If the capture file contains frame markers, it will have a frame marker for every
+        // frame-ending API call such as vkQueuePresentKHR. If this is the first frame marker
+        // encountered, reset the frame count and ignore frame-ending API calls in
+        // IsFrameDelimiter(format::ApiCallId call_id).
+        if (!capture_uses_frame_markers_)
+        {
+            capture_uses_frame_markers_ = true;
+            current_frame_number_       = kFirstFrame;
+        }
+
+        // Make sure to increment the frame number on the way out.
+        ++current_frame_number_;
+        ++block_index_;
+        should_break = true;
+    }
     return success;
 }
 
@@ -1883,11 +2335,13 @@ bool FileProcessor::ProcessStateMarker(const format::BlockHeader& block_header,
         if (marker_type == format::kBeginMarker)
         {
             GFXRECON_LOG_INFO("Loading state for captured frame %" PRId64, frame_number);
+            loading_trimmed_capture_state_ = true;
         }
         else if (marker_type == format::kEndMarker)
         {
             GFXRECON_LOG_INFO("Finished loading state for captured frame %" PRId64, frame_number);
-            first_frame_ = frame_number;
+            first_frame_                   = frame_number;
+            loading_trimmed_capture_state_ = false;
         }
 
         for (auto decoder : decoders_)
@@ -1987,5 +2441,15 @@ bool FileProcessor::IsFrameDelimiter(format::ApiCallId call_id) const
     }
 }
 
+void FileProcessor::PrintBlockInfo() const
+{
+    if (enable_print_block_info_ && ((block_index_from_ < 0 || block_index_to_ < 0) ||
+                                     (block_index_from_ <= block_index_ && block_index_to_ >= block_index_)))
+    {
+        GFXRECON_LOG_INFO(
+            "block info: index: %" PRIu64 ", current frame: %" PRIu64 "", block_index_, current_frame_number_);
+    }
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
