@@ -25,6 +25,7 @@
 #include "util/logging.h"
 #include "util/platform.h"
 #include "decode/decoder_util.h"
+#include "generated/generated_vulkan_enum_to_string.h"
 
 #include <limits>
 
@@ -41,7 +42,6 @@ inline void WriteImageFile(const std::string&     filename,
                            util::ScreenshotFormat file_format,
                            uint32_t               width,
                            uint32_t               height,
-                           uint64_t               size,
                            void*                  data)
 {
     switch (file_format)
@@ -50,14 +50,14 @@ inline void WriteImageFile(const std::string&     filename,
             GFXRECON_LOG_ERROR("Screenshot format invalid!  Expected BMP or PNG, falling back to BMP.");
             // Intentional fall-through
         case util::ScreenshotFormat::kBmp:
-            if (!util::imagewriter::WriteBmpImage(filename + ".bmp", width, height, size, data))
+            if (!util::imagewriter::WriteBmpImage(filename + ".bmp", width, height, data))
             {
                 GFXRECON_LOG_ERROR("Screenshot could not be created: failed to write BMP file %s", filename.c_str());
             }
             break;
 #ifdef GFXRECON_ENABLE_PNG_SCREENSHOT
         case util::ScreenshotFormat::kPng:
-            if (!util::imagewriter::WritePngImage(filename + ".png", width, height, size, data))
+            if (!util::imagewriter::WritePngImage(filename + ".png", width, height, data))
             {
                 GFXRECON_LOG_ERROR("Screenshot could not be created: failed to write PNG file %s", filename.c_str());
             }
@@ -67,8 +67,8 @@ inline void WriteImageFile(const std::string&     filename,
 }
 
 void ScreenshotHandler::WriteImage(const std::string&                      filename_prefix,
-                                   const DeviceInfo*                       device_info,
-                                   const encode::VulkanDeviceTable*        device_table,
+                                   const VulkanDeviceInfo*                 device_info,
+                                   const graphics::VulkanDeviceTable*      device_table,
                                    const VkPhysicalDeviceMemoryProperties& memory_properties,
                                    VulkanResourceAllocator*                allocator,
                                    VkImage                                 image,
@@ -120,6 +120,10 @@ void ScreenshotHandler::WriteImage(const std::string&                      filen
             auto pair           = copy_resources_.emplace(device, std::move(copy_resource));
             copy_resource_entry = pair.first;
         }
+        else
+        {
+            GFXRECON_LOG_ERROR("Screenshot could not be created: failed to create a command pool")
+        }
     }
 
     if (result == VK_SUCCESS)
@@ -343,6 +347,11 @@ void ScreenshotHandler::WriteImage(const std::string&                      filen
                 // Make sure any pending work is finished, as we are not waiting on any semaphores from previous
                 // submissions.
                 result = device_table->DeviceWaitIdle(device);
+                if (result != VK_SUCCESS)
+                {
+                    GFXRECON_LOG_ERROR("ScreenshotHandler: DeviceWaitIdle failed with %s",
+                                       util::ToString(result).c_str());
+                }
 
                 if (result == VK_SUCCESS)
                 {
@@ -356,11 +365,22 @@ void ScreenshotHandler::WriteImage(const std::string&                      filen
                     submit_info.pSignalSemaphores    = nullptr;
 
                     result = device_table->QueueSubmit(queue, 1, &submit_info, VK_NULL_HANDLE);
+                    if (result != VK_SUCCESS)
+                    {
+                        GFXRECON_LOG_ERROR("ScreenshotHandler: Queue submission %sfailed with %s",
+                                           (copy_resource.convert_image != VK_NULL_HANDLE ? "(blit path) " : ""),
+                                           util::ToString(result).c_str());
+                    }
                 }
 
                 if (result == VK_SUCCESS)
                 {
                     result = device_table->QueueWaitIdle(queue);
+                    if (result != VK_SUCCESS)
+                    {
+                        GFXRECON_LOG_ERROR("ScreenshotHandler: Queue wait failed with %s",
+                                           util::ToString(result).c_str());
+                    }
                 }
 
                 if (result == VK_SUCCESS)
@@ -388,11 +408,14 @@ void ScreenshotHandler::WriteImage(const std::string&                      filen
                                        screenshot_format_,
                                        copy_width,
                                        copy_height,
-                                       copy_resource.buffer_size,
                                        data);
 
                         allocator->UnmapResourceMemoryDirect(copy_resource.buffer_data);
                     }
+                    else
+                    {
+                        GFXRECON_LOG_ERROR("Screenshot could not be created: failed to map resource memory");
+                    }
                 }
                 else
                 {
@@ -409,7 +432,7 @@ void ScreenshotHandler::WriteImage(const std::string&                      filen
     }
 }
 
-void ScreenshotHandler::DestroyDeviceResources(VkDevice device, const encode::VulkanDeviceTable* device_table)
+void ScreenshotHandler::DestroyDeviceResources(VkDevice device, const graphics::VulkanDeviceTable* device_table)
 {
     auto entry = copy_resources_.find(device);
     if (entry != copy_resources_.end())
@@ -475,11 +498,11 @@ VkFormat ScreenshotHandler::GetConversionFormat(VkFormat image_format) const
     return IsSrgbFormat(image_format) ? VK_FORMAT_B8G8R8A8_SRGB : VK_FORMAT_B8G8R8A8_UNORM;
 }
 
-VkDeviceSize ScreenshotHandler::GetCopyBufferSize(VkDevice                         device,
-                                                  const encode::VulkanDeviceTable* device_table,
-                                                  VkFormat                         format,
-                                                  uint32_t                         width,
-                                                  uint32_t                         height) const
+VkDeviceSize ScreenshotHandler::GetCopyBufferSize(VkDevice                           device,
+                                                  const graphics::VulkanDeviceTable* device_table,
+                                                  VkFormat                           format,
+                                                  uint32_t                           width,
+                                                  uint32_t                           height) const
 {
     VkImageCreateInfo create_info     = { VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO };
     create_info.pNext                 = nullptr;
@@ -514,27 +537,8 @@ VkDeviceSize ScreenshotHandler::GetCopyBufferSize(VkDevice
     return memory_requirements.size;
 }
 
-uint32_t ScreenshotHandler::GetMemoryTypeIndex(const VkPhysicalDeviceMemoryProperties& memory_properties,
-                                               uint32_t                                type_bits,
-                                               VkMemoryPropertyFlags                   property_flags) const
-{
-    uint32_t memory_type_index = std::numeric_limits<uint32_t>::max();
-
-    for (uint32_t i = 0; i < memory_properties.memoryTypeCount; ++i)
-    {
-        if ((type_bits & (1 << i)) &&
-            ((memory_properties.memoryTypes[i].propertyFlags & property_flags) == property_flags))
-        {
-            memory_type_index = i;
-            break;
-        }
-    }
-
-    return memory_type_index;
-}
-
 VkResult ScreenshotHandler::CreateCopyResource(VkDevice                                device,
-                                               const encode::VulkanDeviceTable*        device_table,
+                                               const graphics::VulkanDeviceTable*      device_table,
                                                const VkPhysicalDeviceMemoryProperties& memory_properties,
                                                VkDeviceSize                            buffer_size,
                                                VkFormat                                image_format,
@@ -572,17 +576,17 @@ VkResult ScreenshotHandler::CreateCopyResource(VkDevice
         device_table->GetBufferMemoryRequirements(device, copy_resource->buffer, &memory_requirements);
 
         uint32_t memory_type_index =
-            GetMemoryTypeIndex(memory_properties,
-                               memory_requirements.memoryTypeBits,
-                               VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_CACHED_BIT);
+            graphics::GetMemoryTypeIndex(memory_properties,
+                                         memory_requirements.memoryTypeBits,
+                                         VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_CACHED_BIT);
 
         if (memory_type_index == std::numeric_limits<uint32_t>::max())
         {
             /* fallback to coherent */
-            memory_type_index =
-                GetMemoryTypeIndex(memory_properties,
-                                   memory_requirements.memoryTypeBits,
-                                   VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT);
+            memory_type_index = graphics::GetMemoryTypeIndex(memory_properties,
+                                                             memory_requirements.memoryTypeBits,
+                                                             VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT |
+                                                                 VK_MEMORY_PROPERTY_HOST_COHERENT_BIT);
         }
 
         assert(memory_type_index != std::numeric_limits<uint32_t>::max());
@@ -635,9 +639,8 @@ VkResult ScreenshotHandler::CreateCopyResource(VkDevice
             VkMemoryRequirements memory_requirements;
             device_table->GetImageMemoryRequirements(device, copy_resource->convert_image, &memory_requirements);
 
-            uint32_t memory_type_index = GetMemoryTypeIndex(
+            uint32_t memory_type_index = graphics::GetMemoryTypeIndex(
                 memory_properties, memory_requirements.memoryTypeBits, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
-
             assert(memory_type_index != std::numeric_limits<uint32_t>::max());
 
             VkMemoryAllocateInfo allocate_info = { VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO };
