@@ -126,6 +126,7 @@ void VulkanDecoderBase::DispatchResizeWindowCommand2(
 
 void VulkanDecoderBase::DispatchCreateHardwareBufferCommand(
     format::ThreadId                                    thread_id,
+    format::HandleId                                    device_id,
     format::HandleId                                    memory_id,
     uint64_t                                            buffer_id,
     uint32_t                                            format,
@@ -141,7 +142,7 @@ void VulkanDecoderBase::DispatchCreateHardwareBufferCommand(
     for (auto consumer : consumers_)
     {
         consumer->ProcessCreateHardwareBufferCommand(
-            memory_id, buffer_id, format, width, height, stride, usage, layers, plane_info);
+            device_id, memory_id, buffer_id, format, width, height, stride, usage, layers, plane_info);
     }
 }
 
@@ -378,8 +379,8 @@ size_t VulkanDecoderBase::Decode_vkCmdPushDescriptorSetWithTemplate2KHR(const Ap
 {
     size_t bytes_read = 0;
 
-    format::HandleId                                                     commandBuffer;
-    StructPointerDecoder<Decoded_VkPushDescriptorSetWithTemplateInfoKHR> pPushDescriptorSetWithTemplateInfo;
+    format::HandleId                                                  commandBuffer;
+    StructPointerDecoder<Decoded_VkPushDescriptorSetWithTemplateInfo> pPushDescriptorSetWithTemplateInfo;
 
     bytes_read +=
         ValueDecoder::DecodeHandleIdValue((parameter_buffer + bytes_read), (buffer_size - bytes_read), &commandBuffer);
@@ -551,6 +552,15 @@ void VulkanDecoderBase::DispatchSetTlasToBlasDependencyCommand(format::HandleId
     }
 }
 
+void VulkanDecoderBase::DispatchSetEnvironmentVariablesCommand(format::SetEnvironmentVariablesCommand& header,
+                                                               const char*                             env_string)
+{
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessSetEnvironmentVariablesCommand(header, env_string);
+    }
+}
+
 void VulkanDecoderBase::SetCurrentBlockIndex(uint64_t block_index)
 {
     for (auto consumer : consumers_)
@@ -559,5 +569,71 @@ void VulkanDecoderBase::SetCurrentBlockIndex(uint64_t block_index)
     }
 }
 
+void VulkanDecoderBase::DispatchVulkanAccelerationStructuresBuildMetaCommand(const uint8_t* parameter_buffer,
+                                                                             size_t         buffer_size)
+{
+
+    format::HandleId                                                          device_id;
+    StructPointerDecoder<Decoded_VkAccelerationStructureBuildGeometryInfoKHR> pInfos;
+    StructPointerDecoder<Decoded_VkAccelerationStructureBuildRangeInfoKHR*>   ppRangeInfos;
+
+    std::size_t bytes_read = ValueDecoder::DecodeHandleIdValue(parameter_buffer, buffer_size, &device_id);
+    bytes_read += pInfos.Decode(parameter_buffer + bytes_read, buffer_size - bytes_read);
+    ppRangeInfos.Decode(parameter_buffer + bytes_read, buffer_size - bytes_read);
+
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessBuildVulkanAccelerationStructuresMetaCommand(
+            device_id, pInfos.GetLength(), &pInfos, &ppRangeInfos);
+    }
+}
+
+void VulkanDecoderBase::DispatchVulkanAccelerationStructuresCopyMetaCommand(const uint8_t* parameter_buffer,
+                                                                            size_t         buffer_size)
+{
+    format::HandleId                                                 device_id;
+    StructPointerDecoder<Decoded_VkCopyAccelerationStructureInfoKHR> pInfos;
+
+    std::size_t bytes_read = ValueDecoder::DecodeHandleIdValue(parameter_buffer, buffer_size, &device_id);
+    bytes_read += pInfos.Decode(parameter_buffer + bytes_read, buffer_size - bytes_read);
+
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessCopyVulkanAccelerationStructuresMetaCommand(device_id, &pInfos);
+    }
+}
+
+void VulkanDecoderBase::DispatchVulkanAccelerationStructuresWritePropertiesMetaCommand(const uint8_t* parameter_buffer,
+                                                                                       size_t         buffer_size)
+{
+    format::HandleId device_id;
+    VkQueryType      query_type;
+    format::HandleId acceleration_structure_id;
+
+    std::size_t bytes_read = ValueDecoder::DecodeHandleIdValue(parameter_buffer, sizeof(format::HandleId), &device_id);
+    bytes_read += ValueDecoder::DecodeEnumValue(parameter_buffer + bytes_read, sizeof(VkQueryType), &query_type);
+    bytes_read += ValueDecoder::DecodeHandleIdValue(
+        parameter_buffer + bytes_read, sizeof(format::HandleId), &acceleration_structure_id);
+
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessVulkanAccelerationStructuresWritePropertiesMetaCommand(
+            device_id, query_type, acceleration_structure_id);
+    }
+}
+
+void VulkanDecoderBase::DispatchExecuteBlocksFromFile(format::ThreadId   thread_id,
+                                                      uint32_t           n_blocks,
+                                                      int64_t            offset,
+                                                      const std::string& filename)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(thread_id);
+
+    for (auto consumer : consumers_)
+    {
+        consumer->ProcessExecuteBlocksFromFile(n_blocks, offset, filename);
+    }
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
