@@ -30,7 +30,10 @@
 #include "format/format.h"
 #include "generated/generated_vulkan_dispatch_table.h"
 #include "graphics/vulkan_device_util.h"
+#include "graphics/vulkan_instance_util.h"
+#include "graphics/vulkan_shader_group_handle.h"
 #include "util/defines.h"
+#include "util/spirv_parsing_util.h"
 
 #include "vulkan/vulkan.h"
 
@@ -41,6 +44,8 @@
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
+#include <future>
+#include <optional>
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
@@ -59,28 +64,30 @@ enum InstanceArrayIndices : uint32_t
 
 enum PhysicalDeviceArrayIndices : uint32_t
 {
-    kPhysicalDeviceArrayGetPhysicalDeviceQueueFamilyProperties                          = 0,
-    kPhysicalDeviceArrayGetPhysicalDeviceSparseImageFormatProperties                    = 1,
-    kPhysicalDeviceArrayGetPhysicalDeviceQueueFamilyProperties2                         = 2,
-    kPhysicalDeviceArrayGetPhysicalDeviceSparseImageFormatProperties2                   = 3,
-    kPhysicalDeviceArrayGetPhysicalDeviceDisplayPropertiesKHR                           = 4,
-    kPhysicalDeviceArrayGetPhysicalDeviceDisplayPlanePropertiesKHR                      = 5,
-    kPhysicalDeviceArrayGetDisplayPlaneSupportedDisplaysKHR                             = 6,
-    kPhysicalDeviceArrayGetPhysicalDeviceSurfaceFormats2KHR                             = 7,
-    kPhysicalDeviceArrayGetPhysicalDeviceDisplayProperties2KHR                          = 8,
-    kPhysicalDeviceArrayGetPhysicalDeviceDisplayPlaneProperties2KHR                     = 9,
-    kPhysicalDeviceArrayGetPhysicalDeviceCalibrateableTimeDomainsEXT                    = 10,
-    kPhysicalDeviceArrayGetPhysicalDeviceCooperativeMatrixPropertiesNV                  = 11,
-    kPhysicalDeviceArrayGetPhysicalDeviceSupportedFramebufferMixedSamplesCombinationsNV = 12,
-    kPhysicalDeviceArrayGetPhysicalDeviceSurfacePresentModes2EXT                        = 13,
-    kPhysicalDeviceArrayEnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR   = 14,
-    kPhysicalDeviceArrayGetPhysicalDeviceToolProperties                                 = 15,
-    kPhysicalDeviceArrayGetPhysicalDeviceFragmentShadingRatesKHR                        = 16,
-    kFramebufferArrayGetFramebufferTilePropertiesQCOM                                   = 17,
-    kPhysicalDeviceArrayGetPhysicalDeviceOpticalFlowImageFormatsNV                      = 18,
-    kPhysicalDeviceArrayGetPhysicalDeviceVideoFormatPropertiesKHR                       = 19,
-    kVideoSessionKHRArrayGetVideoSessionMemoryRequirementsKHR                           = 20,
-    kShaderEXTArrayGetShaderBinaryDataEXT                                               = 21,
+    kPhysicalDeviceArrayGetPhysicalDeviceQueueFamilyProperties                           = 0,
+    kPhysicalDeviceArrayGetPhysicalDeviceSparseImageFormatProperties                     = 1,
+    kPhysicalDeviceArrayGetPhysicalDeviceQueueFamilyProperties2                          = 2,
+    kPhysicalDeviceArrayGetPhysicalDeviceSparseImageFormatProperties2                    = 3,
+    kPhysicalDeviceArrayGetPhysicalDeviceDisplayPropertiesKHR                            = 4,
+    kPhysicalDeviceArrayGetPhysicalDeviceDisplayPlanePropertiesKHR                       = 5,
+    kPhysicalDeviceArrayGetDisplayPlaneSupportedDisplaysKHR                              = 6,
+    kPhysicalDeviceArrayGetPhysicalDeviceSurfaceFormats2KHR                              = 7,
+    kPhysicalDeviceArrayGetPhysicalDeviceDisplayProperties2KHR                           = 8,
+    kPhysicalDeviceArrayGetPhysicalDeviceDisplayPlaneProperties2KHR                      = 9,
+    kPhysicalDeviceArrayGetPhysicalDeviceCalibrateableTimeDomainsEXT                     = 10,
+    kPhysicalDeviceArrayGetPhysicalDeviceCooperativeMatrixPropertiesNV                   = 11,
+    kPhysicalDeviceArrayGetPhysicalDeviceSupportedFramebufferMixedSamplesCombinationsNV  = 12,
+    kPhysicalDeviceArrayGetPhysicalDeviceSurfacePresentModes2EXT                         = 13,
+    kPhysicalDeviceArrayEnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR    = 14,
+    kPhysicalDeviceArrayGetPhysicalDeviceToolProperties                                  = 15,
+    kPhysicalDeviceArrayGetPhysicalDeviceFragmentShadingRatesKHR                         = 16,
+    kFramebufferArrayGetFramebufferTilePropertiesQCOM                                    = 17,
+    kPhysicalDeviceArrayGetPhysicalDeviceOpticalFlowImageFormatsNV                       = 18,
+    kPhysicalDeviceArrayGetPhysicalDeviceVideoFormatPropertiesKHR                        = 19,
+    kVideoSessionKHRArrayGetVideoSessionMemoryRequirementsKHR                            = 20,
+    kShaderEXTArrayGetShaderBinaryDataEXT                                                = 21,
+    kPhysicalDeviceArrayGetPhysicalDeviceCooperativeMatrixFlexibleDimensionsPropertiesNV = 22,
+    kPhysicalDeviceArrayGetPhysicalDeviceCooperativeVectorPropertiesNV                   = 23,
 
     // Aliases for extensions functions that were promoted to core.
     kPhysicalDeviceArrayGetPhysicalDeviceQueueFamilyProperties2KHR =
@@ -100,6 +107,7 @@ enum DeviceArrayIndices : uint32_t
     kDeviceArrayGetEncodedVideoSessionParametersKHR                     = 5,
     kPhysicalDeviceArrayGetPhysicalDeviceCooperativeMatrixPropertiesKHR = 6,
     kPhysicalDeviceArrayGetPhysicalDeviceCalibrateableTimeDomainsKHR    = 7,
+    kDeviceArrayGetPipelineBinaryDataKHR                                = 8,
 
     // Aliases for extensions functions that were promoted to core.
     kDeviceArrayGetImageSparseMemoryRequirements2KHR      = kDeviceArrayGetImageSparseMemoryRequirements2,
@@ -155,10 +163,14 @@ enum ValidationCacheEXTArrayIndices : uint32_t
 // Structures for storing Vulkan object info.
 //
 
-struct ReplayDeviceInfo
+struct VulkanReplayDeviceInfo
 {
-    std::unique_ptr<VkPhysicalDeviceProperties>       properties;
-    std::unique_ptr<VkPhysicalDeviceMemoryProperties> memory_properties;
+    std::optional<VkPhysicalDeviceProperties>       properties;
+    std::optional<VkPhysicalDeviceMemoryProperties> memory_properties;
+
+    // extensions
+    std::optional<VkPhysicalDeviceRayTracingPipelinePropertiesKHR>    raytracing_properties;
+    std::optional<VkPhysicalDeviceAccelerationStructurePropertiesKHR> acceleration_structure_properties;
 };
 
 template <typename T>
@@ -172,6 +184,21 @@ struct VulkanObjectInfo
     format::HandleId parent_id{ format::kNullHandleId };  // ID of the object's parent instance/device object.
 };
 
+template <typename T>
+struct handle_create_result_t
+{
+    VkResult       result = VK_INCOMPLETE;
+    std::vector<T> handles;
+};
+
+template <typename T>
+struct VulkanObjectInfoAsync : public VulkanObjectInfo<T>
+{
+    // track asynchronous compilation status
+    std::shared_future<handle_create_result_t<T>> future;
+    uint32_t                                      future_handle_index = 0;
+};
+
 // Info for a pool object which other objects will be allocated from.
 template <typename T>
 struct VulkanPoolInfo : public VulkanObjectInfo<T>
@@ -190,32 +217,46 @@ struct VulkanPoolObjectInfo : public VulkanObjectInfo<T>
 // Declarations for Vulkan objects without additional replay state info.
 //
 
-typedef VulkanObjectInfo<VkEvent>                         EventInfo;
-typedef VulkanObjectInfo<VkQueryPool>                     QueryPoolInfo;
-typedef VulkanObjectInfo<VkPipelineLayout>                PipelineLayoutInfo;
-typedef VulkanObjectInfo<VkPrivateDataSlot>               PrivateDataSlotInfo;
-typedef VulkanObjectInfo<VkSampler>                       SamplerInfo;
-typedef VulkanPoolInfo<VkCommandPool>                     CommandPoolInfo;
-typedef VulkanObjectInfo<VkSamplerYcbcrConversion>        SamplerYcbcrConversionInfo;
-typedef VulkanObjectInfo<VkDisplayModeKHR>                DisplayModeKHRInfo;
-typedef VulkanObjectInfo<VkDebugReportCallbackEXT>        DebugReportCallbackEXTInfo;
-typedef VulkanObjectInfo<VkIndirectCommandsLayoutNV>      IndirectCommandsLayoutNVInfo;
-typedef VulkanObjectInfo<VkDebugUtilsMessengerEXT>        DebugUtilsMessengerEXTInfo;
-typedef VulkanObjectInfo<VkAccelerationStructureKHR>      AccelerationStructureKHRInfo;
-typedef VulkanObjectInfo<VkAccelerationStructureNV>       AccelerationStructureNVInfo;
-typedef VulkanObjectInfo<VkPerformanceConfigurationINTEL> PerformanceConfigurationINTELInfo;
-typedef VulkanObjectInfo<VkMicromapEXT>                   MicromapEXTInfo;
-typedef VulkanObjectInfo<VkOpticalFlowSessionNV>          OpticalFlowSessionNVInfo;
-typedef VulkanObjectInfo<VkVideoSessionParametersKHR>     VideoSessionParametersKHRInfo;
+typedef VulkanObjectInfo<VkEvent>                              VulkanEventInfo;
+typedef VulkanObjectInfo<VkQueryPool>                          VulkanQueryPoolInfo;
+typedef VulkanObjectInfo<VkPrivateDataSlot>                    VulkanPrivateDataSlotInfo;
+typedef VulkanObjectInfo<VkSampler>                            VulkanSamplerInfo;
+typedef VulkanPoolInfo<VkCommandPool>                          VulkanCommandPoolInfo;
+typedef VulkanObjectInfo<VkSamplerYcbcrConversion>             VulkanSamplerYcbcrConversionInfo;
+typedef VulkanObjectInfo<VkDisplayModeKHR>                     VulkanDisplayModeKHRInfo;
+typedef VulkanObjectInfo<VkDebugReportCallbackEXT>             VulkanDebugReportCallbackEXTInfo;
+typedef VulkanObjectInfo<VkIndirectCommandsLayoutNV>           VulkanIndirectCommandsLayoutNVInfo;
+typedef VulkanObjectInfo<VkDebugUtilsMessengerEXT>             VulkanDebugUtilsMessengerEXTInfo;
+typedef VulkanObjectInfo<VkAccelerationStructureNV>            VulkanAccelerationStructureNVInfo;
+typedef VulkanObjectInfo<VkPerformanceConfigurationINTEL>      VulkanPerformanceConfigurationINTELInfo;
+typedef VulkanObjectInfo<VkMicromapEXT>                        VulkanMicromapEXTInfo;
+typedef VulkanObjectInfo<VkOpticalFlowSessionNV>               VulkanOpticalFlowSessionNVInfo;
+typedef VulkanObjectInfo<VkVideoSessionParametersKHR>          VulkanVideoSessionParametersKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryKHR>                  VulkanPipelineBinaryKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryCreateInfoKHR>        VulkanPipelineBinaryCreateInfoKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryDataInfoKHR>          VulkanPipelineBinaryDataInfoKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryDataKHR>              VulkanPipelineBinaryDataKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryHandlesInfoKHR>       VulkanPipelineBinaryHandlesInfoKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryKeyKHR>               VulkanPipelineBinaryKeyKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryKeysAndDataKHR>       VulkanPipelineBinaryKeysAndDataKHRInfo;
+typedef VulkanObjectInfo<VkReleaseCapturedPipelineDataInfoKHR> VulkanReleaseCapturedPipelineDataInfoKHRInfo;
+typedef VulkanObjectInfo<VkDevicePipelineBinaryInternalCacheControlKHR>
+                                                  VulkanDevicePipelineBinaryInternalCacheControlKHRInfo;
+typedef VulkanObjectInfo<VkPipelineBinaryInfoKHR> VulkanPipelineBinaryInfoKHRInfo;
+typedef VulkanObjectInfo<VkPhysicalDevicePipelineBinaryFeaturesKHR> VulkanPhysicalDevicePipelineBinaryFeaturesKHRInfo;
+typedef VulkanObjectInfo<VkPhysicalDevicePipelineBinaryPropertiesKHR>
+                                                      VulkanPhysicalDevicePipelineBinaryPropertiesKHRInfo;
+typedef VulkanObjectInfo<VkIndirectCommandsLayoutEXT> VulkanIndirectCommandsLayoutEXTInfo;
+typedef VulkanObjectInfo<VkIndirectExecutionSetEXT>   VulkanIndirectExecutionSetEXTInfo;
+typedef VulkanObjectInfo<VkExternalComputeQueueNV>    VulkanExternalComputeQueueNVInfo;
 
 //
 // Declarations for Vulkan objects with additional replay state info.
 //
 
-struct InstanceInfo : public VulkanObjectInfo<VkInstance>
+struct VulkanInstanceInfo : public VulkanObjectInfo<VkInstance>
 {
-    uint32_t                             api_version{ VK_MAKE_VERSION(1, 0, 0) };
-    std::vector<std::string>             enabled_extensions;
+    graphics::VulkanInstanceUtilInfo     util_info{};
     std::unordered_map<uint32_t, size_t> array_counts;
 
     // Capture and replay devices sorted in the order that they were originally retrieved from
@@ -223,14 +264,15 @@ struct InstanceInfo : public VulkanObjectInfo<VkInstance>
     std::vector<format::HandleId> capture_devices;
     std::vector<VkPhysicalDevice> replay_devices;
 
-    std::unordered_map<VkPhysicalDevice, ReplayDeviceInfo> replay_device_info;
+    std::unordered_map<VkPhysicalDevice, VulkanReplayDeviceInfo> replay_device_info;
 };
 
-struct PhysicalDeviceInfo : public VulkanObjectInfo<VkPhysicalDevice>
+struct VulkanPhysicalDeviceInfo : public VulkanObjectInfo<VkPhysicalDevice>
 {
-    VkInstance                           parent{ VK_NULL_HANDLE };
-    uint32_t                             parent_api_version{ 0 };
-    std::vector<std::string>             parent_enabled_extensions;
+    VkInstance parent{ VK_NULL_HANDLE };
+
+    graphics::VulkanInstanceUtilInfo parent_info{};
+
     std::unordered_map<uint32_t, size_t> array_counts;
 
     // Capture device properties.
@@ -243,11 +285,24 @@ struct PhysicalDeviceInfo : public VulkanObjectInfo<VkPhysicalDevice>
     std::string                      capture_device_name;
     VkPhysicalDeviceMemoryProperties capture_memory_properties{};
 
+    // capture raytracing (shader-binding-table) properties
+    std::optional<VkPhysicalDeviceRayTracingPipelinePropertiesKHR> capture_raytracing_properties = {};
+
     // Closest matching replay device.
-    ReplayDeviceInfo* replay_device_info{ nullptr };
+    VulkanReplayDeviceInfo* replay_device_info{ nullptr };
+
+    // Because Vulkan captures unwrapped handles, and layered APIs (like OpenXR)
+    // capture wrapped handles, during replay two HandleId values will reference
+    // the same VkPhysical device.  The vulkan_alias is the handleId as known by
+    // the vulkan_consumers, which will be created/updated, etc, by all Vulkan replay
+    // calls, s.t. the information known by the vulkan_consumer need not be duplicated.
+    // Operations on this, will use the vulkan_alias as the effective HandleId when set.
+
+    // When Non-null, the GetVkObject will recur on the alias Id
+    format::HandleId vulkan_alias{ format::kNullHandleId };
 };
 
-struct DeviceInfo : public VulkanObjectInfo<VkDevice>
+struct VulkanDeviceInfo : public VulkanObjectInfo<VkDevice>
 {
     VkPhysicalDevice                         parent{ VK_NULL_HANDLE };
     std::unique_ptr<VulkanResourceAllocator> allocator;
@@ -271,14 +326,15 @@ struct DeviceInfo : public VulkanObjectInfo<VkDevice>
     std::vector<VkPhysicalDevice> replay_device_group;
 };
 
-struct QueueInfo : public VulkanObjectInfo<VkQueue>
+struct VulkanQueueInfo : public VulkanObjectInfo<VkQueue>
 {
+    VkDevice                             parent{ VK_NULL_HANDLE };
     std::unordered_map<uint32_t, size_t> array_counts;
     uint32_t                             family_index;
     uint32_t                             queue_index;
 };
 
-struct SemaphoreInfo : public VulkanObjectInfo<VkSemaphore>
+struct VulkanSemaphoreInfo : public VulkanObjectInfo<VkSemaphore>
 {
     bool is_external{ false };
 
@@ -293,23 +349,25 @@ struct SemaphoreInfo : public VulkanObjectInfo<VkSemaphore>
     bool signaled{ false };
 };
 
-struct FenceInfo : public VulkanObjectInfo<VkFence>
+struct VulkanFenceInfo : public VulkanObjectInfo<VkFence>
 {
     // If a null-swapchain/surface interacts with a fence, replay needs to to shadow signal it until a future call waits
     // on it.
     bool shadow_signaled{ false };
 };
 
-struct DeviceMemoryInfo : public VulkanObjectInfo<VkDeviceMemory>
+struct VulkanDeviceMemoryInfo : public VulkanObjectInfo<VkDeviceMemory>
 {
     VulkanResourceAllocator*            allocator{ nullptr };
     VulkanResourceAllocator::MemoryData allocator_data{ 0 };
 };
 
-struct BufferInfo : public VulkanObjectInfo<VkBuffer>
+struct VulkanBufferInfo : public VulkanObjectInfo<VkBuffer>
 {
     // The following values are only used for memory portability.
     VulkanResourceAllocator::ResourceData allocator_data{ 0 };
+    VkDeviceAddress                       capture_address{ 0 };
+    VkDeviceAddress                       replay_address{ 0 };
 
     // This is only used when loading the initial state for trimmed files.
     VkMemoryPropertyFlags memory_property_flags{ 0 };
@@ -319,12 +377,12 @@ struct BufferInfo : public VulkanObjectInfo<VkBuffer>
     uint32_t           queue_family_index{ 0 };
 };
 
-struct BufferViewInfo : public VulkanObjectInfo<VkBufferView>
+struct VulkanBufferViewInfo : public VulkanObjectInfo<VkBufferView>
 {
     format::HandleId buffer_id{ format::kNullHandleId };
 };
 
-struct ImageInfo : public VulkanObjectInfo<VkImage>
+struct VulkanImageInfo : public VulkanObjectInfo<VkImage>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
 
@@ -342,6 +400,8 @@ struct ImageInfo : public VulkanObjectInfo<VkImage>
     VkImageUsageFlags     usage{ 0 };
     VkImageType           type{};
     VkFormat              format{};
+    bool                  external_format{ false };
+    bool                  external_memory_android{ false };
     VkExtent3D            extent{ 0, 0, 0 };
     VkImageTiling         tiling{};
     VkSampleCountFlagBits sample_count{};
@@ -352,68 +412,65 @@ struct ImageInfo : public VulkanObjectInfo<VkImage>
 
     VkImageLayout current_layout{ VK_IMAGE_LAYOUT_UNDEFINED };
     VkImageLayout intermediate_layout{ VK_IMAGE_LAYOUT_UNDEFINED };
+
+    VkDeviceSize size{ 0 };
 };
 
-typedef struct PipelineCacheData
+struct VulkanPipelineCacheData
 {
     std::vector<uint8_t> capture_cache_data;
     std::vector<uint8_t> replay_cache_data;
-} PipelineCacheData;
+};
 
-struct PipelineCacheInfo : public VulkanObjectInfo<VkPipelineCache>
+struct VulkanPipelineCacheInfo : public VulkanObjectInfo<VkPipelineCache>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
+
     // hash id of capture time pipeline cache data to capture and replay time pipeline cache data map;
-    std::unordered_map<uint32_t, std::vector<PipelineCacheData>> pipeline_cache_data;
+    std::unordered_map<uint32_t, std::vector<VulkanPipelineCacheData>> pipeline_cache_data;
+
+    // cache was created using VK_PIPELINE_CACHE_CREATE_EXTERNALLY_SYNCHRONIZED_BIT flag.
+    bool requires_external_synchronization = false;
 };
 
-struct ShaderModuleInfo : public VulkanObjectInfo<VkShaderModule>
+struct VulkanShaderModuleInfo : public VulkanObjectInfo<VkShaderModule>
 {
-    // All information stored in ShaderModuleInfo is populated and used
-    // by the dump resources feature
-    struct ShaderDescriptorInfo
-    {
-        ShaderDescriptorInfo(VkDescriptorType type, bool readonly, uint32_t accessed, uint32_t count, bool is_array) :
-            type(type), readonly(readonly), accessed(accessed), count(count), is_array(is_array)
-        {}
-
-        ShaderDescriptorInfo(const ShaderDescriptorInfo& other) :
-            type(other.type), readonly(other.readonly), accessed(other.accessed), count(other.count),
-            is_array(other.is_array)
-        {}
-
-        VkDescriptorType type;
-        bool             readonly;
-        uint32_t         accessed;
-        uint32_t         count;
-        bool             is_array;
-    };
-
-    ShaderModuleInfo() = default;
-    ShaderModuleInfo(const ShaderModuleInfo& other)
-    {
-        handle                = other.handle;
-        parent_id             = other.parent_id;
-        capture_id            = other.capture_id;
-        used_descriptors_info = other.used_descriptors_info;
-    }
-
-    // One entry per descriptor binding
-    using ShaderDescriptorSetInfo = std::map<uint32_t, ShaderDescriptorInfo>;
-
-    // One entry per descriptor set
-    using ShaderDescriptorSetsInfos = std::map<uint32_t, ShaderDescriptorSetInfo>;
-
-    ShaderDescriptorSetsInfos used_descriptors_info;
+    // keep track of existing usage of buffer-references
+    std::vector<gfxrecon::util::SpirVParsingUtil::BufferReferenceInfo> buffer_reference_infos;
 };
 
-struct PipelineInfo : public VulkanObjectInfo<VkPipeline>
+struct DescriptorBindingLayout
+{
+    uint32_t           count;
+    VkDescriptorType   type;
+    VkShaderStageFlags stage_flags;
+};
+
+using DescriptorBindingLayoutMap = std::unordered_map<uint32_t, DescriptorBindingLayout>;
+
+struct VulkanDescriptorSetLayoutInfo : public VulkanObjectInfo<VkDescriptorSetLayout>
+{
+    // One per descriptor binding
+    DescriptorBindingLayoutMap bindings_layout;
+};
+
+struct VulkanPipelineLayoutInfo : public VulkanObjectInfo<VkPipelineLayout>
+{
+    // One per descriptor set
+    std::vector<DescriptorBindingLayoutMap> desc_set_layouts;
+};
+
+struct VulkanPipelineInfo : public VulkanObjectInfoAsync<VkPipeline>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
 
-    // The following information is populated and used only when the
-    // dump resources feature is in use
-    std::unordered_map<VkShaderStageFlagBits, ShaderModuleInfo> shaders;
+    // keep track of existing usage of buffer-references
+    std::vector<gfxrecon::util::SpirVParsingUtil::BufferReferenceInfo> buffer_reference_infos;
+
+    // map capture- to replay-time shader-group-handles
+    std::unordered_map<graphics::shader_group_handle_t, graphics::shader_group_handle_t> shader_group_handle_map;
+
+    // The following information is populated and used only when the dump resources feature is in use
 
     struct InputBindingDescription
     {
@@ -441,9 +498,15 @@ struct PipelineInfo : public VulkanObjectInfo<VkPipeline>
 
     // Is VK_DYNAMIC_STATE_VERTEX_INPUT_BINDING_STRIDE_EXT enabled
     bool dynamic_vertex_binding_stride{ false };
+
+    // Grahpics pipeline library info
+    VkGraphicsPipelineLibraryFlagsEXT gpl_flags{ 0 };
+
+    // Pipeline layout info
+    std::vector<DescriptorBindingLayoutMap> desc_set_layouts;
 };
 
-struct DescriptorPoolInfo : public VulkanPoolInfo<VkDescriptorPool>
+struct VulkanDescriptorPoolInfo : public VulkanPoolInfo<VkDescriptorPool>
 {
     VkDescriptorPoolCreateFlags       flags{};
     uint32_t                          max_sets{ 0 };
@@ -452,18 +515,18 @@ struct DescriptorPoolInfo : public VulkanPoolInfo<VkDescriptorPool>
     std::vector<VkDescriptorPool>     retired_pools;
 };
 
-struct DescriptorUpdateTemplateInfo : public VulkanObjectInfo<VkDescriptorUpdateTemplate>
+struct VulkanDescriptorUpdateTemplateInfo : public VulkanObjectInfo<VkDescriptorUpdateTemplate>
 {
     std::vector<VkDescriptorType>                descriptor_image_types;
     std::vector<VkDescriptorUpdateTemplateEntry> entries;
 };
 
-struct DisplayKHRInfo : public VulkanObjectInfo<VkDisplayKHR>
+struct VulkanDisplayKHRInfo : public VulkanObjectInfo<VkDisplayKHR>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
 };
 
-struct SurfaceKHRInfo : public VulkanObjectInfo<VkSurfaceKHR>
+struct VulkanSurfaceKHRInfo : public VulkanObjectInfo<VkSurfaceKHR>
 {
     Window*                              window{ nullptr };
     std::unordered_map<uint32_t, size_t> array_counts;
@@ -472,11 +535,11 @@ struct SurfaceKHRInfo : public VulkanObjectInfo<VkSurfaceKHR>
     std::unordered_map<VkPhysicalDevice, VkSurfaceCapabilitiesKHR> surface_capabilities;
 };
 
-struct SwapchainKHRInfo : public VulkanObjectInfo<VkSwapchainKHR>
+struct VulkanSwapchainKHRInfo : public VulkanObjectInfo<VkSwapchainKHR>
 {
     VkSurfaceKHR         surface{ VK_NULL_HANDLE };
     format::HandleId     surface_id{ format::kNullHandleId };
-    DeviceInfo*          device_info{ nullptr };
+    VulkanDeviceInfo*    device_info{ nullptr };
     uint32_t             width{ 0 };
     uint32_t             height{ 0 };
     VkFormat             format{ VK_FORMAT_UNDEFINED };
@@ -500,33 +563,33 @@ struct SwapchainKHRInfo : public VulkanObjectInfo<VkSwapchainKHR>
 
     // When replay is restricted to a specific surface, a dummy swapchain is created for the omitted surfaces, requiring
     // backing images.
-    uint32_t                  replay_image_count{ 0 };
-    std::vector<ImageInfo>    image_infos;
-    VkSwapchainCreateFlagsKHR image_flags{ 0 };
-    VkFormat                  image_format{ VK_FORMAT_UNDEFINED };
-    uint32_t                  image_array_layers{ 0 };
-    VkImageUsageFlags         image_usage{ 0 };
-    VkSharingMode             image_sharing_mode{ VK_SHARING_MODE_EXCLUSIVE };
+    uint32_t                     replay_image_count{ 0 };
+    std::vector<VulkanImageInfo> image_infos;
+    VkSwapchainCreateFlagsKHR    image_flags{ 0 };
+    VkFormat                     image_format{ VK_FORMAT_UNDEFINED };
+    uint32_t                     image_array_layers{ 0 };
+    VkImageUsageFlags            image_usage{ 0 };
+    VkSharingMode                image_sharing_mode{ VK_SHARING_MODE_EXCLUSIVE };
 };
 
-struct ValidationCacheEXTInfo : public VulkanObjectInfo<VkValidationCacheEXT>
+struct VulkanValidationCacheEXTInfo : public VulkanObjectInfo<VkValidationCacheEXT>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
 };
 
-struct ImageViewInfo : public VulkanObjectInfo<VkImageView>
+struct VulkanImageViewInfo : public VulkanObjectInfo<VkImageView>
 {
     format::HandleId image_id{ format::kNullHandleId };
 };
 
-struct FramebufferInfo : public VulkanObjectInfo<VkFramebuffer>
+struct VulkanFramebufferInfo : public VulkanObjectInfo<VkFramebuffer>
 {
     VkFramebufferCreateFlags             framebuffer_flags{ 0 };
     std::unordered_map<uint32_t, size_t> array_counts;
     std::vector<format::HandleId>        attachment_image_view_ids;
 };
 
-struct DeferredOperationKHRInfo : public VulkanObjectInfo<VkDeferredOperationKHR>
+struct VulkanDeferredOperationKHRInfo : public VulkanObjectInfo<VkDeferredOperationKHR>
 {
     bool pending_state{ false };
 
@@ -537,24 +600,43 @@ struct DeferredOperationKHRInfo : public VulkanObjectInfo<VkDeferredOperationKHR
     std::vector<format::HandleId>                                  capturePipelines;
 };
 
-struct VideoSessionKHRInfo : VulkanObjectInfo<VkVideoSessionKHR>
+struct VulkanVideoSessionKHRInfo : VulkanObjectInfo<VkVideoSessionKHR>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
+
+    // The following values are only used for memory portability.
+    std::vector<VulkanResourceAllocator::ResourceData> allocator_datas;
+
+    // This is only used when loading the initial state for trimmed files.
+    std::vector<VkMemoryPropertyFlags> memory_property_flags;
+
+    uint32_t queue_family_index{ 0 };
 };
 
-struct ShaderEXTInfo : VulkanObjectInfo<VkShaderEXT>
+struct VulkanShaderEXTInfo : VulkanObjectInfoAsync<VkShaderEXT>
 {
     std::unordered_map<uint32_t, size_t> array_counts;
+
+    // keep track of existing usage of buffer-references
+    std::vector<gfxrecon::util::SpirVParsingUtil::BufferReferenceInfo> buffer_reference_infos;
 };
 
-struct CommandBufferInfo : public VulkanPoolObjectInfo<VkCommandBuffer>
+struct VulkanCommandBufferInfo : public VulkanPoolObjectInfo<VkCommandBuffer>
 {
-    bool                                                is_frame_boundary{ false };
-    std::vector<format::HandleId>                       frame_buffer_ids;
-    std::unordered_map<format::HandleId, VkImageLayout> image_layout_barriers;
+    bool                                                      is_frame_boundary{ false };
+    std::vector<format::HandleId>                             frame_buffer_ids;
+    std::unordered_map<format::HandleId, VkImageLayout>       image_layout_barriers;
+    std::unordered_map<VkPipelineBindPoint, format::HandleId> bound_pipelines;
+    std::vector<uint8_t>                                      push_constant_data;
+    VkShaderStageFlags                                        push_constant_stage_flags     = 0;
+    VkPipelineLayout                                          push_constant_pipeline_layout = VK_NULL_HANDLE;
+
+    // collect buffer-device-addresses of locations to replace before submit
+    std::unordered_set<VkDeviceAddress> addresses_to_replace;
+    bool                                inside_renderpass = false;
 };
 
-struct RenderPassInfo : public VulkanObjectInfo<VkRenderPass>
+struct VulkanRenderPassInfo : public VulkanObjectInfo<VkRenderPass>
 {
     std::vector<VkImageLayout>           attachment_description_final_layouts;
     std::vector<VkAttachmentDescription> attachment_descs;
@@ -576,56 +658,69 @@ struct RenderPassInfo : public VulkanObjectInfo<VkRenderPass>
     std::vector<SubpassReferences> subpass_refs;
 
     std::vector<VkSubpassDependency> dependencies;
-};
 
-struct DescriptorSetLayoutInfo : public VulkanObjectInfo<VkDescriptorSetLayout>
-{
-    struct DescriptorBindingLayout
+    // Multiview info
+    bool has_multiview{ false };
+
+    struct
     {
-        VkDescriptorType type;
-        uint32_t         binding;
-        uint32_t         count;
-    };
-
-    std::vector<DescriptorBindingLayout> bindings_layout;
+        std::vector<uint32_t> view_masks;
+        std::vector<int32_t>  view_offsets;
+        std::vector<uint32_t> correlation_masks;
+    } multiview;
 };
 
-struct DescriptorTypeImageInfo
+struct VulkanDescriptorTypeImageInfo
 {
-    const ImageViewInfo* image_view_info;
-    VkImageLayout        image_layout;
+    const VulkanImageViewInfo* image_view_info;
+    VkImageLayout              image_layout;
 };
 
-struct DescriptorTypeBufferInfo
+struct VulkanDescriptorTypeBufferInfo
 {
-    const BufferInfo* buffer_info;
-    VkDeviceSize      offset;
-    VkDeviceSize      range;
+    const VulkanBufferInfo* buffer_info;
+    VkDeviceSize            offset;
+    VkDeviceSize            range;
 };
 
-struct DescriptorSetBindingInfo
+struct VulkanDescriptorSetBindingInfo
 {
-    VkDescriptorType                      desc_type{ VK_DESCRIPTOR_TYPE_MAX_ENUM };
-    std::vector<DescriptorTypeImageInfo>  image_info;
-    std::vector<DescriptorTypeBufferInfo> buffer_info;
-    std::vector<const BufferViewInfo*>    texel_buffer_view_info;
-    std::vector<uint8_t>                  inline_uniform_block;
+    VkDescriptorType   desc_type{ VK_DESCRIPTOR_TYPE_MAX_ENUM };
+    VkShaderStageFlags stage_flags{ 0 };
+
+    // Use a map to represent array as many entries can be left unpopulated.
+    // Use a sorted map so that array indices are printed in order in the json output
+    std::map<uint32_t, VulkanDescriptorTypeImageInfo>  image_info;
+    std::map<uint32_t, VulkanDescriptorTypeBufferInfo> buffer_info;
+    std::map<uint32_t, const VulkanBufferViewInfo*>    texel_buffer_view_info;
+    std::vector<uint8_t>                               inline_uniform_block;
 };
 
-struct DescriptorSetInfo : public VulkanPoolObjectInfo<VkDescriptorSet>
+struct VulkanDescriptorSetInfo : public VulkanPoolObjectInfo<VkDescriptorSet>
 {
     // One entry per binding
-    using DescriptorBindingsInfo = std::unordered_map<uint32_t, DescriptorSetBindingInfo>;
-    DescriptorBindingsInfo descriptors;
+    using VulkanDescriptorBindingsInfo = std::unordered_map<uint32_t, VulkanDescriptorSetBindingInfo>;
+    VulkanDescriptorBindingsInfo descriptors;
+};
+
+struct VulkanAccelerationStructureKHRInfo : public VulkanObjectInfo<VkAccelerationStructureKHR>
+{
+    VkDeviceAddress capture_address = 0;
+    VkDeviceAddress replay_address  = 0;
+
+    VkAccelerationStructureTypeKHR type = VK_ACCELERATION_STRUCTURE_TYPE_MAX_ENUM_KHR;
+
+    //! associated buffer
+    VkBuffer buffer = VK_NULL_HANDLE;
 };
 
 //
 // Handle alias types for extension handle types that have been promoted to core types.
 //
 
-typedef SamplerYcbcrConversionInfo   SamplerYcbcrConversionKHRInfo;
-typedef DescriptorUpdateTemplateInfo DescriptorUpdateTemplateKHRInfo;
-typedef PrivateDataSlotInfo          PrivateDataSlotEXTInfo;
+typedef VulkanSamplerYcbcrConversionInfo   VulkanSamplerYcbcrConversionKHRInfo;
+typedef VulkanDescriptorUpdateTemplateInfo VulkanDescriptorUpdateTemplateKHRInfo;
+typedef VulkanPrivateDataSlotInfo          VulkanPrivateDataSlotEXTInfo;
 
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
