@@ -660,16 +660,13 @@ void VulkanReferencedResourceConsumerBase::Process_vkCmdPushDescriptorSetWithTem
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCmdPushDescriptorSetWithTemplate2KHR(
-    const ApiCallInfo&                                                    call_info,
-    format::HandleId                                                      commandBuffer,
-    StructPointerDecoder<Decoded_VkPushDescriptorSetWithTemplateInfoKHR>* pPushDescriptorSetWithTemplateInfo)
+    const ApiCallInfo&                                                 call_info,
+    format::HandleId                                                   commandBuffer,
+    StructPointerDecoder<Decoded_VkPushDescriptorSetWithTemplateInfo>* pPushDescriptorSetWithTemplateInfo)
 {
-    Decoded_VkPushDescriptorSetWithTemplateInfoKHR* info = pPushDescriptorSetWithTemplateInfo->GetMetaStructPointer();
-    format::HandleId                                descriptorUpdateTemplate = info->descriptorUpdateTemplate;
-
-    DescriptorUpdateTemplateDecoder pData;
-    const uint8_t*                  ptr = static_cast<const uint8_t*>(info->pNext->GetPointer());
-    pData.Decode(ptr, pData.GetLength());
+    Decoded_VkPushDescriptorSetWithTemplateInfo* info = pPushDescriptorSetWithTemplateInfo->GetMetaStructPointer();
+    format::HandleId                             descriptorUpdateTemplate = info->descriptorUpdateTemplate;
+    DescriptorUpdateTemplateDecoder              pData                    = info->pData;
 
     PushDescriptorSetWithTemplate(commandBuffer, descriptorUpdateTemplate, &pData);
 }
@@ -1161,5 +1158,39 @@ void VulkanReferencedResourceConsumerBase::PushDescriptorSetWithTemplate(format:
     }
 }
 
+void VulkanReferencedResourceConsumerBase::Process_vkGetBufferDeviceAddress(
+    const ApiCallInfo&                                       call_info,
+    VkDeviceAddress                                          returnValue,
+    format::HandleId                                         device,
+    StructPointerDecoder<Decoded_VkBufferDeviceAddressInfo>* pInfo)
+{
+    if (pInfo != nullptr && pInfo->GetMetaStructPointer() != nullptr)
+    {
+        const auto* buffer_device_address = pInfo->GetMetaStructPointer();
+        if (buffer_device_address != nullptr)
+        {
+            table_.MarkResourceAsUsed(buffer_device_address->buffer);
+        }
+    }
+}
+
+void VulkanReferencedResourceConsumerBase::Process_vkGetBufferDeviceAddressKHR(
+    const ApiCallInfo&                                       call_info,
+    VkDeviceAddress                                          returnValue,
+    format::HandleId                                         device,
+    StructPointerDecoder<Decoded_VkBufferDeviceAddressInfo>* pInfo)
+{
+    Process_vkGetBufferDeviceAddress(call_info, returnValue, device, pInfo);
+}
+
+void VulkanReferencedResourceConsumerBase::Process_vkGetBufferDeviceAddressEXT(
+    const ApiCallInfo&                                       call_info,
+    VkDeviceAddress                                          returnValue,
+    format::HandleId                                         device,
+    StructPointerDecoder<Decoded_VkBufferDeviceAddressInfo>* pInfo)
+{
+    Process_vkGetBufferDeviceAddress(call_info, returnValue, device, pInfo);
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
