@@ -24,15 +24,14 @@
 #define GFXRECON_GENERATED_VULKAN_REPLAY_DUMP_RESOURCES_BASE_H
 
 #include "decode/api_decoder.h"
+#include "decode/common_object_info_table.h"
 #include "decode/vulkan_object_info.h"
 #include "decode/vulkan_replay_options.h"
-#include "decode/vulkan_object_info_table.h"
 #include "decode/struct_pointer_decoder.h"
 #include "decode/vulkan_replay_dump_resources_common.h"
 #include "decode/vulkan_replay_dump_resources_draw_calls.h"
 #include "decode/vulkan_replay_dump_resources_compute_ray_tracing.h"
 #include "generated/generated_vulkan_dispatch_table.h"
-#include "decode/vulkan_replay_dump_resources_json.h"
 #include "format/format.h"
 #include "util/defines.h"
 #include "vulkan/vulkan_core.h"
@@ -50,14 +49,14 @@ class VulkanReplayDumpResourcesBase
   public:
     VulkanReplayDumpResourcesBase() = delete;
 
-    VulkanReplayDumpResourcesBase(const VulkanReplayOptions& options, VulkanObjectInfoTable& object_info_table);
+    VulkanReplayDumpResourcesBase(const VulkanReplayOptions& options, CommonObjectInfoTable* object_info_table);
 
     ~VulkanReplayDumpResourcesBase();
 
-    VkResult CloneCommandBuffer(uint64_t                           bcb_index,
-                                CommandBufferInfo*                 original_command_buffer_info,
-                                const encode::VulkanDeviceTable*   device_table,
-                                const encode::VulkanInstanceTable* inst_table);
+    VkResult CloneCommandBuffer(uint64_t                             bcb_index,
+                                VulkanCommandBufferInfo*             original_command_buffer_info,
+                                const graphics::VulkanDeviceTable*   device_table,
+                                const graphics::VulkanInstanceTable* inst_table);
 
     void OverrideCmdDraw(const ApiCallInfo& call_info,
                          PFN_vkCmdDraw      func,
@@ -76,18 +75,18 @@ class VulkanReplayDumpResourcesBase
                                 int32_t              vertex_offset,
                                 uint32_t             first_instance);
 
-    void OverrideCmdDrawIndirect(const ApiCallInfo&    call_info,
-                                 PFN_vkCmdDrawIndirect func,
-                                 VkCommandBuffer       original_command_buffer,
-                                 const BufferInfo*     buffer_info,
-                                 VkDeviceSize          offset,
-                                 uint32_t              draw_count,
-                                 uint32_t              stride);
+    void OverrideCmdDrawIndirect(const ApiCallInfo&      call_info,
+                                 PFN_vkCmdDrawIndirect   func,
+                                 VkCommandBuffer         original_command_buffer,
+                                 const VulkanBufferInfo* buffer_info,
+                                 VkDeviceSize            offset,
+                                 uint32_t                draw_count,
+                                 uint32_t                stride);
 
     void OverrideCmdDrawIndexedIndirect(const ApiCallInfo&           call_info,
                                         PFN_vkCmdDrawIndexedIndirect func,
                                         VkCommandBuffer              original_command_buffer,
-                                        const BufferInfo*            buffer_info,
+                                        const VulkanBufferInfo*      buffer_info,
                                         VkDeviceSize                 offset,
                                         uint32_t                     draw_count,
                                         uint32_t                     stride);
@@ -95,9 +94,9 @@ class VulkanReplayDumpResourcesBase
     void OverrideCmdDrawIndirectCount(const ApiCallInfo&         call_info,
                                       PFN_vkCmdDrawIndirectCount func,
                                       VkCommandBuffer            original_command_buffer,
-                                      const BufferInfo*          buffer_info,
+                                      const VulkanBufferInfo*    buffer_info,
                                       VkDeviceSize               offset,
-                                      const BufferInfo*          count_buffer_info,
+                                      const VulkanBufferInfo*    count_buffer_info,
                                       VkDeviceSize               count_buffer_offset,
                                       uint32_t                   max_draw_count,
                                       uint32_t                   stride);
@@ -105,9 +104,9 @@ class VulkanReplayDumpResourcesBase
     void OverrideCmdDrawIndexedIndirectCount(const ApiCallInfo&                call_info,
                                              PFN_vkCmdDrawIndexedIndirectCount func,
                                              VkCommandBuffer                   original_command_buffer,
-                                             const BufferInfo*                 buffer_info,
+                                             const VulkanBufferInfo*           buffer_info,
                                              VkDeviceSize                      offset,
-                                             const BufferInfo*                 count_buffer_info,
+                                             const VulkanBufferInfo*           count_buffer_info,
                                              VkDeviceSize                      count_buffer_offset,
                                              uint32_t                          max_draw_count,
                                              uint32_t                          stride);
@@ -115,9 +114,9 @@ class VulkanReplayDumpResourcesBase
     void OverrideCmdDrawIndirectCountKHR(const ApiCallInfo&            call_info,
                                          PFN_vkCmdDrawIndirectCountKHR func,
                                          VkCommandBuffer               original_command_buffer,
-                                         const BufferInfo*             buffer_info,
+                                         const VulkanBufferInfo*       buffer_info,
                                          VkDeviceSize                  offset,
-                                         const BufferInfo*             count_buffer_info,
+                                         const VulkanBufferInfo*       count_buffer_info,
                                          VkDeviceSize                  count_buffer_offset,
                                          uint32_t                      max_draw_count,
                                          uint32_t                      stride);
@@ -125,9 +124,9 @@ class VulkanReplayDumpResourcesBase
     void OverrideCmdDrawIndexedIndirectCountKHR(const ApiCallInfo&                   call_info,
                                                 PFN_vkCmdDrawIndexedIndirectCountKHR func,
                                                 VkCommandBuffer                      original_command_buffer,
-                                                const BufferInfo*                    buffer_info,
+                                                const VulkanBufferInfo*              buffer_info,
                                                 VkDeviceSize                         offset,
-                                                const BufferInfo*                    count_buffer_info,
+                                                const VulkanBufferInfo*              count_buffer_info,
                                                 VkDeviceSize                         count_buffer_offset,
                                                 uint32_t                             max_draw_count,
                                                 uint32_t                             stride);
@@ -164,11 +163,11 @@ class VulkanReplayDumpResourcesBase
                                  StructPointerDecoder<Decoded_VkSubpassBeginInfo>* pSubpassBeginInfo,
                                  StructPointerDecoder<Decoded_VkSubpassEndInfo>*   pSubpassEndInfo);
 
-    void OverrideCmdBindPipeline(const ApiCallInfo&    call_info,
-                                 PFN_vkCmdBindPipeline func,
-                                 VkCommandBuffer       original_command_buffer,
-                                 VkPipelineBindPoint   pipelineBindPoint,
-                                 const PipelineInfo*   pipeline);
+    void OverrideCmdBindPipeline(const ApiCallInfo&        call_info,
+                                 PFN_vkCmdBindPipeline     func,
+                                 VkCommandBuffer           original_command_buffer,
+                                 VkPipelineBindPoint       pipelineBindPoint,
+                                 const VulkanPipelineInfo* pipeline);
 
     void OverrideCmdDispatch(const ApiCallInfo& call_info,
                              PFN_vkCmdDispatch  func,
@@ -180,7 +179,7 @@ class VulkanReplayDumpResourcesBase
     void OverrideCmdDispatchIndirect(const ApiCallInfo&        call_info,
                                      PFN_vkCmdDispatchIndirect func,
                                      VkCommandBuffer           original_command_buffer,
-                                     const BufferInfo*         buffer_info,
+                                     const VulkanBufferInfo*   buffer_info,
                                      VkDeviceSize              offset);
 
     void
@@ -210,21 +209,21 @@ class VulkanReplayDumpResourcesBase
                                           VkCommandBuffer                original_command_buffer,
                                           VkDeviceAddress                indirectDeviceAddress);
 
-    void OverrideCmdBindDescriptorSets(const ApiCallInfo&          call_info,
-                                       PFN_vkCmdBindDescriptorSets func,
-                                       VkCommandBuffer             original_command_buffer,
-                                       VkPipelineBindPoint         pipeline_bind_point,
-                                       const PipelineLayoutInfo*   layout_info,
-                                       uint32_t                    first_set,
-                                       uint32_t                    descriptor_sets_count,
-                                       const format::HandleId*     descriptor_sets_ids,
-                                       uint32_t                    dynamicOffsetCount,
-                                       const uint32_t*             pDynamicOffsets);
+    void OverrideCmdBindDescriptorSets(const ApiCallInfo&              call_info,
+                                       PFN_vkCmdBindDescriptorSets     func,
+                                       VkCommandBuffer                 original_command_buffer,
+                                       VkPipelineBindPoint             pipeline_bind_point,
+                                       const VulkanPipelineLayoutInfo* layout_info,
+                                       uint32_t                        first_set,
+                                       uint32_t                        descriptor_sets_count,
+                                       const format::HandleId*         descriptor_sets_ids,
+                                       uint32_t                        dynamicOffsetCount,
+                                       const uint32_t*                 pDynamicOffsets);
 
     void OverrideCmdBindIndexBuffer(const ApiCallInfo&       call_info,
                                     PFN_vkCmdBindIndexBuffer func,
                                     VkCommandBuffer          original_command_buffer,
-                                    const BufferInfo*        buffer,
+                                    const VulkanBufferInfo*  buffer,
                                     VkDeviceSize             offset,
                                     VkIndexType              indexType);
 
@@ -255,10 +254,20 @@ class VulkanReplayDumpResourcesBase
                                        const VkDeviceSize*         pSizes,
                                        const VkDeviceSize*         pStrides);
 
+    void OverrideCmdBindVertexBuffers2EXT(const ApiCallInfo&             call_info,
+                                          PFN_vkCmdBindVertexBuffers2EXT func,
+                                          VkCommandBuffer                original_command_buffer,
+                                          uint32_t                       firstBinding,
+                                          uint32_t                       bindingCount,
+                                          const format::HandleId*        pBuffers_ids,
+                                          const VkDeviceSize*            pOffsets,
+                                          const VkDeviceSize*            pSizes,
+                                          const VkDeviceSize*            pStrides);
+
     void OverrideCmdBindIndexBuffer2KHR(const ApiCallInfo&           call_info,
                                         PFN_vkCmdBindIndexBuffer2KHR func,
                                         VkCommandBuffer              commandBuffer,
-                                        const BufferInfo*            buffer,
+                                        const VulkanBufferInfo*      buffer,
                                         VkDeviceSize                 offset,
                                         VkDeviceSize                 size,
                                         VkIndexType                  indexType);
@@ -280,11 +289,14 @@ class VulkanReplayDumpResourcesBase
                                     PFN_vkCmdEndRenderingKHR func,
                                     VkCommandBuffer          commandBuffer);
 
-    VkResult QueueSubmit(const std::vector<VkSubmitInfo>& modified_submit_infos,
-                         const encode::VulkanDeviceTable& device_table,
-                         VkQueue                          queue,
-                         VkFence                          fence,
-                         uint64_t                         index);
+    void
+    OverrideEndCommandBuffer(const ApiCallInfo& call_info, PFN_vkEndCommandBuffer func, VkCommandBuffer commandBuffer);
+
+    VkResult QueueSubmit(const std::vector<VkSubmitInfo>&   modified_submit_infos,
+                         const graphics::VulkanDeviceTable& device_table,
+                         VkQueue                            queue,
+                         VkFence                            fence,
+                         uint64_t                           index);
 
     bool MustDumpQueueSubmitIndex(uint64_t index) const;
 
@@ -310,6 +322,12 @@ class VulkanReplayDumpResourcesBase
 
     uint64_t GetBeginCommandBufferIndexOfCommandBuffer(VkCommandBuffer original_command_buffer) const;
 
+    void DumpGraphicsPipelineInfos(const StructPointerDecoder<Decoded_VkGraphicsPipelineCreateInfo>* pCreateInfos,
+                                   uint32_t                                                          createInfoCount,
+                                   HandlePointerDecoder<VkPipeline>*                                 pPipelines);
+
+    void DumpResourcesSetFatalErrorHandler(std::function<void(const char*)> handler);
+
   private:
     bool UpdateRecordingStatus(VkCommandBuffer original_command_buffer);
 
@@ -330,22 +348,38 @@ class VulkanReplayDumpResourcesBase
 
     const DrawCallsDumpingContext* FindDrawCallCommandBufferContext(uint64_t bcb_id) const;
 
+    void HandleCmdBindVertexBuffers2(const ApiCallInfo&          call_info,
+                                     PFN_vkCmdBindVertexBuffers2 func,
+                                     VkCommandBuffer             original_command_buffer,
+                                     uint32_t                    firstBinding,
+                                     uint32_t                    bindingCount,
+                                     const format::HandleId*     pBuffers_ids,
+                                     const VkDeviceSize*         pOffsets,
+                                     const VkDeviceSize*         pSizes,
+                                     const VkDeviceSize*         pStrides);
+
     // Mapping between the original VkCommandBuffer handle and BeginCommandBuffer index
     std::unordered_map<VkCommandBuffer, uint64_t> cmd_buf_begin_map_;
 
-    std::unordered_set<uint64_t> QueueSubmit_indices_;
+    std::vector<uint64_t> QueueSubmit_indices_;
 
     // One per BeginCommandBuffer index
     std::unordered_map<uint64_t, DrawCallsDumpingContext>         draw_call_contexts;
     std::unordered_map<uint64_t, DispatchTraceRaysDumpingContext> dispatch_ray_contexts;
 
-    bool                          recording_;
-    bool                          dump_resources_before_;
-    VulkanObjectInfoTable&        object_info_table_;
-    VulkanReplayDumpResourcesJson dump_json_;
-    bool                          output_json_per_command;
+    bool                   recording_;
+    bool                   dump_resources_before_;
+    CommonObjectInfoTable* object_info_table_;
+    bool                   output_json_per_command;
+
+    std::unique_ptr<DefaultVulkanDumpResourcesDelegate> default_delegate_;
+    VulkanDumpResourcesDelegate*                        user_delegate_;
+    VulkanDumpResourcesDelegate*                        active_delegate_;
 
     std::string capture_filename;
+
+    std::function<void(const char*)> fatal_error_handler_;
+    void                             RaiseFatalError(const char* message) const;
 };
 
 GFXRECON_END_NAMESPACE(gfxrecon)
