@@ -23,14 +23,14 @@
 #ifndef GFXRECON_GENERATED_VULKAN_REPLAY_DUMP_RESOURCES_COMPUTE_RAY_TRACING_H
 #define GFXRECON_GENERATED_VULKAN_REPLAY_DUMP_RESOURCES_COMPUTE_RAY_TRACING_H
 
+#include "decode/common_object_info_table.h"
 #include "decode/vulkan_replay_dump_resources_common.h"
 #include "decode/vulkan_object_info.h"
-#include "decode/vulkan_object_info_table.h"
 #include "decode/vulkan_replay_options.h"
 #include "generated/generated_vulkan_dispatch_table.h"
-#include "decode/vulkan_replay_dump_resources_json.h"
 #include "format/format.h"
 #include "util/defines.h"
+#include "util/logging.h"
 #include "vulkan/vulkan_core.h"
 
 #include <cstdint>
@@ -46,34 +46,31 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 class DispatchTraceRaysDumpingContext
 {
   public:
-    DispatchTraceRaysDumpingContext(const std::vector<uint64_t>&   dispatch_indices,
-                                    const std::vector<uint64_t>&   trace_rays_indices,
-                                    VulkanObjectInfoTable&         object_info_table,
-                                    const VulkanReplayOptions&     options,
-                                    VulkanReplayDumpResourcesJson& dump_json,
-                                    std::string                    capture_filename);
+    DispatchTraceRaysDumpingContext(const std::vector<uint64_t>& dispatch_indices,
+                                    const std::vector<uint64_t>& trace_rays_indices,
+                                    CommonObjectInfoTable&       object_info_table,
+                                    const VulkanReplayOptions&   options,
+                                    VulkanDumpResourcesDelegate& delegate);
 
     ~DispatchTraceRaysDumpingContext();
 
-    VkResult CloneCommandBuffer(CommandBufferInfo*                 orig_cmd_buf_info,
-                                const encode::VulkanDeviceTable*   dev_table,
-                                const encode::VulkanInstanceTable* inst_table);
+    VkResult CloneCommandBuffer(VulkanCommandBufferInfo*             orig_cmd_buf_info,
+                                const graphics::VulkanDeviceTable*   dev_table,
+                                const graphics::VulkanInstanceTable* inst_table);
 
     VkCommandBuffer GetDispatchRaysCommandBuffer() const { return DR_command_buffer; }
 
-    void BindPipeline(VkPipelineBindPoint bind_point, const PipelineInfo* pipeline);
-
     bool IsRecording() const;
 
     bool MustDumpDispatch(uint64_t index) const;
 
     bool MustDumpTraceRays(uint64_t index) const;
 
-    void BindDescriptorSets(VkPipelineBindPoint                          pipeline_bind_point,
-                            uint32_t                                     first_set,
-                            const std::vector<const DescriptorSetInfo*>& descriptor_sets_infos,
-                            uint32_t                                     dynamicOffsetCount,
-                            const uint32_t*                              pDynamicOffsets);
+    void BindDescriptorSets(VkPipelineBindPoint                                pipeline_bind_point,
+                            uint32_t                                           first_set,
+                            const std::vector<const VulkanDescriptorSetInfo*>& descriptor_sets_infos,
+                            uint32_t                                           dynamicOffsetCount,
+                            const uint32_t*                                    pDynamicOffsets);
 
     VkResult DumpDispatchTraceRays(
         VkQueue queue, uint64_t qs_index, uint64_t bcb_index, const VkSubmitInfo& submit_info, VkFence fence);
@@ -84,7 +81,7 @@ class DispatchTraceRaysDumpingContext
 
     void InsertNewDispatchParameters(uint64_t index, uint32_t groupCountX, uint32_t groupCountY, uint32_t groupCountZ);
 
-    void InsertNewDispatchParameters(uint64_t index, const BufferInfo* buffer_info, VkDeviceSize offset);
+    void InsertNewDispatchParameters(uint64_t index, const VulkanBufferInfo* buffer_info, VkDeviceSize offset);
 
     void InsertNewTraceRaysParameters(uint64_t                               index,
                                       const VkStridedDeviceAddressRegionKHR* pRaygenShaderBindingTable,
@@ -108,56 +105,23 @@ class DispatchTraceRaysDumpingContext
 
     VkResult CloneTraceRaysMutableResources(uint64_t index, bool cloning_before_cmd);
 
-    void SnapshotBoundDescriptorsDispatch(uint64_t index);
+    void SnapshotDispatchState(uint64_t index);
 
-    void SnapshotBoundDescriptorsTraceRays(uint64_t index);
+    void SnapshotTraceRaysState(uint64_t index);
 
-    VkResult CopyDispatchIndirectParameters(uint64_t index);
+    void BindPipeline(VkPipelineBindPoint bind_point, const VulkanPipelineInfo* pipeline);
 
-    VkResult CopyTraceRaysIndirectParameters(uint64_t index);
+    void EndCommandBuffer();
 
     void Release();
 
   private:
-    std::vector<std::string> GenerateDispatchTraceRaysImageFilename(VkFormat              format,
-                                                                    uint32_t              levels,
-                                                                    uint32_t              layers,
-                                                                    bool                  is_dispatch,
-                                                                    uint64_t              qs_index,
-                                                                    uint64_t              bcb_index,
-                                                                    uint64_t              cmd_index,
-                                                                    uint32_t              desc_set,
-                                                                    uint32_t              desc_binding,
-                                                                    uint32_t              array_index,
-                                                                    VkShaderStageFlagBits stage,
-                                                                    bool                  before_cmd,
-                                                                    bool                  dump_all_subresources) const;
+    void CopyImageResource(const VulkanImageInfo* src_image_info, VkImage dst_image);
 
-    std::string GenerateDispatchTraceRaysBufferFilename(bool                  is_dispatch,
-                                                        uint64_t              qs_index,
-                                                        uint64_t              bcb_index,
-                                                        uint64_t              cmd_index,
-                                                        uint32_t              desc_set,
-                                                        uint32_t              desc_binding,
-                                                        uint32_t              array_index,
-                                                        VkShaderStageFlagBits stage,
-                                                        bool                  before_cmd) const;
-
-    std::vector<std::string>
-    GenerateImageDescriptorFilename(uint64_t bcb_index, uint64_t cmd_index, const ImageInfo* img_info) const;
-
-    std::string
-    GenerateBufferDescriptorFilename(uint64_t bcb_index, uint64_t cmd_index, format::HandleId buffer_id) const;
-
-    std::string GenerateInlineUniformBufferDescriptorFilename(uint64_t bcb_index,
-                                                              uint64_t cmd_index,
-                                                              uint32_t set,
-                                                              uint32_t binding) const;
-
-    void CopyImageResource(const ImageInfo* src_image_info, VkImage dst_image);
-
-    void
-    CopyBufferResource(const BufferInfo* src_buffer_info, VkDeviceSize offset, VkDeviceSize range, VkBuffer dst_buffer);
+    void CopyBufferResource(const VulkanBufferInfo* src_buffer_info,
+                            VkDeviceSize            offset,
+                            VkDeviceSize            range,
+                            VkBuffer                dst_buffer);
 
     void DestroyMutableResourcesClones();
 
@@ -167,34 +131,22 @@ class DispatchTraceRaysDumpingContext
 
     VkResult DumpImmutableDescriptors(uint64_t qs_index, uint64_t bcb_index, uint64_t cmd_index, bool is_dispatch);
 
-    void GenerateOutputJsonDispatchInfo(uint64_t qs_index,
-                                        uint64_t bcb_index,
-                                        uint64_t disp_index,
-                                        uint64_t cmd_index) const;
-
-    void GenerateOutputJsonTraceRaysIndex(uint64_t qs_index,
-                                          uint64_t bcb_index,
-                                          uint64_t tr_index,
-                                          uint64_t cmd_index) const;
-
-    const CommandBufferInfo*       original_command_buffer_info;
+    const VulkanCommandBufferInfo* original_command_buffer_info;
     VkCommandBuffer                DR_command_buffer;
     std::vector<uint64_t>          dispatch_indices;
     std::vector<uint64_t>          trace_rays_indices;
-    const PipelineInfo*            bound_pipelines[kBindPoint_count];
     bool                           dump_resources_before;
-    const std::string&             dump_resource_path;
-    util::ScreenshotFormat         image_file_format;
-    float                          dump_resources_scale;
-    VulkanReplayDumpResourcesJson& dump_json;
-    bool                           output_json_per_command;
+    VulkanDumpResourcesDelegate&   delegate_;
     bool                           dump_immutable_resources;
-    bool                           dump_all_image_subresources;
 
     // One entry per descriptor set for each compute and ray tracing binding points
-    std::unordered_map<uint32_t, DescriptorSetInfo> bound_descriptor_sets_compute;
-    std::unordered_map<uint32_t, DescriptorSetInfo> bound_descriptor_sets_ray_tracing;
+    std::unordered_map<uint32_t, VulkanDescriptorSetInfo> bound_descriptor_sets_compute;
+    std::unordered_map<uint32_t, VulkanDescriptorSetInfo> bound_descriptor_sets_ray_tracing;
 
+    const VulkanPipelineInfo* bound_pipeline_compute;
+    const VulkanPipelineInfo* bound_pipeline_trace_rays;
+
+  public:
     // For each Dispatch/TraceRays that we dump we create a clone of all mutable resources used in the
     // shaders/pipeline and the content of the original resources are copied into the clones
     struct MutableResourcesBackupContext
@@ -203,28 +155,28 @@ class DispatchTraceRaysDumpingContext
 
         struct ImageContext
         {
-            const ImageInfo*      original_image{ nullptr };
-            VkImage               image{ VK_NULL_HANDLE };
-            VkDeviceMemory        image_memory{ VK_NULL_HANDLE };
-            VkShaderStageFlagBits stage;
-            VkDescriptorType      desc_type;
-            uint32_t              desc_set;
-            uint32_t              desc_binding;
-            uint32_t              array_index;
+            const VulkanImageInfo* original_image{ nullptr };
+            VkImage                image{ VK_NULL_HANDLE };
+            VkDeviceMemory         image_memory{ VK_NULL_HANDLE };
+            VkShaderStageFlags     stages;
+            VkDescriptorType       desc_type;
+            uint32_t               desc_set;
+            uint32_t               desc_binding;
+            uint32_t               array_index;
         };
 
         std::vector<ImageContext> images;
 
         struct BufferContext
         {
-            const BufferInfo*     original_buffer{ nullptr };
-            VkBuffer              buffer{ VK_NULL_HANDLE };
-            VkDeviceMemory        buffer_memory{ VK_NULL_HANDLE };
-            VkShaderStageFlagBits stage;
-            VkDescriptorType      desc_type;
-            uint32_t              desc_set;
-            uint32_t              desc_binding;
-            uint32_t              array_index;
+            const VulkanBufferInfo* original_buffer{ nullptr };
+            VkBuffer                buffer{ VK_NULL_HANDLE };
+            VkDeviceMemory          buffer_memory{ VK_NULL_HANDLE };
+            VkShaderStageFlags      stages;
+            VkDescriptorType        desc_type;
+            uint32_t                desc_set;
+            uint32_t                desc_binding;
+            uint32_t                array_index;
         };
 
         std::vector<BufferContext> buffers;
@@ -237,6 +189,24 @@ class DispatchTraceRaysDumpingContext
         kDispatchBase
     };
 
+    static bool IsDispatchIndirect(DispatchTypes type)
+    {
+        switch (type)
+        {
+            case kDispatch:
+            case kDispatchBase:
+                return false;
+
+            case kDispatchIndirect:
+                return true;
+
+            default:
+                GFXRECON_LOG_ERROR("%s() Unrecognized dispatch call type (%u)", __func__, static_cast<uint32_t>(type))
+                GFXRECON_ASSERT(0);
+                return false;
+        }
+    }
+
     static const char* DispatchTypeToStr(DispatchTypes type)
     {
         switch (type)
@@ -280,8 +250,8 @@ class DispatchTraceRaysDumpingContext
 
             struct DispatchIndirect
             {
-                const BufferInfo* params_buffer_info;
-                VkDeviceSize      params_buffer_offset;
+                const VulkanBufferInfo* params_buffer_info;
+                VkDeviceSize            params_buffer_offset;
 
                 VkBuffer       new_params_buffer;
                 VkDeviceMemory new_params_memory;
@@ -299,7 +269,7 @@ class DispatchTraceRaysDumpingContext
                 dispatch{ groupCountX, groupCountY, groupCountZ }
             {}
 
-            DispatchParamsUnion(const BufferInfo* params_buffer_info, VkDeviceSize offset) :
+            DispatchParamsUnion(const VulkanBufferInfo* params_buffer_info, VkDeviceSize offset) :
                 dispatch_indirect{ params_buffer_info, offset, VK_NULL_HANDLE, VK_NULL_HANDLE, nullptr }
             {}
 
@@ -319,7 +289,7 @@ class DispatchTraceRaysDumpingContext
             assert(type == kDispatch);
         }
 
-        DispatchParameters(DispatchTypes type, const BufferInfo* params_buffer_info, VkDeviceSize offset) :
+        DispatchParameters(DispatchTypes type, const VulkanBufferInfo* params_buffer_info, VkDeviceSize offset) :
             dispatch_params_union{ params_buffer_info, offset }, type(type)
         {
             assert(type == kDispatchIndirect);
@@ -340,7 +310,7 @@ class DispatchTraceRaysDumpingContext
 
         DispatchTypes type;
 
-        std::unordered_map<uint32_t, DescriptorSetInfo::DescriptorBindingsInfo> referenced_descriptors;
+        std::unordered_map<uint32_t, VulkanDescriptorSetInfo::VulkanDescriptorBindingsInfo> referenced_descriptors;
 
         MutableResourcesBackupContext mutable_resources_clones;
         MutableResourcesBackupContext mutable_resources_clones_before;
@@ -429,28 +399,31 @@ class DispatchTraceRaysDumpingContext
 
         TraceRaysTypes type;
 
-        std::unordered_map<VkShaderStageFlagBits,
-                           std::unordered_map<uint32_t, DescriptorSetInfo::DescriptorBindingsInfo>>
-            referenced_descriptors;
+        std::unordered_map<uint32_t, VulkanDescriptorSetInfo::VulkanDescriptorBindingsInfo> referenced_descriptors;
 
         // Keep copies of all mutable resources that are changed by the dumped commands/shaders
         MutableResourcesBackupContext mutable_resources_clones;
         MutableResourcesBackupContext mutable_resources_clones_before;
     };
 
+  private:
     VkResult CloneMutableResources(MutableResourcesBackupContext& backup_context, bool is_dispatch);
 
-    void SnapshotBoundDescriptors(DispatchParameters& disp_params);
+    void SnapshotDispatchState(DispatchParameters& disp_params);
 
-    void SnapshotBoundDescriptors(TraceRaysParameters& tr_params);
+    void SnapshotTraceRaysState(TraceRaysParameters& tr_params);
+
+    VkResult CopyDispatchIndirectParameters(DispatchParameters& disp_params);
+
+    VkResult CopyTraceRaysIndirectParameters(TraceRaysParameters& tr_params);
 
     // Gather here all descriptors referenced by commands that have already been dumped
     // in order to avoid dumping descriptors referenced from multiple shader stages,
     // multiple times
     struct DumpedDescriptors
     {
-        std::unordered_set<const ImageInfo*>            image_descriptors;
-        std::unordered_set<const BufferInfo*>           buffer_descriptors;
+        std::unordered_set<const VulkanImageInfo*>      image_descriptors;
+        std::unordered_set<const VulkanBufferInfo*>     buffer_descriptors;
         std::unordered_set<const std::vector<uint8_t>*> inline_uniform_blocks;
     };
 
@@ -469,14 +442,14 @@ class DispatchTraceRaysDumpingContext
     // One entry for each trace rays command
     std::unordered_map<uint64_t, TraceRaysParameters> trace_rays_params;
 
-    const encode::VulkanDeviceTable*        device_table;
+    const graphics::VulkanDeviceTable*      device_table;
     VkDevice                                parent_device;
-    const encode::VulkanInstanceTable*      instance_table;
-    VulkanObjectInfoTable&                  object_info_table;
+    const graphics::VulkanInstanceTable*    instance_table;
+    CommonObjectInfoTable&                  object_info_table;
     const VkPhysicalDeviceMemoryProperties* replay_device_phys_mem_props;
     size_t                                  current_dispatch_index;
     size_t                                  current_trace_rays_index;
-    std::string                             capture_filename;
+    bool                                    reached_end_command_buffer;
 };
 
 GFXRECON_END_NAMESPACE(gfxrecon)
