@@ -1,7 +1,7 @@
 /*
 ** Copyright (c) 2018-2020 Valve Corporation
 ** Copyright (c) 2018-2021 LunarG, Inc.
-** Copyright (c) 2019-2024 Advanced Micro Devices, Inc. All rights reserved.
+** Copyright (c) 2019-2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -251,13 +251,17 @@ class D3D12CaptureManager : public ApiCaptureManager
                                              UINT                           present_flags,
                                              const DXGI_PRESENT_PARAMETERS* present_parameters);
 
-    void
-    PostProcess_IDXGISwapChain_Present(IDXGISwapChain_Wrapper* wrapper, HRESULT result, UINT sync_interval, UINT flags);
+    void PostProcess_IDXGISwapChain_Present(std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+                                            IDXGISwapChain_Wrapper*                                wrapper,
+                                            HRESULT                                                result,
+                                            UINT                                                   sync_interval,
+                                            UINT                                                   flags);
 
-    void PostProcess_IDXGISwapChain1_Present1(IDXGISwapChain_Wrapper*        wrapper,
-                                              HRESULT                        result,
-                                              UINT                           sync_interval,
-                                              UINT                           present_flags,
+    void PostProcess_IDXGISwapChain1_Present1(std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+                                              IDXGISwapChain_Wrapper*                                wrapper,
+                                              HRESULT                                                result,
+                                              UINT                                                   sync_interval,
+                                              UINT                                                   present_flags,
                                               const DXGI_PRESENT_PARAMETERS* present_parameters);
 
     void PreProcess_IDXGISwapChain_ResizeBuffers(IDXGISwapChain_Wrapper* wrapper,
@@ -294,6 +298,13 @@ class D3D12CaptureManager : public ApiCaptureManager
                                                     const UINT*             node_mask,
                                                     IUnknown* const*        present_queue);
 
+    void PreProcess_IDXGISwapChain_ResizeTarget(IDXGISwapChain_Wrapper* wrapper,
+                                                const DXGI_MODE_DESC*   pNewTargetParameters);
+
+    void PostProcess_IDXGISwapChain_ResizeTarget(IDXGISwapChain_Wrapper* wrapper,
+                                                 HRESULT                 result,
+                                                 const DXGI_MODE_DESC*   pNewTargetParameters);
+
     void Destroy_IDXGISwapChain(IDXGISwapChain_Wrapper* wrapper);
 
     void PostProcess_ID3D12Device_CreateDescriptorHeap(ID3D12Device_Wrapper*             wrapper,
@@ -444,13 +455,23 @@ class D3D12CaptureManager : public ApiCaptureManager
 
     void PostProcess_ID3D12Heap_GetDesc(ID3D12Heap_Wrapper* wrapper, D3D12_HEAP_DESC& desc);
 
-    void PreProcess_ID3D12CommandQueue_ExecuteCommandLists(ID3D12CommandQueue_Wrapper* wrapper,
-                                                           UINT                        num_lists,
-                                                           ID3D12CommandList* const*   lists);
+    void PreProcess_ID3D12CommandQueue_ExecuteCommandLists(
+        std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+        ID3D12CommandQueue_Wrapper*                            wrapper,
+        UINT                                                   num_lists,
+        ID3D12CommandList* const*                              lists);
 
-    void PostProcess_ID3D12CommandQueue_ExecuteCommandLists(ID3D12CommandQueue_Wrapper* wrapper,
-                                                            UINT                        num_lists,
-                                                            ID3D12CommandList* const*   lists);
+    void PostProcess_ID3D12CommandQueue_ExecuteCommandLists(
+        std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+        ID3D12CommandQueue_Wrapper*                            wrapper,
+        UINT                                                   num_lists,
+        ID3D12CommandList* const*                              lists);
+
+    void
+    OverrideID3D12CommandQueue_ExecuteCommandLists(std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+                                                   ID3D12CommandQueue_Wrapper*                            wrapper,
+                                                   UINT                                                   num_lists,
+                                                   ID3D12CommandList* const*                              lists);
 
     void PreProcess_D3D12CreateDevice(IUnknown*         pAdapter,
                                       D3D_FEATURE_LEVEL MinimumFeatureLevel,
@@ -476,6 +497,11 @@ class D3D12CaptureManager : public ApiCaptureManager
                                                                UINT                          num_barriers,
                                                                const D3D12_RESOURCE_BARRIER* barriers);
 
+    void PostProcess_ID3D12GraphicsCommandList_Reset(ID3D12CommandList_Wrapper* list_wrapper,
+                                                     HRESULT                    result,
+                                                     ID3D12CommandAllocator*    pAllocator,
+                                                     ID3D12PipelineState*       pInitialState);
+
     void PostProcess_ID3D12GraphicsCommandList4_BuildRaytracingAccelerationStructure(
         ID3D12GraphicsCommandList4_Wrapper*                                list_wrapper,
         const D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_DESC*          desc,
@@ -601,6 +627,10 @@ class D3D12CaptureManager : public ApiCaptureManager
                                                         ID3D12Pageable* const*          ppObjects,
                                                         const D3D12_RESIDENCY_PRIORITY* pPriorities);
 
+    HRESULT OverrideD3D12SerializeVersionedRootSignature(const D3D12_VERSIONED_ROOT_SIGNATURE_DESC* pRootSignature,
+                                                         ID3DBlob**                                 ppBlob,
+                                                         ID3DBlob**                                 ppErrorBlob);
+
     D3D12_CPU_DESCRIPTOR_HANDLE
     OverrideID3D12DescriptorHeap_GetCPUDescriptorHandleForHeapStart(ID3D12DescriptorHeap_Wrapper* wrapper);
 
@@ -713,6 +743,32 @@ class D3D12CaptureManager : public ApiCaptureManager
         const D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_INPUTS* pDesc,
         D3D12_RAYTRACING_ACCELERATION_STRUCTURE_PREBUILD_INFO*      pInfo);
 
+    HRESULT OverrideID3D12GraphicsCommandList_Reset(ID3D12GraphicsCommandList_Wrapper* wrapper,
+                                                    ID3D12CommandAllocator*            pAllocator,
+                                                    ID3D12PipelineState*               pInitialState);
+
+    void OverrideID3D12GraphicsCommandList_ExecuteBundle(ID3D12GraphicsCommandList_Wrapper* wrapper,
+                                                         ID3D12GraphicsCommandList*         pCommandList);
+
+    void OverrideID3D12GraphicsCommandList4_BeginRenderPass(ID3D12GraphicsCommandList4_Wrapper* wrapper,
+                                                            UINT                                NumRenderTargets,
+                                                            const D3D12_RENDER_PASS_RENDER_TARGET_DESC* pRenderTargets,
+                                                            const D3D12_RENDER_PASS_DEPTH_STENCIL_DESC* pDepthStencil,
+                                                            D3D12_RENDER_PASS_FLAGS                     Flags);
+
+    HRESULT OverrideD3D12CreateVersionedRootSignatureDeserializerFromSubobjectInLibrary(
+        LPCVOID pSrcData,
+        SIZE_T  SrcDataSizeInBytes,
+        LPCWSTR RootSignatureSubobjectName,
+        REFIID  pRootSignatureDeserializerInterface,
+        void**  ppRootSignatureDeserializer);
+
+    void OverrideID3D12GraphicsCommandList10_SetProgram(ID3D12GraphicsCommandList10_Wrapper* wrapper,
+                                                        const D3D12_SET_PROGRAM_DESC*        pDesc);
+
+    void OverrideID3D12GraphicsCommandList10_DispatchGraph(ID3D12GraphicsCommandList10_Wrapper* wrapper,
+                                                           const D3D12_DISPATCH_GRAPH_DESC*     pDesc);
+
     virtual CaptureSettings::TraceSettings GetDefaultTraceSettings();
 
     inline format::HandleId GetEnableDebugLayerObjectId() { return track_enable_debug_layer_object_id_; }
@@ -760,6 +816,49 @@ class D3D12CaptureManager : public ApiCaptureManager
 
     int GetAgsVersion() { return ags_version_; }
 
+    void TrimDrawCalls_D3D12SerializeVersionedRootSignature(const D3D12_VERSIONED_ROOT_SIGNATURE_DESC* pRootSignature,
+                                                            ID3DBlob**                                 ppBlob,
+                                                            ID3DBlob**                                 ppErrorBlob);
+
+    void TrimDrawCalls_ID3D12GraphicsCommandList_Reset(HRESULT                            replay_result,
+                                                       ID3D12GraphicsCommandList_Wrapper* wrapper,
+                                                       ID3D12CommandAllocator*            pAllocator,
+                                                       ID3D12PipelineState*               pInitialState);
+
+    void TrimDrawCalls_ID3D12GraphicsCommandList_ExecuteBundle(ID3D12GraphicsCommandList_Wrapper* wrapper,
+                                                               ID3D12GraphicsCommandList*         pCommandList);
+    void
+    TrimDrawCalls_ID3D12GraphicsCommandList4_BeginRenderPass(ID3D12GraphicsCommandList4_Wrapper* wrapper,
+                                                             UINT                                NumRenderTargets,
+                                                             const D3D12_RENDER_PASS_RENDER_TARGET_DESC* pRenderTargets,
+                                                             const D3D12_RENDER_PASS_DEPTH_STENCIL_DESC* pDepthStencil,
+                                                             D3D12_RENDER_PASS_FLAGS                     Flags);
+
+    bool TrimDrawCalls_ID3D12CommandQueue_ExecuteCommandLists(
+        std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+        ID3D12CommandQueue_Wrapper*                            wrapper,
+        UINT                                                   num_lists,
+        ID3D12CommandList* const*                              lists);
+
+    bool HasSplitCommandLists(UINT num_lists, ID3D12CommandList* const* lists);
+
+    std::vector<graphics::dx12::CommandSet> GetCommandListsForTrimDrawCalls(ID3D12CommandList_Wrapper* wrapper,
+                                                                            format::ApiCallId          api_call_id);
+
+    void PostProcess_IDXGISwapChain3_SetColorSpace1(IDXGISwapChain_Wrapper* wrapper,
+                                                    HRESULT                 result,
+                                                    DXGI_COLOR_SPACE_TYPE   ColorSpace);
+
+    void PostProcess_IDXGISwapChain4_SetHDRMetaData(
+        IDXGISwapChain_Wrapper* wrapper, HRESULT result, DXGI_HDR_METADATA_TYPE Type, UINT Size, void* pMetaData);
+
+    void PostProcess_SetName(IUnknown_Wrapper* wrapper, HRESULT result, LPCWSTR Name);
+
+    void PostProcess_InitializeMetaCommand(ID3D12GraphicsCommandList4_Wrapper* wrapper,
+                                           ID3D12MetaCommand*                  pMetaCommand,
+                                           const void*                         pInitializationParametersData,
+                                           SIZE_T                              InitializationParametersDataSizeInBytes);
+
   protected:
     D3D12CaptureManager();
 
@@ -769,7 +868,23 @@ class D3D12CaptureManager : public ApiCaptureManager
 
     virtual void DestroyStateTracker() override { state_tracker_ = nullptr; }
 
-    virtual void WriteTrackedState(util::FileOutputStream* file_stream, format::ThreadId thread_id) override;
+    virtual void WriteTrackedState(util::FileOutputStream* file_stream, util::ThreadData* thread_data) override;
+
+    virtual void WriteTrackedStateWithAssetFile(util::FileOutputStream* file_stream,
+                                                util::ThreadData*       thread_data,
+                                                util::FileOutputStream* asset_file_stream,
+                                                const std::string*      asset_file_name) override
+    {
+        GFXRECON_UNREFERENCED_PARAMETER(file_stream);
+        GFXRECON_UNREFERENCED_PARAMETER(thread_data);
+        GFXRECON_UNREFERENCED_PARAMETER(asset_file_stream);
+        GFXRECON_UNREFERENCED_PARAMETER(asset_file_name);
+    }
+
+    virtual void WriteAssets(util::FileOutputStream* assert_file_stream,
+                             const std::string*      asset_file_name,
+                             util::ThreadData*       thread_data) override
+    {}
 
     void PreAcquireSwapChainImages(IDXGISwapChain_Wrapper* wrapper,
                                    IUnknown*               command_queue,
@@ -793,7 +908,8 @@ class D3D12CaptureManager : public ApiCaptureManager
                                       ID3D12Heap_Wrapper*      heap_wrapper,
                                       uint64_t                 heap_offset);
 
-    void InitializeSwapChainBufferResourceInfo(ID3D12Resource_Wrapper* resource_wrapper,
+    void InitializeSwapChainBufferResourceInfo(IDXGISwapChain_Wrapper* wrapper,
+                                               ID3D12Resource_Wrapper* resource_wrapper,
                                                D3D12_RESOURCE_STATES   initial_state);
 
     void InitializeID3D12DeviceInfo(IUnknown* pAdapter, void** device);
@@ -812,8 +928,11 @@ class D3D12CaptureManager : public ApiCaptureManager
     void                          EnableDRED();
     bool                          RvAnnotationActive();
 
-    void                              PrePresent(IDXGISwapChain_Wrapper* wrapper);
-    void                              PostPresent(IDXGISwapChain_Wrapper* wrapper, UINT flags);
+    void PrePresent(IDXGISwapChain_Wrapper* wrapper);
+    void PostPresent(std::shared_lock<CommonCaptureManager::ApiCallMutexT>& current_lock,
+                     IDXGISwapChain_Wrapper*                                wrapper,
+                     UINT                                                   flags);
+
     static D3D12CaptureManager*       singleton_;
     std::set<ID3D12Resource_Wrapper*> mapped_resources_; ///< Track mapped resources for unassisted tracking mode.
     DxgiDispatchTable  dxgi_dispatch_table_;  ///< DXGI dispatch table for functions retrieved from the DXGI DLL.
