@@ -1,5 +1,6 @@
 /*
 ** Copyright (c) 2019 LunarG, Inc.
+** Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -25,11 +26,13 @@
 
 #include "encode/handle_unwrap_memory.h"
 #include "encode/vulkan_handle_wrappers.h"
+#include "scoped_destroy_lock.h"
 #include "format/format.h"
 #include "format/format_util.h"
 #include "generated/generated_vulkan_dispatch_table.h"
 #include "generated/generated_vulkan_state_table.h"
 #include "util/defines.h"
+#include "graphics/vulkan_util.h"
 
 #include <algorithm>
 #include <iterator>
@@ -38,12 +41,7 @@
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(encode)
-
-#if VK_USE_64_BIT_PTR_DEFINES == 1
-#define UINT64_TO_VK_HANDLE(handle_type, value) reinterpret_cast<handle_type>(value)
-#else
-#define UINT64_TO_VK_HANDLE(handle_type, value) static_cast<handle_type>(value)
-#endif
+GFXRECON_BEGIN_NAMESPACE(vulkan_wrappers)
 
 // Temporary resource IDs for state processing.
 static const format::HandleId kTempQueueId = std::numeric_limits<format::HandleId>::max() - 1;
@@ -51,7 +49,6 @@ static const VkCommandPool    kTempCommandPool =
     UINT64_TO_VK_HANDLE(VkCommandPool, std::numeric_limits<uint64_t>::max() - 2);
 static const format::HandleId kTempCommandPoolId   = std::numeric_limits<format::HandleId>::max() - 2;
 static const format::HandleId kTempCommandBufferId = std::numeric_limits<format::HandleId>::max() - 3;
-
 typedef format::HandleId (*PFN_GetHandleId)();
 
 extern VulkanStateHandleTable state_handle_table_;
@@ -73,7 +70,7 @@ inline format::HandleId GetTempWrapperId<CommandPoolWrapper>(const VkCommandPool
 }
 
 template <typename Wrapper>
-format::HandleId GetWrappedId(const typename Wrapper::HandleType& handle)
+format::HandleId GetWrappedId(const typename Wrapper::HandleType& handle, bool log_warning = true)
 {
     if (handle == VK_NULL_HANDLE)
     {
@@ -88,25 +85,33 @@ format::HandleId GetWrappedId(const typename Wrapper::HandleType& handle)
     auto wrapper = state_handle_table_.GetWrapper<Wrapper>(handle);
     if (wrapper == nullptr)
     {
-        GFXRECON_LOG_WARNING("GetWrappedId() couldn't find Handle: %" PRIu64 "'s wrapper. It might have been destroyed",
-                             handle);
+        if (log_warning)
+        {
+            GFXRECON_LOG_WARNING("vulkan_wrappers::GetWrappedId() couldn't find Handle: 0x%" PRIx64
+                                 "'s wrapper. It might have been destroyed",
+                                 handle);
+        }
         return format::kNullHandleId;
     }
     return wrapper->handle_id;
 }
 
 template <typename Wrapper>
-Wrapper* GetWrapper(const typename Wrapper::HandleType& handle)
+Wrapper* GetWrapper(const typename Wrapper::HandleType& handle, bool log_warning = true)
 {
     if (handle == VK_NULL_HANDLE)
     {
-        return 0;
+        return nullptr;
     }
     auto wrapper = state_handle_table_.GetWrapper<Wrapper>(handle);
     if (wrapper == nullptr)
     {
-        GFXRECON_LOG_WARNING("GetWrapper() couldn't find Handle: %" PRIu64 "'s wrapper. It might have been destroyed",
-                             handle);
+        if (log_warning)
+        {
+            GFXRECON_LOG_WARNING("vulkan_wrappers::GetWrapper() couldn't find Handle: 0x%" PRIx64
+                                 "'s wrapper. It might have been destroyed",
+                                 handle);
+        }
     }
     return wrapper;
 }
@@ -121,14 +126,14 @@ uint64_t GetWrappedId(uint64_t, VkObjectType object_type);
 
 uint64_t GetWrappedId(uint64_t object, VkDebugReportObjectTypeEXT object_type);
 
-inline const InstanceTable* GetInstanceTable(VkInstance handle)
+inline const graphics::VulkanInstanceTable* GetInstanceTable(VkInstance handle)
 {
     assert(handle != VK_NULL_HANDLE);
     auto wrapper = GetWrapper<InstanceWrapper>(handle);
     return &wrapper->layer_table;
 }
 
-inline const InstanceTable* GetInstanceTable(VkPhysicalDevice handle)
+inline const graphics::VulkanInstanceTable* GetInstanceTable(VkPhysicalDevice handle)
 {
     assert(handle != VK_NULL_HANDLE);
     auto wrapper = GetWrapper<PhysicalDeviceWrapper>(handle);
@@ -136,14 +141,14 @@ inline const InstanceTable* GetInstanceTable(VkPhysicalDevice handle)
     return wrapper->layer_table_ref;
 }
 
-inline const DeviceTable* GetDeviceTable(VkDevice handle)
+inline const graphics::VulkanDeviceTable* GetDeviceTable(VkDevice handle)
 {
     assert(handle != VK_NULL_HANDLE);
     auto wrapper = GetWrapper<DeviceWrapper>(handle);
     return &wrapper->layer_table;
 }
 
-inline const DeviceTable* GetDeviceTable(VkQueue handle)
+inline const graphics::VulkanDeviceTable* GetDeviceTable(VkQueue handle)
 {
     assert(handle != VK_NULL_HANDLE);
     auto wrapper = GetWrapper<QueueWrapper>(handle);
@@ -151,7 +156,7 @@ inline const DeviceTable* GetDeviceTable(VkQueue handle)
     return wrapper->layer_table_ref;
 }
 
-inline const DeviceTable* GetDeviceTable(VkCommandBuffer handle)
+inline const graphics::VulkanDeviceTable* GetDeviceTable(VkCommandBuffer handle)
 {
     assert(handle != VK_NULL_HANDLE);
     auto wrapper = GetWrapper<CommandBufferWrapper>(handle);
@@ -170,6 +175,7 @@ void CreateWrappedDispatchHandle(typename ParentWrapper::HandleType parent,
                                  typename Wrapper::HandleType*      handle,
                                  PFN_GetHandleId                    get_id)
 {
+    ScopedDestroyLock shared_scoped_lock(true);
     assert(handle != nullptr);
     if ((*handle) != VK_NULL_HANDLE)
     {
@@ -189,9 +195,12 @@ void CreateWrappedDispatchHandle(typename ParentWrapper::HandleType parent,
         }
         if (!state_handle_table_.InsertWrapper(wrapper))
         {
-            GFXRECON_LOG_WARNING("Create a duplicated Handle: %" PRIu64
-                                 ". This wrapper can't be written into VulkanStateHandleTable.",
-                                 *handle);
+            auto existing_wrapper = state_handle_table_.GetWrapper<Wrapper>(wrapper->handle);
+            GFXRECON_LOG_WARNING("Cannot add duplicate entry to VulkanStateHandleTable for handle 0x%" PRIx64
+                                 " with ID %" PRIu64 ". This handle is already wrapped with ID %" PRIu64 ".",
+                                 wrapper->handle,
+                                 wrapper->handle_id,
+                                 existing_wrapper->handle_id);
         }
     }
 }
@@ -199,6 +208,7 @@ void CreateWrappedDispatchHandle(typename ParentWrapper::HandleType parent,
 template <typename Wrapper>
 void CreateWrappedNonDispatchHandle(typename Wrapper::HandleType* handle, PFN_GetHandleId get_id)
 {
+    ScopedDestroyLock shared_scoped_lock(false);
     assert(handle != nullptr);
     if ((*handle) != VK_NULL_HANDLE)
     {
@@ -207,9 +217,12 @@ void CreateWrappedNonDispatchHandle(typename Wrapper::HandleType* handle, PFN_Ge
         wrapper->handle_id = get_id();
         if (!state_handle_table_.InsertWrapper(wrapper))
         {
-            GFXRECON_LOG_WARNING("Create a duplicated Handle: %" PRIu64
-                                 ". This wrapper can't be written into VulkanStateHandleTable.",
-                                 *handle);
+            auto existing_wrapper = state_handle_table_.GetWrapper<Wrapper>(wrapper->handle);
+            GFXRECON_LOG_WARNING("Cannot add duplicate entry to VulkanStateHandleTable for handle 0x%" PRIx64
+                                 " with ID %" PRIu64 ". This handle is already wrapped with ID %" PRIu64 ".",
+                                 wrapper->handle,
+                                 wrapper->handle_id,
+                                 existing_wrapper->handle_id);
         }
     }
 }
@@ -267,15 +280,27 @@ inline void CreateWrappedHandle<InstanceWrapper, NoParentWrapper, PhysicalDevice
 }
 
 template <>
-inline void CreateWrappedHandle<PhysicalDeviceWrapper, NoParentWrapper, DeviceWrapper>(
-    VkPhysicalDevice,            // Unused for device creation.
-    NoParentWrapper::HandleType, // VkDevice does not have a co-parent.
-    VkDevice*       handle,
-    PFN_GetHandleId get_id)
+inline void CreateWrappedHandle<PhysicalDeviceWrapper,
+                                NoParentWrapper,
+                                DeviceWrapper>(VkPhysicalDevice,            // Unused for device creation.
+                                               NoParentWrapper::HandleType, // VkDevice does not have a co-parent.
+                                               VkDevice*       handle,
+                                               PFN_GetHandleId get_id)
 {
     CreateWrappedDispatchHandle<PhysicalDeviceWrapper, DeviceWrapper>(VK_NULL_HANDLE, handle, get_id);
 }
 
+template <>
+inline void CreateWrappedHandle<DeviceWrapper, NoParentWrapper, DeviceMemoryWrapper>(VkDevice device,
+                                                                                     NoParentWrapper::HandleType,
+                                                                                     VkDeviceMemory* handle,
+                                                                                     PFN_GetHandleId get_id)
+{
+    CreateWrappedNonDispatchHandle<DeviceMemoryWrapper>(handle, get_id);
+    auto memory_wrapper           = GetWrapper<DeviceMemoryWrapper>(*handle);
+    memory_wrapper->parent_device = GetWrapper<DeviceWrapper>(device);
+}
+
 template <>
 inline void CreateWrappedHandle<DeviceWrapper, NoParentWrapper, QueueWrapper>(
     VkDevice parent,
@@ -309,6 +334,17 @@ inline void CreateWrappedHandle<DeviceWrapper, NoParentWrapper, QueueWrapper>(
     }
 }
 
+template <>
+inline void CreateWrappedHandle<DeviceWrapper, NoParentWrapper, CommandPoolWrapper>(VkDevice device,
+                                                                                    NoParentWrapper::HandleType,
+                                                                                    VkCommandPool*  handle,
+                                                                                    PFN_GetHandleId get_id)
+{
+    CreateWrappedNonDispatchHandle<CommandPoolWrapper>(handle, get_id);
+    auto pool_wrapper    = GetWrapper<CommandPoolWrapper>(*handle);
+    pool_wrapper->device = GetWrapper<DeviceWrapper>(device);
+}
+
 template <>
 inline void CreateWrappedHandle<DeviceWrapper, CommandPoolWrapper, CommandBufferWrapper>(VkDevice         parent,
                                                                                          VkCommandPool    co_parent,
@@ -332,11 +368,12 @@ inline void CreateWrappedHandle<DeviceWrapper, CommandPoolWrapper, CommandBuffer
 }
 
 template <>
-inline void CreateWrappedHandle<DeviceWrapper, DescriptorPoolWrapper, DescriptorSetWrapper>(
-    VkDevice, // Unused for descriptor set creation.
-    VkDescriptorPool co_parent,
-    VkDescriptorSet* handle,
-    PFN_GetHandleId  get_id)
+inline void CreateWrappedHandle<DeviceWrapper,
+                                DescriptorPoolWrapper,
+                                DescriptorSetWrapper>(VkDevice, // Unused for descriptor set creation.
+                                                      VkDescriptorPool co_parent,
+                                                      VkDescriptorSet* handle,
+                                                      PFN_GetHandleId  get_id)
 {
     assert(co_parent != VK_NULL_HANDLE);
     assert(handle != nullptr);
@@ -393,18 +430,21 @@ inline void CreateWrappedHandle<PhysicalDeviceWrapper, NoParentWrapper, DisplayK
 // Override for images retrieved from a swapchain, which requires the handle wrapper to be owned by a parent to ensure
 // the wrapper memory is released when the parent is destroyed.
 template <>
-inline void
-CreateWrappedHandle<DeviceWrapper, SwapchainKHRWrapper, ImageWrapper>(VkDevice, // Unused for swapchain image retrieval.
-                                                                      VkSwapchainKHR  co_parent,
-                                                                      VkImage*        handle,
-                                                                      PFN_GetHandleId get_id)
+inline void CreateWrappedHandle<DeviceWrapper,
+                                SwapchainKHRWrapper,
+                                ImageWrapper>(VkDevice, // Unused for swapchain image retrieval.
+                                              VkSwapchainKHR  co_parent,
+                                              VkImage*        handle,
+                                              PFN_GetHandleId get_id)
 {
     assert(co_parent != VK_NULL_HANDLE);
     assert(handle != nullptr);
 
     auto parent_wrapper = GetWrapper<SwapchainKHRWrapper>(co_parent);
 
-    // Filter duplicate display retrieval.
+    GFXRECON_ASSERT(!parent_wrapper->retired);
+
+    // Filter duplicate image retrieval.
     ImageWrapper* wrapper = nullptr;
     for (auto entry : parent_wrapper->child_images)
     {
@@ -427,11 +467,12 @@ CreateWrappedHandle<DeviceWrapper, SwapchainKHRWrapper, ImageWrapper>(VkDevice,
 // Override for display mode creation/retrieval, which requires the handle wrapper to be owned by a parent to ensure
 // the wrapper memory is released when the parent is destroyed.
 template <>
-inline void CreateWrappedHandle<PhysicalDeviceWrapper, DisplayKHRWrapper, DisplayModeKHRWrapper>(
-    VkPhysicalDevice, // Unused for display mode creation.
-    VkDisplayKHR      co_parent,
-    VkDisplayModeKHR* handle,
-    PFN_GetHandleId   get_id)
+inline void CreateWrappedHandle<PhysicalDeviceWrapper,
+                                DisplayKHRWrapper,
+                                DisplayModeKHRWrapper>(VkPhysicalDevice, // Unused for display mode creation.
+                                                       VkDisplayKHR      co_parent,
+                                                       VkDisplayModeKHR* handle,
+                                                       PFN_GetHandleId   get_id)
 {
     assert(co_parent != VK_NULL_HANDLE);
     assert(handle != nullptr);
@@ -645,6 +686,7 @@ inline void ResetDescriptorPoolWrapper(VkDescriptorPool handle)
     wrapper->child_sets.clear();
 }
 
+GFXRECON_END_NAMESPACE(vulkan_wrappers)
 GFXRECON_END_NAMESPACE(encode)
 GFXRECON_END_NAMESPACE(gfxrecon)
 
