@@ -1,7 +1,7 @@
 /*
 ** Copyright (c) 2018-2022 Valve Corporation
 ** Copyright (c) 2018-2022 LunarG, Inc.
-** Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
+** Copyright (c) 2023-2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -54,6 +54,7 @@ typedef uint64_t AddressEncodeType;
 typedef uint8_t  CharEncodeType;  // Encoding type for UTF-8 strings.
 typedef uint16_t WCharEncodeType; // Encoding type for LPCWSTR (UTF-16) strings.
 typedef uint32_t FormatEncodeType;
+typedef uint32_t D3D_FEATURE_LEVELEncodeType;
 
 typedef HandleEncodeType HandleId;
 typedef uint64_t         ThreadId;
@@ -70,6 +71,7 @@ const int8_t   kNoneIndex                 = -1;
 const char* const kAnnotationLabelOperation          = "operation";
 const char* const kAnnotationLabelReplayOptions      = "replayopts";
 const char* const kAnnotationLabelRemovedResource    = "removed-resource";
+const char* const kAnnotationLabelTransformer        = "transformer";
 const char* const kAnnotationPipelineCreationAttempt = "pipelinecreationattempt";
 
 const char* const kOperationAnnotationGfxreconstructVersion = "gfxrecon-version";
@@ -121,38 +123,43 @@ enum AdapterType
 
 enum class MetaDataType : uint16_t
 {
-    kUnknownMetaDataType                    = 0,
-    kDisplayMessageCommand                  = 1,
-    kFillMemoryCommand                      = 2,
-    kResizeWindowCommand                    = 3,
-    kSetSwapchainImageStateCommand          = 4,
-    kBeginResourceInitCommand               = 5,
-    kEndResourceInitCommand                 = 6,
-    kInitBufferCommand                      = 7,
-    kInitImageCommand                       = 8,
-    kCreateHardwareBufferCommand_deprecated = 9,
-    kDestroyHardwareBufferCommand           = 10,
-    kSetDevicePropertiesCommand             = 11,
-    kSetDeviceMemoryPropertiesCommand       = 12,
-    kResizeWindowCommand2                   = 13,
-    kSetOpaqueAddressCommand                = 14,
-    kSetRayTracingShaderGroupHandlesCommand = 15,
-    kCreateHeapAllocationCommand            = 16,
-    kInitSubresourceCommand                 = 17,
-    kExeFileInfoCommand                     = 18,
-    kInitDx12AccelerationStructureCommand   = 19,
-    kFillMemoryResourceValueCommand         = 20,
-    kDxgiAdapterInfoCommand                 = 21,
-    kDriverInfoCommand                      = 22,
-    kReserved23                             = 23,
-    kCreateHardwareBufferCommand            = 24,
-    kReserved25                             = 25,
-    kDx12RuntimeInfoCommand                 = 26,
-    kParentToChildDependency                = 27,
-    kReserved28                             = 28,
-    kReserved29                             = 29,
-    kReserved30                             = 30,
-    kReserved31                             = 31,
+    kUnknownMetaDataType                                = 0,
+    kDisplayMessageCommand                              = 1,
+    kFillMemoryCommand                                  = 2,
+    kResizeWindowCommand                                = 3,
+    kSetSwapchainImageStateCommand                      = 4,
+    kBeginResourceInitCommand                           = 5,
+    kEndResourceInitCommand                             = 6,
+    kInitBufferCommand                                  = 7,
+    kInitImageCommand                                   = 8,
+    kCreateHardwareBufferCommand_deprecated             = 9,
+    kDestroyHardwareBufferCommand                       = 10,
+    kSetDevicePropertiesCommand                         = 11,
+    kSetDeviceMemoryPropertiesCommand                   = 12,
+    kResizeWindowCommand2                               = 13,
+    kSetOpaqueAddressCommand                            = 14,
+    kSetRayTracingShaderGroupHandlesCommand             = 15,
+    kCreateHeapAllocationCommand                        = 16,
+    kInitSubresourceCommand                             = 17,
+    kExeFileInfoCommand                                 = 18,
+    kInitDx12AccelerationStructureCommand               = 19,
+    kFillMemoryResourceValueCommand                     = 20,
+    kDxgiAdapterInfoCommand                             = 21,
+    kDriverInfoCommand                                  = 22,
+    kReserved23                                         = 23,
+    kCreateHardwareBufferCommand_deprecated2            = 24,
+    kReserved25                                         = 25,
+    kDx12RuntimeInfoCommand                             = 26,
+    kParentToChildDependency                            = 27,
+    kVulkanBuildAccelerationStructuresCommand           = 28,
+    kVulkanCopyAccelerationStructuresCommand            = 29,
+    kVulkanWriteAccelerationStructuresPropertiesCommand = 30,
+    kReserved31                                         = 31,
+    kSetEnvironmentVariablesCommand                     = 32,
+    kViewRelativeLocation                               = 33,
+    kExecuteBlocksFromFile                              = 34,
+    kCreateHardwareBufferCommand                        = 35,
+    kInitializeMetaCommand                              = 36,
 };
 
 // MetaDataId is stored in the capture file and its type must be uint32_t to avoid breaking capture file compatibility.
@@ -225,12 +232,13 @@ struct EnabledOptions
 // Resource values are values contained in resource data that may require special handling (e.g., mapping for replay).
 enum class ResourceValueType : uint8_t
 {
-    kUnknown                      = 0,
-    kGpuVirtualAddress            = 1,
-    kGpuDescriptorHandle          = 2,
-    kShaderIdentifier             = 3,
-    kIndirectArgumentDispatchRays = 4,
-    kExecuteIndirectCountBuffer   = 5
+    kUnknown                       = 0,
+    kGpuVirtualAddress             = 1,
+    kGpuDescriptorHandle           = 2,
+    kShaderIdentifier              = 3,
+    kIndirectArgumentDispatchRays  = 4,
+    kExecuteIndirectCountBuffer    = 5,
+    kRaytracingInstanceDescPointer = 6,
 };
 
 #pragma pack(push)
@@ -400,10 +408,27 @@ struct CreateHardwareBufferCommandHeader_deprecated
                            // HardwareBufferLayerInfo records.  When unavailable, 'planes' is zero.
 };
 
+struct CreateHardwareBufferCommandHeader_deprecated2
+{
+    MetaDataHeader meta_header;
+    ThreadId       thread_id;
+    HandleId       memory_id; // Globally unique ID assigned to the buffer for tracking memory modifications.
+    uint64_t       buffer_id; // Address of the buffer object.
+    uint32_t       format;
+    uint32_t       width;
+    uint32_t       height;
+    uint32_t       stride; // Size of a row in pixels.
+    uint64_t       usage;
+    uint32_t       layers;
+    uint32_t       planes; // When additional multi-plane data is available, header is followed by 'planes' count
+                           // HardwareBufferLayerInfo records.  When unavailable, 'planes' is zero.
+};
+
 struct CreateHardwareBufferCommandHeader
 {
     MetaDataHeader meta_header;
     ThreadId       thread_id;
+    HandleId       device_id;
     HandleId       memory_id; // Globally unique ID assigned to the buffer for tracking memory modifications.
     uint64_t       buffer_id; // Address of the buffer object.
     uint32_t       format;
@@ -648,6 +673,90 @@ struct ParentToChildDependencyHeader
     uint32_t                    child_count;
 };
 
+static constexpr char kEnvironmentStringDelimeter = (char)-1;
+struct SetEnvironmentVariablesCommand
+{
+    MetaDataHeader meta_header;
+    ThreadId       thread_id;
+    uint64_t       string_length;
+
+    // In the capture file, a string will immediately follow this block
+    // containing a list of environment variables and their values
+};
+
+struct VulkanMetaBuildAccelerationStructuresHeader
+{
+    format::MetaDataHeader meta_header;
+};
+
+struct VulkanWriteAccelerationStructuresPropertiesCommandHeader
+{
+    format::MetaDataHeader meta_header;
+};
+
+struct VulkanCopyAccelerationStructuresCommandHeader
+{
+    format::MetaDataHeader meta_header;
+};
+
+struct ExecuteBlocksFromFile
+{
+    MetaDataHeader   meta_header;
+    format::ThreadId thread_id;
+
+    // Number of commands to execute from file.
+    // 0 means execute till the end of file.
+    uint32_t n_blocks;
+
+    // The offset from the start of the file to start executing
+    int64_t offset;
+
+    // Number of characters in file name
+    uint32_t filename_length;
+};
+
+struct ViewRelativeLocation
+{
+    format::HandleId session_id;
+    format::HandleId space_id;
+
+    // Locate status
+    uint64_t flags;
+
+    // Orientation
+    float qx;
+    float qy;
+    float qz;
+    float qw;
+
+    // Position
+    float x;
+    float y;
+    float z;
+};
+
+struct ViewRelativeLocationCmd
+{
+    MetaDataHeader   meta_header;
+    format::ThreadId thread_id;
+
+    ViewRelativeLocation location;
+};
+
+struct InitializeMetaCommand
+{
+    MetaDataHeader   meta_header{};
+    ThreadId         thread_id;
+    format::HandleId capture_id;
+    uint32_t         block_index{ 0 };
+    uint32_t         total_number_of_initializemetacommand{ 0 };
+    uint64_t         initialization_parameters_data_size{ 0 };
+
+    // In the capture file, initialize metacommand data is written in the following order:
+    // InitializeMetaCommandHeder
+    // parameters data
+};
+
 // Restore size_t to normal behavior.
 #undef size_t
 
