@@ -23,6 +23,7 @@
 #include "spirv_parsing_util.h"
 #include <functional>
 #include <optional>
+#include <deque>
 #include "spirv_reflect.h"
 #include "util/spirv_helper.h"
 #include "util/logging.h"
@@ -33,8 +34,8 @@ GFXRECON_BEGIN_NAMESPACE(util)
 // used to enable type as key for std::set/map
 bool operator<(const SpirVParsingUtil::BufferReferenceInfo& lhs, const SpirVParsingUtil::BufferReferenceInfo& rhs)
 {
-    return std::make_tuple(lhs.set, lhs.binding, lhs.push_constant_block, lhs.buffer_offset, lhs.array_stride) <
-           std::make_tuple(rhs.set, rhs.binding, rhs.push_constant_block, rhs.buffer_offset, rhs.array_stride);
+    return std::make_tuple(lhs.source, lhs.set, lhs.binding, lhs.buffer_offset, lhs.array_stride) <
+           std::make_tuple(rhs.source, rhs.set, rhs.binding, rhs.buffer_offset, rhs.array_stride);
 }
 
 // Instruction represents a single Spv::Op instruction.
@@ -134,10 +135,28 @@ bool SpirVParsingUtil::GetVariableDecorations(const Instruction*   variable_insn
     const uint32_t variable_id   = variable_insn->resultId();
     const uint32_t storage_class = variable_insn->operand(0);
 
-    if (storage_class == spv::StorageClassPushConstant)
+    switch (storage_class)
     {
-        buffer_reference_info.push_constant_block = true;
-        return true;
+
+        case spv::StorageClassUniform:
+            buffer_reference_info.source = BufferReferenceLocation::UNIFORM_BUFFER;
+            break;
+
+        case spv::StorageClassStorageBuffer:
+            buffer_reference_info.source = BufferReferenceLocation::STORAGE_BUFFER;
+            break;
+
+        case spv::StorageClassShaderRecordBufferKHR:
+            buffer_reference_info.source = BufferReferenceLocation::SHADER_RECORD_BUFFER;
+            break;
+
+        case spv::StorageClassPushConstant:
+            buffer_reference_info.source = BufferReferenceLocation::PUSH_CONSTANT_BLOCK;
+            return true;
+
+        default:
+            GFXRECON_LOG_WARNING_ONCE("Storage class %u not handled\n", storage_class);
+            return false;
     }
 
     for (const Instruction* insn : decorations_instructions_)
@@ -155,16 +174,7 @@ bool SpirVParsingUtil::GetVariableDecorations(const Instruction*   variable_insn
             buffer_reference_info.binding = insn->operand(2);
         }
     }
-
-    if (storage_class == spv::StorageClassStorageBuffer || storage_class == spv::StorageClassUniform)
-    {
-        return true;
-    }
-    else
-    {
-        printf("Storage class %u not handled\n", storage_class);
-        return false;
-    }
+    return true;
 }
 
 bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, size_t spirv_num_bytes)
@@ -192,12 +202,30 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
 
     std::vector<Instruction> instructions;
 
-    // First build up instructions object to make it easier to work with the SPIR-V
+    bool found_buffer_ref = false;
+
+    // build up instructions object to make it easier to work with the SPIR-V
+    // also checks for required capability
     while (spirv_ptr < spirv_end)
     {
         Instruction& insn = instructions.emplace_back(spirv_ptr);
         spirv_ptr += insn.length();
         GFXRECON_ASSERT(insn.length() > 0);
+
+        if (insn.opcode() == spv::OpCapability && insn.word(1) == spv::CapabilityPhysicalStorageBufferAddresses)
+        {
+            found_buffer_ref = true;
+        }
+
+        // arrived at 'OpFunction' -> we have seen all metadata incl. capabilities
+        if (insn.opcode() == spv::OpFunction)
+        {
+            // CapabilityPhysicalStorageBufferAddresses not found
+            if (!found_buffer_ref)
+            {
+                return true;
+            }
+        }
     }
     if (spirv_ptr != spirv_end)
     {
@@ -206,8 +234,143 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
     }
     instructions.shrink_to_fit();
 
-    auto track_back_instruction = [this, &spv_shader_module, spirv_code, spirv_num_bytes](
-                                      const Instruction* object_insn) {
+    if (spv_shader_module == std::nullopt)
+    {
+        // spirv-reflect parsing only on-demand
+        spv_shader_module = SpvReflectShaderModule();
+        spvReflectCreateShaderModule(spirv_num_bytes, spirv_code, &spv_shader_module.value());
+    }
+
+    // forward spirv-reflect-pass
+    {
+        // define a function to walk blocks breadth-first and check for buffer-references
+        auto check_buffer_references = [this](const SpvReflectTypeDescription* type,
+                                              BufferReferenceLocation          source,
+                                              uint32_t                         set,
+                                              uint32_t                         binding) {
+            struct queue_item_t
+            {
+                const SpvReflectTypeDescription* type_description = nullptr;
+                uint32_t                         offset           = 0;
+                uint32_t                         stride           = 0;
+                std::vector<std::string>         member_names;
+            };
+            // iterate bfs
+            std::deque<queue_item_t> queue = { { type } };
+
+            while (!queue.empty())
+            {
+                auto queue_item = std::move(queue.front());
+                queue.pop_front();
+                auto& [td, offset, stride, member_names] = queue_item;
+
+                if (td)
+                {
+                    member_names.emplace_back(td->struct_member_name ? td->struct_member_name : "unknown");
+
+                    if (td->op == SpvOpTypeArray || td->op == SpvOpTypeRuntimeArray)
+                    {
+                        stride = td->traits.array.stride;
+                    }
+
+                    // we pick up potential buffer-references here and confirm later.
+                    bool is_potential_ref = td->op == SpvOpTypeInt && td->traits.numeric.scalar.width == 64 &&
+                                            !td->traits.numeric.scalar.signedness;
+
+                    if (td->storage_class == spv::StorageClassPhysicalStorageBuffer || is_potential_ref)
+                    {
+                        BufferReferenceInfo ref_info;
+                        ref_info.source        = source;
+                        ref_info.set           = set;
+                        ref_info.binding       = binding;
+                        ref_info.buffer_offset = offset;
+                        ref_info.array_stride  = stride;
+
+                        // insert into map
+                        buffer_reference_map_[ref_info] = member_names;
+                    }
+
+                    for (uint32_t j = 0; j < td->member_count; ++j)
+                    {
+                        auto* member = td->members + j;
+                        queue.push_back({ member, offset, stride, member_names });
+
+                        uint32_t num_scalar_bytes = member->traits.numeric.scalar.width / 8;
+
+                        if (member->op == SpvOpTypeVector)
+                        {
+                            num_scalar_bytes *= member->traits.numeric.vector.component_count;
+                        }
+                        else if (member->op == SpvOpTypeMatrix)
+                        {
+                            num_scalar_bytes *= member->traits.numeric.matrix.column_count;
+                            num_scalar_bytes *= member->traits.numeric.matrix.row_count;
+                            num_scalar_bytes = std::max(num_scalar_bytes, member->traits.numeric.matrix.stride);
+                        }
+                        else if (member->op == SpvOpTypePointer || member->op == SpvOpTypeForwardPointer)
+                        {
+                            num_scalar_bytes = sizeof(uint64_t);
+                        }
+                        else if (member->op == SpvOpTypeArray || member->op == SpvOpTypeRuntimeArray)
+                        {
+                            num_scalar_bytes = std::max(num_scalar_bytes, member->traits.array.stride);
+                            for (uint32_t d = 0; d < member->traits.array.dims_count; ++d)
+                            {
+                                num_scalar_bytes *= member->traits.array.dims[d] == spv::OpTypeRuntimeArray
+                                                        ? 1
+                                                        : member->traits.array.dims[d];
+                            }
+                        }
+                        offset += num_scalar_bytes;
+                    }
+                }
+            }
+        };
+
+        // check descriptor sets
+        uint32_t num_descriptor_set;
+        spvReflectEnumerateDescriptorSets(&*spv_shader_module, &num_descriptor_set, nullptr);
+        std::vector<SpvReflectDescriptorSet*> descriptor_sets(num_descriptor_set);
+        spvReflectEnumerateDescriptorSets(&*spv_shader_module, &num_descriptor_set, descriptor_sets.data());
+
+        for (const auto& descriptor_set : descriptor_sets)
+        {
+            for (uint32_t i = 0; i < descriptor_set->binding_count; ++i)
+            {
+                auto*                   binding = descriptor_set->bindings[i];
+                BufferReferenceLocation source  = BufferReferenceLocation::INVALID;
+
+                switch (binding->descriptor_type)
+                {
+                    case SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_BUFFER:
+                    case SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
+                        source = BufferReferenceLocation::UNIFORM_BUFFER;
+                        break;
+                    case SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_BUFFER:
+                    case SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC:
+                        source = BufferReferenceLocation::STORAGE_BUFFER;
+                        break;
+                    default:
+                        break;
+                }
+                check_buffer_references(binding->type_description, source, descriptor_set->set, binding->binding);
+            }
+        }
+
+        // check push-constants
+        uint32_t num_push_constant_blocks;
+        spvReflectEnumeratePushConstantBlocks(&*spv_shader_module, &num_push_constant_blocks, nullptr);
+        std::vector<SpvReflectBlockVariable*> push_constant_blocks(num_push_constant_blocks);
+        spvReflectEnumeratePushConstantBlocks(
+            &*spv_shader_module, &num_push_constant_blocks, push_constant_blocks.data());
+
+        for (const auto& block : push_constant_blocks)
+        {
+            check_buffer_references(block->type_description, BufferReferenceLocation::PUSH_CONSTANT_BLOCK, 0, 0);
+        }
+    } // forward spirv-reflect pass
+
+    auto track_back_instruction = [this, &spv_shader_module](const Instruction* object_insn) {
         // keep track of access-chain
         std::vector<uint32_t> access_indices;
 
@@ -216,6 +379,7 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
         {
             switch (object_insn->opcode())
             {
+                case spv::OpFunctionParameter:
                 case spv::OpConvertUToPtr:
                 case spv::OpCopyLogical:
                 case spv::OpLoad:
@@ -245,7 +409,8 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
                 case spv::OpVariable:
                 {
                     const uint32_t storage_class = object_insn->operand(0);
-                    if (storage_class == spv::StorageClassFunction)
+                    if (storage_class == spv::StorageClassFunction ||
+                        storage_class == spv::StorageClassShaderRecordBufferKHR)
                     {
                         // When casting to a struct, can get a 2nd function variable, just keep following
                         object_insn = FindVariableStoring(object_insn->resultId());
@@ -256,23 +421,17 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
 
                         if (GetVariableDecorations(object_insn, buffer_reference_info))
                         {
-                            if (spv_shader_module == std::nullopt)
-                            {
-                                // spirv-reflect parsing only on-demand
-                                spv_shader_module = SpvReflectShaderModule();
-                                spvReflectCreateShaderModule(spirv_num_bytes, spirv_code, &spv_shader_module.value());
-                            }
-
                             SpvReflectResult                 spv_result;
                             const SpvReflectTypeDescription* td = nullptr;
 
                             // access-chain starts with descriptor-binding root
                             std::string root_name;
 
-                            if (buffer_reference_info.push_constant_block)
+                            if (buffer_reference_info.source == BufferReferenceLocation::PUSH_CONSTANT_BLOCK)
                             {
                                 const SpvReflectBlockVariable* block = spvReflectGetEntryPointPushConstantBlock(
                                     &spv_shader_module.value(), spv_shader_module->entry_point_name, &spv_result);
+                                GFXRECON_ASSERT(spv_result == SPV_REFLECT_RESULT_SUCCESS);
                                 td = block->type_description;
                             }
                             else
@@ -282,8 +441,13 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
                                                                    buffer_reference_info.binding,
                                                                    buffer_reference_info.set,
                                                                    &spv_result);
-                                td        = spv_descriptor_binding->type_description;
-                                root_name = spv_descriptor_binding->name;
+                                GFXRECON_ASSERT(spv_result == SPV_REFLECT_RESULT_SUCCESS);
+                                td = spv_descriptor_binding->type_description;
+
+                                // spirv_reflect sets the name by tracking SPIR-V instructions like OpName. Some
+                                // optimizations may remove these instructions, resulting in a nullptr name. Actually we
+                                // can find some title removes all such names.
+                                root_name = spv_descriptor_binding->name ? spv_descriptor_binding->name : "";
                             }
 
                             if (root_name.empty())
@@ -322,14 +486,13 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
                                             num_scalar_bytes =
                                                 std::max(num_scalar_bytes, member.traits.numeric.matrix.stride);
                                         }
-                                        else if (member.op == SpvOpTypeForwardPointer)
+                                        else if (member.op == SpvOpTypePointer || member.op == SpvOpTypeForwardPointer)
                                         {
                                             num_scalar_bytes = sizeof(uint64_t);
                                         }
                                         else if (member.op == SpvOpTypeArray || member.op == SpvOpTypeRuntimeArray)
                                         {
                                             num_scalar_bytes = std::max(num_scalar_bytes, member.traits.array.stride);
-                                            assert(false); // not handled
                                         }
                                         buffer_reference_info.buffer_offset += num_scalar_bytes;
                                     }
@@ -379,7 +542,8 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
         }
     };
 
-    // Now we can walk the SPIR-V one more time to find what we need
+    // now we walk the SPIR-V one more time to find remaining occurances of buffer-references.
+    // that's e.g. cases when a uint64_t is casted. we can only tell by following back the dereferencing spv::OpLoad
     for (const Instruction& insn : instructions)
     {
         // because it is SSA, we can build this up as we are looping in this pass
@@ -443,7 +607,7 @@ bool SpirVParsingUtil::ParseBufferReferences(const uint32_t* const spirv_code, s
         name = name.substr(0, name.size() - 4);
 
         char buf[128];
-        if (buffer_reference_info.push_constant_block)
+        if (buffer_reference_info.source == BufferReferenceLocation::PUSH_CONSTANT_BLOCK)
         {
             snprintf(buf, sizeof(buf), "push-constant-block");
         }
@@ -480,5 +644,52 @@ std::vector<SpirVParsingUtil::BufferReferenceInfo> SpirVParsingUtil::GetBufferRe
     return ret;
 }
 
+static VkDescriptorType SpvReflectToVkDescriptorType(SpvReflectDescriptorType type)
+{
+    switch (type)
+    {
+        case SPV_REFLECT_DESCRIPTOR_TYPE_SAMPLER:
+            return VK_DESCRIPTOR_TYPE_SAMPLER;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER:
+            return VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_SAMPLED_IMAGE:
+            return VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_IMAGE:
+            return VK_DESCRIPTOR_TYPE_STORAGE_IMAGE;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
+            return VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
+            return VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_BUFFER:
+            return VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_BUFFER:
+            return VK_DESCRIPTOR_TYPE_STORAGE_BUFFER;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
+            return VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC:
+            return VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_INPUT_ATTACHMENT:
+            return VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT;
+
+        case SPV_REFLECT_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR:
+            return VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR;
+
+        default:
+            GFXRECON_LOG_WARNING("%s(): Unrecognised SPIRV-Reflect descriptor type");
+            assert(0);
+            return VK_DESCRIPTOR_TYPE_MAX_ENUM;
+    }
+}
+
 GFXRECON_END_NAMESPACE(util)
 GFXRECON_END_NAMESPACE(gfxrecon)
