@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 2018-2020 Valve Corporation
-** Copyright (c) 2018-2020 LunarG, Inc.
+** Copyright (c) 2018-2024 LunarG, Inc.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -26,15 +26,24 @@
 #include "application/android_context.h"
 #include "application/android_window.h"
 #include "decode/file_processor.h"
+#include "decode/preload_file_processor.h"
 #include "decode/vulkan_replay_options.h"
 #include "decode/vulkan_tracked_object_info_table.h"
+#include "decode/vulkan_pre_process_consumer.h"
 #include "format/format.h"
+
+#if ENABLE_OPENXR_SUPPORT
+#include "decode/openxr_tracked_object_info_table.h"
+#include "generated/generated_openxr_decoder.h"
+#include "generated/generated_openxr_replay_consumer.h"
+#endif
 #include "generated/generated_vulkan_decoder.h"
 #include "generated/generated_vulkan_replay_consumer.h"
 #include "util/argument_parser.h"
 #include "util/logging.h"
 #include "util/platform.h"
 #include "parse_dump_resources_cli.h"
+#include "replay_pre_processing.h"
 
 #include <android_native_app_glue.h>
 #include <android/log.h>
@@ -58,8 +67,24 @@ void        ProcessAppCmd(struct android_app* app, int32_t cmd);
 int32_t     ProcessInputEvent(struct android_app* app, AInputEvent* event);
 void        DestroyActivity(struct android_app* app);
 
+static std::unique_ptr<gfxrecon::decode::FileProcessor> file_processor;
+
+extern "C"
+{
+    uint64_t MainGetCurrentBlockIndex()
+    {
+        return file_processor->GetCurrentBlockIndex();
+    }
+
+    bool MainGetLoadingTrimmedState()
+    {
+        return file_processor->GetLoadingTrimmedState();
+    }
+}
+
 void android_main(struct android_app* app)
 {
+    GFXRECON_WRITE_CONSOLE("====== Entering android_main");
     gfxrecon::util::Log::Init();
 
     // Keep screen on while window is active.
@@ -101,56 +126,88 @@ void android_main(struct android_app* app)
 
         try
         {
-            gfxrecon::decode::FileProcessor file_processor;
+            file_processor = arg_parser.IsOptionSet(kPreloadMeasurementRangeOption)
+                                 ? std::make_unique<gfxrecon::decode::PreloadFileProcessor>()
+                                 : std::make_unique<gfxrecon::decode::FileProcessor>();
 
-            if (!file_processor.Initialize(filename))
+            if (!file_processor->Initialize(filename))
             {
                 GFXRECON_WRITE_CONSOLE("Failed to load file %s.", filename.c_str());
             }
             else
             {
                 auto application =
-                    std::make_shared<gfxrecon::application::Application>(kApplicationName, &file_processor);
+                    std::make_shared<gfxrecon::application::Application>(kApplicationName, file_processor.get());
                 application->InitializeWsiContext(VK_KHR_ANDROID_SURFACE_EXTENSION_NAME, app);
 
                 gfxrecon::decode::VulkanTrackedObjectInfoTable tracked_object_info_table;
                 gfxrecon::decode::VulkanReplayOptions          replay_options =
                     GetVulkanReplayOptions(arg_parser, filename, &tracked_object_info_table);
 
-                // Process --dump-resources arg. We do it here so that other gfxr tools that use
-                // the VulkanReplayOptions class won't have to link in the json library.
-                if (!gfxrecon::parse_dump_resources::parse_dump_resources_arg(replay_options))
+                gfxrecon::decode::VulkanReplayConsumer vulkan_replay_consumer(application, replay_options);
+                gfxrecon::decode::VulkanDecoder        vulkan_decoder;
+
+                ApiReplayOptions  api_replay_options;
+                ApiReplayConsumer api_replay_consumer;
+                api_replay_options.vk_replay_options   = &replay_options;
+                api_replay_consumer.vk_replay_consumer = &vulkan_replay_consumer;
+
+                if (IsRunPreProcessConsumer(api_replay_options))
                 {
-                    GFXRECON_LOG_FATAL("There was an error while parsing dump resource indices. Terminating");
-                    return;
+                    RunPreProcessConsumer(filename, api_replay_options, api_replay_consumer);
                 }
 
-                gfxrecon::decode::VulkanReplayConsumer replay_consumer(application, replay_options);
-                gfxrecon::decode::VulkanDecoder        decoder;
-                uint32_t                               start_frame, end_frame;
-                bool        has_mfr = GetMeasurementFrameRange(arg_parser, start_frame, end_frame);
+                uint32_t measurement_start_frame;
+                uint32_t measurement_end_frame;
+                bool     has_mfr = GetMeasurementFrameRange(arg_parser, measurement_start_frame, measurement_end_frame);
+
                 std::string measurement_file_name;
+                GetMeasurementFilename(arg_parser, measurement_file_name);
 
-                if (has_mfr)
+                bool     quit_after_frame = false;
+                uint32_t quit_frame;
+
+                if (replay_options.quit_after_frame)
                 {
-                    GetMeasurementFilename(arg_parser, measurement_file_name);
+                    quit_after_frame = true;
+                    GetQuitAfterFrame(arg_parser, quit_frame);
                 }
 
-                gfxrecon::graphics::FpsInfo fps_info(static_cast<uint64_t>(start_frame),
-                                                     static_cast<uint64_t>(end_frame),
+                gfxrecon::graphics::FpsInfo fps_info(static_cast<uint64_t>(measurement_start_frame),
+                                                     static_cast<uint64_t>(measurement_end_frame),
                                                      has_mfr,
                                                      replay_options.quit_after_measurement_frame_range,
                                                      replay_options.flush_measurement_frame_range,
                                                      replay_options.flush_inside_measurement_range,
-                                                     measurement_file_name);
+                                                     replay_options.preload_measurement_range,
+                                                     measurement_file_name,
+                                                     quit_after_frame,
+                                                     quit_frame);
 
-                replay_consumer.SetFatalErrorHandler([](const char* message) { throw std::runtime_error(message); });
-                replay_consumer.SetFpsInfo(&fps_info);
+                vulkan_replay_consumer.SetFatalErrorHandler(
+                    [](const char* message) { throw std::runtime_error(message); });
+                vulkan_replay_consumer.SetFpsInfo(&fps_info);
+
+                vulkan_decoder.AddConsumer(&vulkan_replay_consumer);
+
+                file_processor->AddDecoder(&vulkan_decoder);
+
+                file_processor->SetPrintBlockInfoFlag(replay_options.enable_print_block_info,
+                                                      replay_options.block_index_from,
+                                                      replay_options.block_index_to);
 
-                decoder.AddConsumer(&replay_consumer);
-                file_processor.AddDecoder(&decoder);
                 application->SetPauseFrame(GetPauseFrame(arg_parser));
 
+#if ENABLE_OPENXR_SUPPORT
+                gfxrecon::decode::OpenXrReplayOptions  openxr_replay_options = {};
+                gfxrecon::decode::OpenXrDecoder        openxr_decoder;
+                gfxrecon::decode::OpenXrReplayConsumer openxr_replay_consumer(application, openxr_replay_options);
+                openxr_replay_consumer.SetVulkanReplayConsumer(&vulkan_replay_consumer);
+                openxr_replay_consumer.SetAndroidApp(app);
+                openxr_decoder.AddConsumer(&openxr_replay_consumer);
+                file_processor->AddDecoder(&openxr_decoder);
+#endif
+
                 // Warn if the capture layer is active.
                 CheckActiveLayers(kLayerProperty);
 
@@ -166,25 +223,25 @@ void android_main(struct android_app* app)
                 application->Run();
 
                 // Add one so that it matches the trim range frame number semantic
-                fps_info.EndFile(file_processor.GetCurrentFrameNumber() + 1);
+                fps_info.EndFile(file_processor->GetCurrentFrameNumber() + 1);
 
-                if ((file_processor.GetCurrentFrameNumber() > 0) &&
-                    (file_processor.GetErrorState() == gfxrecon::decode::FileProcessor::kErrorNone))
+                if ((file_processor->GetCurrentFrameNumber() > 0) &&
+                    (file_processor->GetErrorState() == gfxrecon::decode::FileProcessor::kErrorNone))
                 {
-                    if (file_processor.GetCurrentFrameNumber() < start_frame)
+                    if (file_processor->GetCurrentFrameNumber() < measurement_start_frame)
                     {
                         GFXRECON_LOG_WARNING(
                             "Measurement range start frame (%u) is greater than the last replayed frame (%u). "
                             "Measurements were never started, cannot calculate measurement range FPS.",
-                            start_frame,
-                            file_processor.GetCurrentFrameNumber());
+                            measurement_start_frame,
+                            file_processor->GetCurrentFrameNumber());
                     }
                     else
                     {
-                        fps_info.LogToConsole();
+                        fps_info.LogMeasurements();
                     }
                 }
-                else if (file_processor.GetErrorState() != gfxrecon::decode::FileProcessor::kErrorNone)
+                else if (file_processor->GetErrorState() != gfxrecon::decode::FileProcessor::kErrorNone)
                 {
                     GFXRECON_WRITE_CONSOLE("A failure has occurred during replay");
                 }
@@ -194,7 +251,7 @@ void android_main(struct android_app* app)
                 }
             }
         }
-        catch (std::runtime_error error)
+        catch (std::runtime_error& error)
         {
             GFXRECON_WRITE_CONSOLE("Replay failed with error message: %s", error.what());
         }
@@ -288,11 +345,13 @@ void ProcessAppCmd(struct android_app* app, int32_t cmd)
             }
             case APP_CMD_GAINED_FOCUS:
             {
+                GFXRECON_LOG_INFO("Application gained focus.")
                 application->SetPaused(false);
                 break;
             }
             case APP_CMD_LOST_FOCUS:
             {
+                GFXRECON_LOG_INFO("Application lost focus.")
                 application->SetPaused(true);
                 break;
             }
