@@ -25,10 +25,12 @@
 #include "replay_settings.h"
 #include "util/json_util.h"
 #include "util/logging.h"
+#include "decode/vulkan_pre_process_consumer.h"
 
 #include <cstdint>
 #include <string>
 #include <vector>
+#include <algorithm>
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(parse_dump_resources)
@@ -78,53 +80,48 @@ static bool CheckIndicesForErrors(const gfxrecon::decode::VulkanReplayOptions& v
     // Indices should be provided sorted
     if (vulkan_replay_options.Draw_Indices.size())
     {
-        bool is_2d = false;
+        bool is_complete = false;
         for (const auto& indices : vulkan_replay_options.Draw_Indices)
         {
-            is_2d |= (indices.size() > 0);
+            is_complete |= (indices.size() > 0);
 
             if (!AreIndicesSorted(indices))
             {
-                GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
-                GFXRECON_LOG_ERROR("DrawCall indices are not sorted")
+                GFXRECON_LOG_ERROR("ERROR - incorrect --dump-resources block indices parameters");
+                GFXRECON_LOG_ERROR("Draw indices are not sorted")
                 return true;
             }
         }
 
-        if (!is_2d)
+        if (!is_complete)
         {
-            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
-            GFXRECON_LOG_ERROR("DrawCall indices must be a 2 dimensional array")
+            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources block indices parameters");
+            GFXRECON_LOG_ERROR("Draw indices should be a 2 dimensional array");
             return true;
         }
     }
 
     if (vulkan_replay_options.RenderPass_Indices.size())
     {
-        bool is_3d = false;
+        bool is_complete = false;
         for (const auto& indices0 : vulkan_replay_options.RenderPass_Indices)
         {
-            if (!indices0.size())
-            {
-                break;
-            }
-
             for (const auto& indices1 : indices0)
             {
-                is_3d |= (indices1.size() > 0);
+                is_complete |= (indices1.size() > 0);
 
                 if (!AreIndicesSorted(indices1))
                 {
-                    GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
+                    GFXRECON_LOG_ERROR("ERROR - incorrect --dump-resources block indices parameters");
                     GFXRECON_LOG_ERROR("Render pass indices are not sorted")
                     return true;
                 }
             }
         }
 
-        if (!is_3d)
+        if (!is_complete)
         {
-            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
+            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources block indices parameters");
             GFXRECON_LOG_ERROR("RenderPass indices should be a 3 dimensional array");
             return true;
         }
@@ -132,51 +129,32 @@ static bool CheckIndicesForErrors(const gfxrecon::decode::VulkanReplayOptions& v
 
     if (vulkan_replay_options.Dispatch_Indices.size())
     {
-        bool is_2d = false;
         for (const auto& indices : vulkan_replay_options.Dispatch_Indices)
         {
-            is_2d |= (indices.size() > 0);
-
             if (!AreIndicesSorted(indices))
             {
-                GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
+                GFXRECON_LOG_ERROR("ERROR - incorrect --dump-resources block indices parameters");
                 GFXRECON_LOG_ERROR("Dispatch indices are not sorted")
                 return true;
             }
         }
-
-        if (!is_2d)
-        {
-            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
-            GFXRECON_LOG_ERROR("Dispatch indices must be a 2 dimensional array")
-            return true;
-        }
     }
 
     if (vulkan_replay_options.TraceRays_Indices.size())
     {
-        bool is_2d = false;
         for (const auto& indices : vulkan_replay_options.TraceRays_Indices)
         {
-            is_2d |= (indices.size() > 0);
-
             if (!AreIndicesSorted(indices))
             {
-                GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
+                GFXRECON_LOG_ERROR("ERROR - incorrect --dump-resources parameters");
                 GFXRECON_LOG_ERROR("TraceRays indices are not sorted")
                 return true;
             }
         }
-
-        if (!is_2d)
-        {
-            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
-            GFXRECON_LOG_ERROR("TraceRays indices must be a 2 dimensional array")
-            return true;
-        }
     }
 
-    // Do some further logical assertions on the indices
+    // The number of BeginCommandBuffer and QueueSubmit entries must be more than
+    // zero, and there must be the same number of them.
     if ((vulkan_replay_options.BeginCommandBuffer_Indices.size() == 0 &&
          vulkan_replay_options.QueueSubmit_Indices.size() == 0) ||
         ((vulkan_replay_options.BeginCommandBuffer_Indices.size() ||
@@ -191,20 +169,49 @@ static bool CheckIndicesForErrors(const gfxrecon::decode::VulkanReplayOptions& v
         return true;
     }
 
-    if (vulkan_replay_options.Draw_Indices.size() && !vulkan_replay_options.RenderPass_Indices.size())
+    // Make sure each Draw has corresponding RenderPass
+    if (vulkan_replay_options.Draw_Indices.size() != vulkan_replay_options.RenderPass_Indices.size())
     {
         GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
-        GFXRECON_LOG_ERROR("DrawCall indices must always be acompanied by render pass indices");
+        GFXRECON_LOG_ERROR("Draw indices must always be accompanied by RenderPass indices");
         return true;
     }
-
-    if (vulkan_replay_options.Draw_Indices.size() == 0 && vulkan_replay_options.Dispatch_Indices.size() == 0 &&
-        vulkan_replay_options.TraceRays_Indices.size() == 0)
+    for (int i = 0; i < vulkan_replay_options.Draw_Indices.size(); i++)
     {
-        GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
-        GFXRECON_WRITE_CONSOLE(
-            "No DrawCall, Dispatch or TraceRays indices were specified. At least one of these must be provided");
-        return true;
+        if (!vulkan_replay_options.Draw_Indices[i].empty() && vulkan_replay_options.RenderPass_Indices[i].empty())
+        {
+            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
+            GFXRECON_LOG_ERROR("Each Draw item must have a corresponding RenderPass item");
+            return true;
+        }
+    }
+
+    // Each dump command must specify one and only one of draw, tracerays, or dispatch
+    size_t loopcount =
+        std::max(vulkan_replay_options.Draw_Indices.size(), vulkan_replay_options.Dispatch_Indices.size());
+    loopcount = std::max(loopcount, vulkan_replay_options.TraceRays_Indices.size());
+    for (int i = 0; i < loopcount; i++)
+    {
+        uint64_t count = 0;
+        if (vulkan_replay_options.Draw_Indices.size() >= i + 1 && vulkan_replay_options.Draw_Indices[i].size())
+        {
+            count++;
+        }
+        if (vulkan_replay_options.Dispatch_Indices.size() >= i + 1 && vulkan_replay_options.Dispatch_Indices[i].size())
+        {
+            count++;
+        }
+        if (vulkan_replay_options.TraceRays_Indices.size() >= i + 1 &&
+            vulkan_replay_options.TraceRays_Indices[i].size())
+        {
+            count++;
+        }
+        if (count != 1)
+        {
+            GFXRECON_LOG_ERROR("ERROR - incomplete --dump-resources parameters");
+            GFXRECON_WRITE_CONSOLE("One and only one of Draw, Dispatch or TraceRays indices must be specified");
+            return true;
+        }
     }
 
     return false;
@@ -222,7 +229,8 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
 #if defined(D3D12_SUPPORT)
     // Parse dump_resource arg to see if it is for d3d12
     // (i.e. it consists of 3 comma-separated integers)
-    std::vector<std::string> values = gfxrecon::util::strings::SplitString(vulkan_replay_options.dump_resources, ',');
+    std::vector<std::string> values =
+        gfxrecon::util::strings::SplitString(vulkan_replay_options.dump_resources_block_indices, ',');
     if (values.size() == 3)
     {
         dump_resource_option_is_d3d12 = true;
@@ -234,7 +242,7 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
     }
 #endif
 
-    if (vulkan_replay_options.dump_resources.empty() || dump_resource_option_is_d3d12)
+    if (vulkan_replay_options.dump_resources_block_indices.empty() || dump_resource_option_is_d3d12)
     {
         // Clear dump resources indices and return if arg is either null or intended for d3d12
         vulkan_replay_options.BeginCommandBuffer_Indices.clear();
@@ -246,70 +254,84 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
         return true;
     }
 
-    if (ends_with(to_lower(vulkan_replay_options.dump_resources), ".json"))
+    if (ends_with(to_lower(vulkan_replay_options.dump_resources_block_indices), ".json"))
     {
         // dump-resource arg value is a json file. Read and parse the json file.
         try
         {
-            std::ifstream          dr_json_file(vulkan_replay_options.dump_resources, std::ifstream::binary);
+            std::ifstream dr_json_file(vulkan_replay_options.dump_resources_block_indices, std::ifstream::binary);
+            if (!dr_json_file.is_open())
+            {
+                GFXRECON_LOG_ERROR("Could not open \"%s\" for input",
+                                   vulkan_replay_options.dump_resources_block_indices.c_str());
+                vulkan_replay_options.dumping_resources = false;
+                return false;
+            }
+
             nlohmann::ordered_json jargs;
             dr_json_file >> jargs;
 
             // Transfer jargs to vectors in vulkan_replay_options
-            for (int idx0 = 0; idx0 < jargs["BeginCommandBuffer"].size(); idx0++)
+            for (int idx0 = 0; idx0 < jargs[decode::DUMP_ARG_BEGIN_COMMAND_BUFFER].size(); idx0++)
             {
-                vulkan_replay_options.BeginCommandBuffer_Indices.push_back(jargs["BeginCommandBuffer"][idx0]);
+                vulkan_replay_options.BeginCommandBuffer_Indices.push_back(
+                    jargs[decode::DUMP_ARG_BEGIN_COMMAND_BUFFER][idx0]);
             }
 
-            for (int idx0 = 0; idx0 < jargs["Draw"].size(); idx0++)
+            for (int idx0 = 0; idx0 < jargs[decode::DUMP_ARG_DRAW].size(); idx0++)
             {
                 vulkan_replay_options.Draw_Indices.push_back(std::vector<uint64_t>());
-                for (int idx1 = 0; idx1 < jargs["Draw"][idx0].size(); idx1++)
+                for (int idx1 = 0; idx1 < jargs[decode::DUMP_ARG_DRAW][idx0].size(); idx1++)
                 {
-                    vulkan_replay_options.Draw_Indices[idx0].push_back(jargs["Draw"][idx0][idx1]);
+                    vulkan_replay_options.Draw_Indices[idx0].push_back(jargs[decode::DUMP_ARG_DRAW][idx0][idx1]);
                 }
             }
 
-            for (int idx0 = 0; idx0 < jargs["RenderPass"].size(); idx0++)
+            for (int idx0 = 0; idx0 < jargs[decode::DUMP_ARG_RENDER_PASS].size(); idx0++)
             {
                 vulkan_replay_options.RenderPass_Indices.push_back(std::vector<std::vector<uint64_t>>());
-                for (int idx1 = 0; idx1 < jargs["RenderPass"][idx0].size(); idx1++)
+                for (int idx1 = 0; idx1 < jargs[decode::DUMP_ARG_RENDER_PASS][idx0].size(); idx1++)
                 {
                     vulkan_replay_options.RenderPass_Indices[idx0].push_back(std::vector<uint64_t>());
-                    for (int idx2 = 0; idx2 < jargs["RenderPass"][idx0][idx1].size(); idx2++)
+                    for (int idx2 = 0; idx2 < jargs[decode::DUMP_ARG_RENDER_PASS][idx0][idx1].size(); idx2++)
                     {
                         vulkan_replay_options.RenderPass_Indices[idx0][idx1].push_back(
-                            jargs["RenderPass"][idx0][idx1][idx2]);
+                            jargs[decode::DUMP_ARG_RENDER_PASS][idx0][idx1][idx2]);
                     }
                 }
             }
 
-            for (int idx0 = 0; idx0 < jargs["TraceRays"].size(); idx0++)
+            for (int idx0 = 0; idx0 < jargs[decode::DUMP_ARG_TRACE_RAYS].size(); idx0++)
             {
                 vulkan_replay_options.TraceRays_Indices.push_back(std::vector<uint64_t>());
-                for (int idx1 = 0; idx1 < jargs["TraceRays"][idx0].size(); idx1++)
+                for (int idx1 = 0; idx1 < jargs[decode::DUMP_ARG_TRACE_RAYS][idx0].size(); idx1++)
                 {
-                    vulkan_replay_options.TraceRays_Indices[idx0].push_back(jargs["TraceRays"][idx0][idx1]);
+                    vulkan_replay_options.TraceRays_Indices[idx0].push_back(
+                        jargs[decode::DUMP_ARG_TRACE_RAYS][idx0][idx1]);
                 }
             }
 
-            for (int idx0 = 0; idx0 < jargs["Dispatch"].size(); idx0++)
+            for (int idx0 = 0; idx0 < jargs[decode::DUMP_ARG_DISPATCH].size(); idx0++)
             {
                 vulkan_replay_options.Dispatch_Indices.push_back(std::vector<uint64_t>());
-                for (int idx1 = 0; idx1 < jargs["Dispatch"][idx0].size(); idx1++)
+                for (int idx1 = 0; idx1 < jargs[decode::DUMP_ARG_DISPATCH][idx0].size(); idx1++)
                 {
-                    vulkan_replay_options.Dispatch_Indices[idx0].push_back(jargs["Dispatch"][idx0][idx1]);
+                    vulkan_replay_options.Dispatch_Indices[idx0].push_back(
+                        jargs[decode::DUMP_ARG_DISPATCH][idx0][idx1]);
                 }
             }
 
-            for (int idx0 = 0; idx0 < jargs["QueueSubmit"].size(); idx0++)
+            for (int idx0 = 0; idx0 < jargs[decode::DUMP_ARG_QUEUE_SUBMIT].size(); idx0++)
             {
-                vulkan_replay_options.QueueSubmit_Indices.insert(static_cast<uint64_t>(jargs["QueueSubmit"][idx0]));
+                uint64_t qs = static_cast<uint64_t>(jargs[decode::DUMP_ARG_QUEUE_SUBMIT][idx0]);
+                vulkan_replay_options.QueueSubmit_Indices.push_back(
+                    static_cast<uint64_t>(jargs[decode::DUMP_ARG_QUEUE_SUBMIT][idx0]));
             }
         }
         catch (...)
         {
-            parse_error_message = "Error reading file " + vulkan_replay_options.dump_resources + ". Bad json format?";
+            parse_error_message =
+                "Error reading file " + vulkan_replay_options.dump_resources_block_indices + ". Bad json format?";
             parse_error         = true;
         }
     }
@@ -317,7 +339,7 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
     {
         // Check to see if dump-resource arg value is a file. If it is, read the dump args from the file.
         // Allow either spaces or commas to separate fields in the file.
-        std::ifstream            infile(vulkan_replay_options.dump_resources);
+        std::ifstream            infile(vulkan_replay_options.dump_resources_block_indices);
         std::vector<std::string> drargs;
         if (!infile.fail())
         {
@@ -356,10 +378,14 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
         else
         {
             // dump-resource args are all specified on the command line
-            drargs.push_back(vulkan_replay_options.dump_resources);
+            drargs.push_back(vulkan_replay_options.dump_resources_block_indices);
         }
 
-        // Process non-json dump_resources args
+        // Process non-json dump_resources args.
+        // Go through the non-json args one line at at time.
+        // Each line should add indices for BeginCommandBuffer, Draw, RenderPass, Dispatch,
+        // TraceRays, and QueueSubmit. If on a particular line one of those is not specified,
+        // an empty entry is added. We validate these indicies later.
         for (int i = 0; i < drargs.size() && !parse_error; i++)
         {
             uint64_t              num;
@@ -387,29 +413,34 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
                 // Extract number after '='
                 num = strtol(drargs[i].c_str() + epos + 1, &endptr, 10);
                 parse_error |= ((errno != 0) || (*endptr != ',' && *endptr != 0));
-                parse_error_message =
-                    "Parameter value for " + drargs[i].substr(apos, epos - apos) + "is not a valid number";
+                if (parse_error)
+                {
+                    parse_error_message =
+                        "Parameter value for " + drargs[i].substr(apos, epos - apos) + " is not a valid number";
+                    break;
+                }
 
-                if (drargs[i].compare(apos, epos - apos, "BeginCommandBuffer") == 0)
+                if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_BEGIN_COMMAND_BUFFER) == 0)
                     BeginCommandBuffer = num;
-                else if (drargs[i].compare(apos, epos - apos, "Draw") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_DRAW) == 0)
                     Draw = num;
-                else if (drargs[i].compare(apos, epos - apos, "BeginRenderPass") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_BEGIN_RENDER_PASS) == 0)
                     BeginRenderPass = num;
-                else if (drargs[i].compare(apos, epos - apos, "NextSubPass") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_NEXT_SUB_PASS) == 0)
                     NextSubPass.push_back(num);
-                else if (drargs[i].compare(apos, epos - apos, "EndRenderPass") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_END_RENDER_PASS) == 0)
                     EndRenderPass = num;
-                else if (drargs[i].compare(apos, epos - apos, "Dispatch") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_DISPATCH) == 0)
                     Dispatch = num;
-                else if (drargs[i].compare(apos, epos - apos, "TraceRays") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_TRACE_RAYS) == 0)
                     TraceRays = num;
-                else if (drargs[i].compare(apos, epos - apos, "QueueSubmit") == 0)
+                else if (drargs[i].compare(apos, epos - apos, decode::DUMP_ARG_QUEUE_SUBMIT) == 0)
                     QueueSubmit = num;
                 else
                 {
                     parse_error         = true;
                     parse_error_message = "Bad --dump-resources parameter: " + drargs[i].substr(apos, epos - apos);
+                    break;
                 }
 
                 apos = cpos + 1;
@@ -417,17 +448,26 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
 
             if (!parse_error)
             {
-                vulkan_replay_options.BeginCommandBuffer_Indices.push_back(BeginCommandBuffer);
+                if (!BeginCommandBuffer)
+                {
+                    parse_error         = true;
+                    parse_error_message = "Bad --dump-resources parameter: Missing BeginCommandBuffer";
+                    break;
+                }
+                else
+                {
+                    vulkan_replay_options.BeginCommandBuffer_Indices.push_back(BeginCommandBuffer);
+                }
 
+                vulkan_replay_options.Draw_Indices.push_back(std::vector<uint64_t>());
                 if (Draw)
                 {
-                    vulkan_replay_options.Draw_Indices.push_back(std::vector<uint64_t>());
                     vulkan_replay_options.Draw_Indices[i].push_back(Draw);
                 }
 
+                vulkan_replay_options.RenderPass_Indices.push_back(std::vector<std::vector<uint64_t>>());
                 if (BeginRenderPass || NextSubPass.size() || EndRenderPass)
                 {
-                    vulkan_replay_options.RenderPass_Indices.push_back(std::vector<std::vector<uint64_t>>());
                     vulkan_replay_options.RenderPass_Indices[i].push_back(std::vector<uint64_t>());
                     vulkan_replay_options.RenderPass_Indices[i][0].push_back(BeginRenderPass);
 
@@ -437,21 +477,63 @@ bool parse_dump_resources_arg(gfxrecon::decode::VulkanReplayOptions& vulkan_repl
                     vulkan_replay_options.RenderPass_Indices[i][0].push_back(EndRenderPass);
                 }
 
+                vulkan_replay_options.Dispatch_Indices.push_back(std::vector<uint64_t>());
                 if (Dispatch)
                 {
-                    vulkan_replay_options.Dispatch_Indices.push_back(std::vector<uint64_t>());
                     vulkan_replay_options.Dispatch_Indices[i].push_back(Dispatch);
                 }
 
+                vulkan_replay_options.TraceRays_Indices.push_back(std::vector<uint64_t>());
                 if (TraceRays)
                 {
-                    vulkan_replay_options.TraceRays_Indices.push_back(std::vector<uint64_t>());
                     vulkan_replay_options.TraceRays_Indices[i].push_back(TraceRays);
                 }
 
-                vulkan_replay_options.QueueSubmit_Indices.insert(QueueSubmit);
+                vulkan_replay_options.QueueSubmit_Indices.push_back(QueueSubmit);
             }
         }
+
+        // Some indices are only specified for a particular type of resource dump.
+        // For each one that is not specified in the args, clear its vector.
+        bool clear_it = true;
+        for (const auto& indices : vulkan_replay_options.Draw_Indices)
+        {
+            clear_it &= (indices.size() == 0);
+        }
+        if (clear_it)
+        {
+            vulkan_replay_options.Draw_Indices.clear();
+        }
+
+        clear_it = true;
+        for (const auto& indices : vulkan_replay_options.RenderPass_Indices)
+        {
+            clear_it &= (indices.size() == 0);
+        }
+        if (clear_it)
+        {
+            vulkan_replay_options.RenderPass_Indices.clear();
+        }
+
+        clear_it = true;
+        for (const auto& indices : vulkan_replay_options.Dispatch_Indices)
+        {
+            clear_it &= (indices.size() == 0);
+        }
+        if (clear_it)
+        {
+            vulkan_replay_options.Dispatch_Indices.clear();
+        }
+
+        clear_it = true;
+        for (const auto& indices : vulkan_replay_options.TraceRays_Indices)
+        {
+            clear_it &= (indices.size() == 0);
+        }
+        if (clear_it)
+        {
+            vulkan_replay_options.TraceRays_Indices.clear();
+        }
     }
 
     if (parse_error)
